@@ -124,12 +124,6 @@ all_os[${os}_1]="almalinux:9"
 
 all_os[${os}_3]="true"
 
-os=buster
-all_os[${os}_0]="buster"
-all_os[${os}_1]="debian:buster"
-all_os[${os}_2]="percona-release_latest.buster_all.deb"
-all_os[${os}_3]="false"
-
 os=bullseye
 all_os[${os}_0]="bullseye"
 all_os[${os}_1]="debian:bullseye"
@@ -154,43 +148,42 @@ all_os[${os}_3]="true"
 list_el_os=("el7" "el8" "el9")
 list_el_os_without_el=( "7" "8" "9" )
 list_ubuntu_os=("bionic" "focal" "jammy" "noble")
-list_debian_os=("buster" "bullseye" "bookworm" "trixie")
-list_all_os=("bionic" "focal" "jammy" "noble" "el7" "el8" "el9" "buster" "bullseye" "bookworm" "trixie")
+list_debian_os=("bullseye" "bookworm" "trixie")
+list_all_os=("bionic" "focal" "jammy" "noble" "el7" "el8" "el9" "bullseye" "bookworm" "trixie")
 
-build_man_os="jammy_percona80_amd64"
+build_man_os="jammy_mysql80_amd64"
 
 list_build=(
   "bionic_percona80_amd64"   
-  "focal_percona80_amd64"   # "focal_mariadb1011_arm64"
-  "jammy_percona80_amd64"   # "jammy_mariadb1011_arm64"
+  "focal_mysql80_amd64"   # "focal_mariadb1011_arm64"
+  "jammy_mysql80_amd64"   # "jammy_mariadb1011_arm64"
   "noble_mysql84_amd64"         "noble_ubuntu_default_arm64"
   "el7_percona57_x86_64" 
-  "el8_mysql84_x86_64"        "el8_mysql84_aarch64"
-  "el9_mysql84_x86_64"        "el9_mysql84_aarch64"
-  "bullseye_percona80_amd64" 
-  "buster_percona80_amd64"
-  "bookworm_mysql84_amd64"    "bookworm_mariadb1011_arm64"
+  "el8_mysql84_x86_64"          "el8_mysql84_aarch64"
+  "el9_mysql84_x86_64"          "el9_mysql84_aarch64"
+  "bullseye_percona57_amd64"
+  "bookworm_mysql84_amd64"      "bookworm_mariadb1011_arm64"
   "trixie_debian_default_amd64" "trixie_debian_default_arm64"
 )
 
 #   "noble_percona57"    "noble_percona80"    "noble_mariadb1011"    "noble_mariadb1006"
 list_compile=(
   "bionic_percona57"   "bionic_percona80"
-  "focal_percona57"    "focal_percona80"    "focal_mariadb1011"    "focal_mariadb1006"
+  "focal_percona57"    "focal_percona80"    "focal_mariadb1011"    #"focal_mariadb1006"
 # jammy is in the tests list 
 #                                                                                          "noble_mysql84" This is already on the list of test
                                                                                                              "noble_ubuntu_default"
   "el7_percona57"      "el7_percona80"      "el7_mariadb1011"      "el7_mariadb1006"      "el7_mysql84"
   "el8_percona57"      "el8_percona80"      "el8_mariadb1011"      "el8_mariadb1006"      "el8_mysql84"
                        "el9_percona80"      "el9_mariadb1011"      "el9_mariadb1006"      "el9_mysql84"
-  "buster_percona57"   "buster_percona80"
   "bullseye_percona57" "bullseye_percona80" "bullseye_mariadb1011" "bullseye_mariadb1006"
   "bookworm_percona57" "bookworm_percona80" "bookworm_mariadb1011"                        "bookworm_mysql84"
                                                                                                              "trixie_debian_default"
 
 )
 
-list_test=("jammy_percona57" "jammy_percona80" "jammy_mariadb1011" "jammy_mariadb1006" "noble_mysql84")
+list_test=("jammy_percona57" "jammy_percona80" "jammy_mariadb1011" # "jammy_mariadb1006" 
+                                                                                        "noble_mysql84")
 
 echo "---
 version: 2.1
@@ -302,6 +295,15 @@ commands:
     - run: sudo percona-release setup -y ps80
     - run: sudo apt-get install -y gdb screen time libperconaserverclient21 libperconaserverclient21-dev percona-server-client
 
+  prepare_apt_mysql80:
+    steps:
+    - run: echo "mysql-apt-config mysql-apt-config/select-product string Ok" | sudo debconf-set-selections
+    - run: echo "mysql-apt-config mysql-apt-config/select-server string mysql-8.0-lts" | sudo debconf-set-selections
+    - run: sudo rm /usr/share/keyrings/mysql-apt-config.gpg
+    - run: echo "4" | DEBIAN_FRONTEND=noninteractive sudo dpkg-reconfigure mysql-apt-config
+    - run: sudo apt-get update
+    - run: sudo apt-get install -y gdb screen time libmysqlclient21 libmysqlclient-dev mysql-client
+
   prepare_apt_mysql84:
     steps:
     - run: echo "mysql-apt-config mysql-apt-config/select-product string Ok" | sudo debconf-set-selections
@@ -393,6 +395,7 @@ do
     echo "
   prepare_${all_os[${os}_0]}_${vendor}:
     steps:
+    - run: sudo apt-get update
     - prepare_apt_percona57
 "
     # For Percona and MySQL will be the standar apt preparation
@@ -401,6 +404,7 @@ do
 echo "
   prepare_${all_os[${os}_0]}_${all_vendors[${vendor}_0]}:
     steps:
+    - run: sudo apt-get update
     - prepare_apt_${all_vendors[${vendor}_0]}
 "
     done
@@ -411,6 +415,7 @@ echo "
 echo "
   prepare_${all_os[${os}_0]}_${all_vendors[${vendor}_0]}:
     steps:
+    - run: sudo apt-get update
     - prepare_${all_vendors[${vendor}_0]}
     - prepare_apt_${all_vendors[${vendor}_0]}
 "
@@ -457,10 +462,16 @@ cat <<EOF
     - when:
         condition: << parameters.test >>
         steps:
-        - run: bash ./test_mydumper.sh SSL
+        - run: bash ./test_mydumper.sh -d SSL
     - store_artifacts:
         path: /tmp/stream.sql
-        destination: artifact-file
+        destination: stream.sql
+    - store_artifacts:
+        path: /tmp/test_mydumper.log.tmp
+        destination: test_mydumper.log
+    - store_artifacts:
+        path: /tmp/test_myloader.log.tmp
+        destination: test_loader.log
     - store_artifacts:
         path: /tmp/data/
 
