@@ -31,17 +31,18 @@
 #include "common_options.h"
 //#include "mydumper_global.h"
 
-extern gboolean success_on_1146;
 extern gboolean help;
 guint errors=0;
 GList *ignore_errors_list=NULL;
 GAsyncQueue *stream_queue = NULL;
 gboolean use_defer= FALSE;
 gboolean check_row_count= FALSE;
-gchar **optimize_key_engines=NULL;
+extern gchar **optimize_key_engines;
 guint throttle_time=0;
 guint throttle_max_usleep_limit=60000000;
 
+gchar *set_names_in_conn_for_sct=NULL, *set_names_in_file_for_sct=NULL, *set_names_in_file_by_default=NULL;
+
 gchar *get_zstd_cmd(){
   return g_find_program_in_path("zstd");
 }
@@ -60,74 +61,71 @@ GHashTable * initialize_hash_of_session_variables(){
 }
 
 void initialize_set_names(){
-  if (set_names_str){
-    if (strlen(set_names_str)!=0){
-      set_names_statement=g_strdup_printf("/*!40101 SET NAMES %s*/",set_names_str);
-    }else
-      set_names_str=NULL;
-  } else {
-    set_names_str=g_strdup("binary");
-    set_names_statement=g_strdup("/*!40101 SET NAMES binary*/");
-  }
+  if (!set_names_in_conn_by_default)
+    set_names_in_conn_by_default=g_strdup(BINARY_CHARSET);
+
+  if (!set_names_in_conn_for_sct)
+    set_names_in_conn_for_sct=g_strdup(AUTO_CHARSET);
+
+  if (!set_names_in_file_by_default)
+    set_names_in_file_by_default=g_strdup(BINARY_CHARSET);
+
+  if (!set_names_in_file_for_sct)
+    set_names_in_file_for_sct=set_names_in_file_by_default;
+}
+
+gchar *set_names_statement_template(gchar *_set_names){
+  return g_strdup_printf("/*!40101 SET NAMES %s*/", _set_names);
 }
 
 void free_set_names(){
-  g_free(set_names_str);
+  g_free(set_names_in_conn_by_default);
+  set_names_in_conn_by_default=NULL;
   g_free(set_names_statement);
 }
 
-char *generic_checksum(MYSQL *conn, char *database, char *table, int *errn,const gchar *query_template, int column_number){
-  MYSQL_RES *result = NULL;
-  MYSQL_ROW row;
-  *errn=0;
+char *generic_checksum(MYSQL *conn, char *database, char *table, const gchar *query_template, int column_number){
   char *query = g_strdup_printf(query_template, database, table);
-  if (mysql_query(conn, query) || !(result = mysql_use_result(conn))) {
-    g_critical("Error dumping checksum (%s.%s): %s", database, table, mysql_error(conn));
-    *errn=mysql_errno(conn);
-    g_free(query);
-    return NULL;
-  }
-
-  /* There should never be more than one row */
-  row = mysql_fetch_row(result);
-  char * r=NULL;
-  if (row != NULL) r=g_strdup_printf("%s",row[column_number]);
+  struct M_ROW *mr = m_store_result_single_row( conn, query, "Error dumping checksum (%s.%s)", database, table);
   g_free(query);
-  mysql_free_result(result);
+  char * r=NULL;
+  if (mr->row)
+    r=g_strdup_printf("%s",mr->row[column_number]);
+  m_store_result_row_free(mr);
   return r;
 }
 
-char * checksum_table(MYSQL *conn, char *database, char *table, int *errn){
-  return generic_checksum(conn, database, table, errn, "CHECKSUM TABLE `%s`.`%s`", 1);
+char * checksum_table(MYSQL *conn, char *database, char *table){
+  return generic_checksum(conn, database, table, "CHECKSUM TABLE `%s`.`%s`", 1);
 }
 
 
-char * checksum_table_structure(MYSQL *conn, char *database, char *table, int *errn){
-  return generic_checksum(conn, database, table, errn,"SELECT COALESCE(LOWER(CONV(BIT_XOR(CAST(CRC32(CONCAT_WS(column_name, ordinal_position, data_type)) AS UNSIGNED)), 10, 16)), 0) AS crc FROM information_schema.columns WHERE table_schema='%s' AND table_name='%s';", 0);
+char * checksum_table_structure(MYSQL *conn, char *database, char *table){
+  return generic_checksum(conn, database, table, "SELECT COALESCE(LOWER(CONV(BIT_XOR(CAST(CRC32(CONCAT_WS(column_name, ordinal_position, data_type)) AS UNSIGNED)), 10, 16)), 0) AS crc FROM information_schema.columns WHERE table_schema='%s' AND table_name='%s';", 0);
 }
 
-char * checksum_process_structure(MYSQL *conn, char *database, char *table, int *errn){
-  return generic_checksum(conn, database, table, errn,"SELECT COALESCE(LOWER(CONV(BIT_XOR(CAST(CRC32(replace(ROUTINE_DEFINITION,' ','')) AS UNSIGNED)), 10, 16)), 0) AS crc FROM information_schema.routines WHERE ROUTINE_SCHEMA='%s' order by ROUTINE_TYPE,ROUTINE_NAME", 0);
+char * checksum_process_structure(MYSQL *conn, char *database, char *table){
+  return generic_checksum(conn, database, table, "SELECT COALESCE(LOWER(CONV(BIT_XOR(CAST(CRC32(replace(ROUTINE_DEFINITION,' ','')) AS UNSIGNED)), 10, 16)), 0) AS crc FROM information_schema.routines WHERE ROUTINE_SCHEMA='%s' order by ROUTINE_TYPE,ROUTINE_NAME", 0);
 }
 
-char * checksum_trigger_structure(MYSQL *conn, char *database, char *table, int *errn){
-  return generic_checksum(conn, database, table, errn,"SELECT COALESCE(LOWER(CONV(BIT_XOR(CAST(CRC32(REPLACE(REPLACE(REPLACE(REPLACE(ACTION_STATEMENT, CHAR(32), ''), CHAR(13), ''), CHAR(10), ''), CHAR(9), '')) AS UNSIGNED)), 10, 16)), 0) AS crc FROM information_schema.triggers WHERE EVENT_OBJECT_SCHEMA='%s' AND EVENT_OBJECT_TABLE='%s';",0);
+char * checksum_trigger_structure(MYSQL *conn, char *database, char *table){
+  return generic_checksum(conn, database, table, "SELECT COALESCE(LOWER(CONV(BIT_XOR(CAST(CRC32(REPLACE(REPLACE(REPLACE(REPLACE(ACTION_STATEMENT, CHAR(32), ''), CHAR(13), ''), CHAR(10), ''), CHAR(9), '')) AS UNSIGNED)), 10, 16)), 0) AS crc FROM information_schema.triggers WHERE EVENT_OBJECT_SCHEMA='%s' AND EVENT_OBJECT_TABLE='%s';",0);
 }
 
-char * checksum_trigger_structure_from_database(MYSQL *conn, char *database, char *table, int *errn){
-  return generic_checksum(conn, database, table, errn,"SELECT COALESCE(LOWER(CONV(BIT_XOR(CAST(CRC32(REPLACE(REPLACE(REPLACE(REPLACE(ACTION_STATEMENT, CHAR(32), ''), CHAR(13), ''), CHAR(10), ''), CHAR(9), '')) AS UNSIGNED)), 10, 16)), 0) AS crc FROM information_schema.triggers WHERE EVENT_OBJECT_SCHEMA='%s';",0);
+char * checksum_trigger_structure_from_database(MYSQL *conn, char *database, char *table){
+  return generic_checksum(conn, database, table, "SELECT COALESCE(LOWER(CONV(BIT_XOR(CAST(CRC32(REPLACE(REPLACE(REPLACE(REPLACE(ACTION_STATEMENT, CHAR(32), ''), CHAR(13), ''), CHAR(10), ''), CHAR(9), '')) AS UNSIGNED)), 10, 16)), 0) AS crc FROM information_schema.triggers WHERE EVENT_OBJECT_SCHEMA='%s';",0);
 }
 
-char * checksum_view_structure(MYSQL *conn, char *database, char *table, int *errn){
-  return generic_checksum(conn, database, table, errn,"SELECT COALESCE(LOWER(CONV(BIT_XOR(CAST(CRC32(REPLACE(VIEW_DEFINITION,TABLE_SCHEMA,'')) AS UNSIGNED)), 10, 16)), 0) AS crc FROM information_schema.views WHERE TABLE_SCHEMA='%s' AND TABLE_NAME='%s';",0);
+char * checksum_view_structure(MYSQL *conn, char *database, char *table){
+  return generic_checksum(conn, database, table,"SELECT COALESCE(LOWER(CONV(BIT_XOR(CAST(CRC32(REPLACE(VIEW_DEFINITION,TABLE_SCHEMA,'')) AS UNSIGNED)), 10, 16)), 0) AS crc FROM information_schema.views WHERE TABLE_SCHEMA='%s' AND TABLE_NAME='%s';",0);
 }
 
-char * checksum_database_defaults(MYSQL *conn, char *database, char *table, int *errn){
-  return generic_checksum(conn, database, table, errn,"SELECT COALESCE(LOWER(CONV(BIT_XOR(CAST(CRC32(concat(DEFAULT_CHARACTER_SET_NAME,DEFAULT_COLLATION_NAME)) AS UNSIGNED)), 10, 16)), 0) AS crc FROM information_schema.SCHEMATA WHERE SCHEMA_NAME='%s' ;",0);
+char * checksum_database_defaults(MYSQL *conn, char *database, char *table){
+  return generic_checksum(conn, database, table, "SELECT COALESCE(LOWER(CONV(BIT_XOR(CAST(CRC32(concat(DEFAULT_CHARACTER_SET_NAME,DEFAULT_COLLATION_NAME)) AS UNSIGNED)), 10, 16)), 0) AS crc FROM information_schema.SCHEMATA WHERE SCHEMA_NAME='%s' ;",0);
 }
 
-char * checksum_table_indexes(MYSQL *conn, char *database, char *table, int *errn){
-  return generic_checksum(conn, database, table, errn,"SELECT COALESCE(LOWER(CONV(BIT_XOR(CAST(CRC32(CONCAT_WS(TABLE_NAME,INDEX_NAME,SEQ_IN_INDEX,COLUMN_NAME)) AS UNSIGNED)), 10, 16)), 0) AS crc FROM information_schema.STATISTICS WHERE TABLE_SCHEMA='%s' AND TABLE_NAME='%s' ORDER BY INDEX_NAME,SEQ_IN_INDEX,COLUMN_NAME", 0);
+char * checksum_table_indexes(MYSQL *conn, char *database, char *table){
+  return generic_checksum(conn, database, table, "SELECT COALESCE(LOWER(CONV(BIT_XOR(CAST(CRC32(CONCAT_WS(TABLE_NAME,INDEX_NAME,SEQ_IN_INDEX,COLUMN_NAME)) AS UNSIGNED)), 10, 16)), 0) AS crc FROM information_schema.STATISTICS WHERE TABLE_SCHEMA='%s' AND TABLE_NAME='%s' ORDER BY INDEX_NAME,SEQ_IN_INDEX,COLUMN_NAME", 0);
 }
 
 GKeyFile * load_config_file(gchar * config_file){
@@ -1265,7 +1263,7 @@ void parse_object_to_export(struct object_to_export *object_to_export,gchar *val
 gchar *build_dbt_key(gchar *a, gchar *b){
   return g_strdup_printf("%c%s%c.%c%s%c", identifier_quote_character, a, identifier_quote_character, identifier_quote_character, b, identifier_quote_character);
 }
-
+/*
 gboolean common_arguments_callback(const gchar *option_name,const gchar *value, gpointer data, GError **error){
   *error=NULL;
   (void) data;
@@ -1315,7 +1313,7 @@ gboolean common_arguments_callback(const gchar *option_name,const gchar *value,
   }
   return FALSE;
 }
-
+*/
 void discard_mysql_output(MYSQL *conn){
   MYSQL_RES *result = NULL;
   MYSQL_ROW row = NULL;
@@ -1337,9 +1335,11 @@ static void m_log(MYSQL *conn, void log_fun_1(const char *, ...), void log_fun_2
     if (log_fun_2 && g_list_find(ignore_errors_list, GINT_TO_POINTER(mysql_errno(conn))))
       log_fun_2("%s - ERROR %d: %s",c, mysql_errno(conn), mysql_error(conn));
     else{
-      if (mysql_errno(conn))
+      if (mysql_errno(conn)){
         log_fun_1("%s - ERROR %d: %s",c, mysql_errno(conn), mysql_error(conn));
-      else
+        if (log_fun_1 != m_message)
+          errors++; 
+      }else
         log_fun_1("%s",c);
     }
     g_free(c);
@@ -1460,6 +1460,12 @@ void m_store_result_row_free(struct M_ROW* mr){
   g_free(mr);
 }
 
+void execute_set_names(MYSQL *conn, gchar *_set_names){
+  gchar *_set_names_statement=set_names_statement_template(_set_names); 
+  m_query_warning(conn, _set_names_statement, "Not able to execute SET NAMES statement", NULL);
+  g_free(_set_names_statement);
+}
+
 GThread * m_thread_new(const gchar* title, GThreadFunc func, gpointer data, const gchar* error_text){
   GThread * thread=g_thread_new(title, func, data);
   if (thread == NULL) {
