@@ -29,6 +29,7 @@
 
 extern guint64 min_integer_chunk_step_size;
 extern guint64 max_integer_chunk_step_size;
+extern guint max_time_per_select;
 
 enum sync_thread_lock_mode sync_thread_lock_mode=AUTO;
 const gchar *compress_method=NULL;
@@ -37,13 +38,12 @@ const gchar *rows_file_extension=SQL;
 guint output_format=SQL_INSERT;
 gchar *output_directory_str = NULL;
 gboolean masquerade_filename=FALSE;
-gboolean trx_tables=FALSE;
+guint trx_tables=1;
 gboolean use_single_column=FALSE;
 const gchar *table_engine_for_view_dependency=MEMORY;
 guint ftwrl_max_wait_time=60;
 guint ftwrl_timeout_retries=0;
 
-
 gboolean arguments_callback(const gchar *option_name,const gchar *value, gpointer data, GError **error){
   *error=NULL;
   if (g_strstr_len(option_name,10,"--compress") || g_strstr_len(option_name,2,"-c")){
@@ -74,6 +74,13 @@ gboolean arguments_callback(const gchar *option_name,const gchar *value, gpointe
     }
     return FALSE;
   }
+  if (!strcmp(option_name,"--trx-tables")){
+    if (value)
+      trx_tables=atoi(value);
+    else
+      trx_tables=1;
+    return TRUE;
+  }
   if (g_strstr_len(option_name,6,"--rows") || g_strstr_len(option_name,2,"-r")){
     if (value!=NULL)
       split_integer_tables=parse_rows_per_chunk(value, &min_chunk_step_size, &starting_chunk_step_size, &max_chunk_step_size, "Invalid option on --rows");
@@ -106,7 +113,7 @@ gboolean arguments_callback(const gchar *option_name,const gchar *value, gpointe
     }
   }
   if (!strcmp(option_name,"--trx-consistency-only")){
-    m_critical("--lock-all-tables is deprecated use --trx-tables instead");
+    m_critical("--trx-consistency-only is deprecated use --trx-tables instead");
   }
   if (!strcmp(option_name,"--less-locking")){
     m_critical("--less-locking is deprecated and its behaviour is the default which is useful if you don't have transaction tables. Use --trx-tables otherwise");
@@ -138,10 +145,37 @@ gboolean arguments_callback(const gchar *option_name,const gchar *value, gpointe
       sync_thread_lock_mode=NO_LOCK;
       return TRUE;
     }
+    if (!g_ascii_strcasecmp(value,"SAFE_NO_LOCK")){
+      sync_thread_lock_mode=SAFE_NO_LOCK;
+      return TRUE;
+    }
   }
   if (!strcmp(option_name,"--success-on-1146")){
-    ignore_errors_list=g_list_append(ignore_errors_list,GINT_TO_POINTER(1146));
-    success_on_1146=TRUE;
+    m_critical("--success-on-1146 is deprecated use --ignore-errors instead");
+  }
+
+  if (!g_strcmp0(option_name,"--default-character-set")){
+    gchar ** value_split=g_strsplit(value, ",", 2);
+    if (!value_split)
+      return FALSE;
+    set_names_in_conn_for_sct=g_strdup(value_split[0]);
+    if (g_strv_length(value_split)>1)
+      set_names_in_conn_by_default=g_strdup(value_split[1]);
+    else
+      set_names_in_conn_by_default=set_names_in_conn_for_sct;
+    g_strfreev(value_split);
+    return TRUE;
+  }
+  if (!g_strcmp0(option_name,"--set-names")){
+    gchar ** value_split=g_strsplit(value, ",", 2);
+    if (!value_split)
+      return FALSE;
+    set_names_in_file_for_sct=g_strdup(value_split[0]);
+    if (g_strv_length(value_split)>1)
+      set_names_in_file_by_default=g_strdup(value_split[1]);
+    else
+      set_names_in_file_by_default=set_names_in_file_for_sct;
+    g_strfreev(value_split);
     return TRUE;
   }
 
@@ -175,6 +209,8 @@ static GOptionEntry entries[] = {
       "Sets the max time that we are going to wait before kill the FLUSH TABLES related commands. Default: 60", NULL},
     {"ftwrl-timeout-retries", 0, 0, G_OPTION_ARG_INT, &ftwrl_timeout_retries,
       "Sets the amount of retries before give up acquiring FLUSH TABLES. Default: 0, never gives up.", NULL},
+    {"replica-data", 0, G_OPTION_FLAG_OPTIONAL_ARG, G_OPTION_ARG_CALLBACK, &common_arguments_callback,
+      "Includes the replica information", NULL},
     {NULL, 0, 0, G_OPTION_ARG_NONE, NULL, NULL, NULL}};
 
 static GOptionEntry extra_entries[] = {
@@ -182,8 +218,8 @@ static GOptionEntry extra_entries[] = {
       "Split data files into pieces of this size in MB. Useful for myloader multi-threading.", NULL},
     {"exit-if-broken-table-found", 0, 0, G_OPTION_ARG_NONE, &exit_if_broken_table_found,
       "Exits if a broken table has been found", NULL},
-    {"success-on-1146", 0, 0, G_OPTION_ARG_CALLBACK, &arguments_callback,
-      "Not increment error count and Warning instead of Critical in case of table doesn't exist", NULL},
+    {"success-on-1146", 0, G_OPTION_FLAG_OPTIONAL_ARG, G_OPTION_ARG_CALLBACK, &arguments_callback,
+      "This option is deprecated use --ignore-errors instead", NULL},
     {"build-empty-files", 'e', 0, G_OPTION_ARG_NONE, &build_empty_files,
       "Build dump files even if no data available from table", NULL},
     { "no-check-generated-fields", 0, 0, G_OPTION_ARG_NONE, &ignore_generated_fields,
@@ -210,7 +246,7 @@ static GOptionEntry lock_entries[] = {
     {"lock-all-tables", 0, G_OPTION_FLAG_OPTIONAL_ARG, G_OPTION_ARG_CALLBACK , &arguments_callback,
       "This option is deprecated use --sync-thread-lock-mode instead", NULL},
     {"sync-thread-lock-mode", 0, 0, G_OPTION_ARG_CALLBACK , &arguments_callback,
-      "There are 3 modes that can be use to sync: FTWRL, LOCK_ALL and GTID. "
+      "There are 4 modes that can be use to sync: SAFE_NO_LOCK, FTWRL, LOCK_ALL and GTID. "
       "If you don't need a consistent backup, use: NO_LOCK. More info https://mydumper.github.io/mydumper/docs/html/locks.html. "
       "Default: AUTO which uses the best option depending on the database vendor", NULL},
     {"use-savepoints", 0, 0, G_OPTION_ARG_NONE, &use_savepoints,
@@ -221,7 +257,7 @@ static GOptionEntry lock_entries[] = {
       "This option is deprecated and its behaviour is the default which is useful if you don't have transaction tables. Use --trx-tables otherwise", NULL},
     {"trx-consistency-only", 0, G_OPTION_FLAG_OPTIONAL_ARG, G_OPTION_ARG_CALLBACK , &arguments_callback,
       "This option is deprecated use --trx-tables instead", NULL},
-    {"trx-tables", 0, 0, G_OPTION_ARG_NONE, &trx_tables, 
+    {"trx-tables", 0, G_OPTION_FLAG_OPTIONAL_ARG, G_OPTION_ARG_CALLBACK, &arguments_callback, 
       "The backup process changes, if we know that we are exporting transactional tables only", NULL},
     {"skip-ddl-locks", 0, 0, G_OPTION_ARG_NONE, &skip_ddl_locks, 
       "Do not send DDL locks when possible", NULL},
@@ -249,13 +285,6 @@ static GOptionEntry exec_entries[] = {
       "Set the extension for the STDOUT file when --exec-per-thread is used", NULL},
     {NULL, 0, 0, G_OPTION_ARG_NONE, NULL, NULL, NULL}};
 
-static GOptionEntry pmm_entries[] = {
-    {"pmm-path", 0, 0, G_OPTION_ARG_STRING, &pmm_path,
-      "which default value will be /usr/local/percona/pmm2/collectors/textfile-collector/high-resolution", NULL },
-    {"pmm-resolution", 0, 0, G_OPTION_ARG_STRING, &pmm_resolution,
-      "which default will be high", NULL },
-    {NULL, 0, 0, G_OPTION_ARG_NONE, NULL, NULL, NULL}};
-
 static GOptionEntry daemon_entries[] = {
     {"daemon", 'D', 0, G_OPTION_ARG_NONE, &daemon_mode, 
       "Enable daemon mode", NULL},
@@ -266,6 +295,8 @@ static GOptionEntry daemon_entries[] = {
     {NULL, 0, 0, G_OPTION_ARG_NONE, NULL, NULL, NULL}};
 
 static GOptionEntry chunks_entries[] = {
+    {"max-time-per-select", 0, 0, G_OPTION_ARG_INT, &max_time_per_select,
+      "Maximum amount of seconds that a select should take. Default: 2", NULL},
     {"max-threads-per-table", 0, 0, G_OPTION_ARG_INT, &max_threads_per_table,
       "Maximum number of threads per table to use", NULL},
     {"use-single-column", 0, 0, G_OPTION_ARG_NONE, &use_single_column, 
@@ -374,8 +405,10 @@ static GOptionEntry statement_entries[] = {
       "--skip-tz-utc to disable.", NULL},
     {"skip-tz-utc", 0, 0, G_OPTION_ARG_NONE, &skip_tz,
       "Doesn't add SET TIMEZONE on the backup files", NULL},
-    {"set-names",0, 0, G_OPTION_ARG_STRING, &set_names_str,
-      "Sets the names, use it at your own risk, default binary", NULL },
+    {"set-names",0, 0, G_OPTION_ARG_CALLBACK, &arguments_callback,
+      "Accepts a list of up to 2 charsets, and adds 'SET NAMES' with the proper charset from the list, where the first item is used for the schema files and the second item is used for the data files. Use it at your own risk as it might cause inconsistencies #1974. Default: binary,binary", NULL },
+    {"default-character-set",0, 0, G_OPTION_ARG_CALLBACK, &arguments_callback,
+      "Accepts a list of up to 2 charsets, and executes 'SET NAMES' with the proper charset from the list, where the first item is used when executes SHOW CREATE TABLE and the second item is used for the rest. Use it at your own risk as it might cause inconsistencies #1974. Default: auto,binary. auto means that it is going to use the table character set.", NULL },
     {"table-engine-for-view-dependency", 0, 0, G_OPTION_ARG_STRING, &table_engine_for_view_dependency, 
       "Table engine to be used for the CREATE TABLE statement for temporary tables when using views",NULL},
     {NULL, 0, 0, G_OPTION_ARG_NONE, NULL, NULL, NULL}};
