@@ -20,7 +20,7 @@
 */
 
 #include <gio/gio.h>
-
+#include <math.h>
 #include "mydumper.h"
 #include "mydumper_start_dump.h"
 #include "mydumper_database.h"
@@ -68,6 +68,7 @@ void initialize_chunk_step_as_none(struct chunk_step_item * csi){
   csi->part=0;
   csi->chunk_type=NONE;
   csi->chunk_functions.process=&process_none_chunk;
+  csi->chunk_functions.free=NULL;
   csi->chunk_step = NULL;
 }
 
@@ -77,7 +78,7 @@ struct chunk_step_item * new_none_chunk_step(){
   return csi;
 }
 
-struct chunk_step_item * initialize_chunk_step_item (MYSQL *conn, struct db_table *dbt, guint position, GString *prefix) {
+struct chunk_step_item * initialize_chunk_step_item (MYSQL *conn, struct db_table *dbt, guint position, guint64 rows, GString *prefix) {
   struct chunk_step_item * csi=NULL;
 
   gchar *field=g_list_nth_data(dbt->primary_key, position);
@@ -107,7 +108,6 @@ struct chunk_step_item * initialize_chunk_step_item (MYSQL *conn, struct db_tabl
   }
   MYSQL_FIELD *fields = mysql_fetch_fields(mr->res);
   /* Support just bigger INTs for now, very dumb, no verify approach */
-  guint64 diff_btwn_max_min;
   guint64 unmin, unmax;
   gint64 nmin, nmax;
 //    union chunk_step *cs = NULL;
@@ -123,13 +123,9 @@ struct chunk_step_item * initialize_chunk_step_item (MYSQL *conn, struct db_tabl
       nmin  = strtoll (mr->row[0], NULL, 10);
       nmax  = strtoll (mr->row[1], NULL, 10);
 
-      if (fields[0].flags & UNSIGNED_FLAG){
-        diff_btwn_max_min=gint64_abs(unmax-unmin);
-      }else{
-        diff_btwn_max_min=gint64_abs(nmax-nmin);
-      }
-
       gboolean unsign = fields[0].flags & UNSIGNED_FLAG;
+      guint64 diff_btwn_max_min = unsign ? gint64_abs(unmax-unmin) : gint64_abs(nmax-nmin); 
+
       m_store_result_row_free(mr);
 
       // If !(diff_btwn_max_min > min_chunk_step_size), then there is no need to split the table.
@@ -146,29 +142,27 @@ struct chunk_step_item * initialize_chunk_step_item (MYSQL *conn, struct db_tabl
           type.sign.max=nmax;
         }
         guint64 _starting_chunk_step_size=0;
+        guint percentage_of_fragmentation = diff_btwn_max_min / rows;
+        trace("percentage_of_fragmentation of `%s`.`%s` %f", dbt->database->name, dbt->table, log(percentage_of_fragmentation ));
         if (dbt->starting_chunk_step_size == 0){
-          if (unsign){
-            _starting_chunk_step_size= dbt->max_chunk_step_size!=0?
-                                             (gint64_abs(type.unsign.max - type.unsign.min)/num_threads>dbt->max_chunk_step_size?
-                                               dbt->max_chunk_step_size:
-                                               gint64_abs(type.unsign.max - type.unsign.min)/num_threads):
-                                             gint64_abs(type.unsign.max - type.unsign.min)/num_threads;
-          }else{
-            _starting_chunk_step_size= dbt->max_chunk_step_size!=0?
-                                             (gint64_abs(type.sign.max - type.sign.min)/num_threads>dbt->max_chunk_step_size?
-                                               dbt->max_chunk_step_size:
-                                               gint64_abs(type.sign.max - type.sign.min)/num_threads):
-                                             gint64_abs(type.sign.max - type.sign.min)/num_threads;
 
-          }
+          _starting_chunk_step_size= dbt->max_chunk_step_size!=0 ?
+                                             (rows/num_threads>dbt->max_chunk_step_size?
+                                               dbt->max_chunk_step_size:
+ //                                              rows/num_threads):
+                                               rows/((log(percentage_of_fragmentation )+1)*num_threads)):
+ //                                               rows/num_threads;
+                                             rows/((log(percentage_of_fragmentation )+1)*num_threads);
+          if (dbt->max_chunk_step_size==0)
+//            max_chunk_step_size=rows/num_threads;
+            max_chunk_step_size=diff_btwn_max_min/((log(percentage_of_fragmentation )+1)*num_threads);
         }
         if (_starting_chunk_step_size < dbt->min_chunk_step_size)
           _starting_chunk_step_size=dbt->min_chunk_step_size;
 
         g_assert(_starting_chunk_step_size>0);
 
-        csi = new_integer_step_item( TRUE, prefix, field, unsign, type, 0, dbt->is_fixed_length, _starting_chunk_step_size, dbt->min_chunk_step_size, dbt->max_chunk_step_size, 0, FALSE, FALSE, NULL, position, dbt->multicolumn);
-//        determine_if_we_can_go_deeper(dbt,csi, rows);
+        csi = new_integer_step_item( TRUE, prefix, field, unsign, type, 0, dbt->is_fixed_length, _starting_chunk_step_size, dbt->min_chunk_step_size, dbt->max_chunk_step_size, 0, FALSE, FALSE, NULL, position, dbt->multicolumn, rows);
 
         if (csi->chunk_step->integer_step.is_step_fixed_length){
           if (csi->chunk_step->integer_step.is_unsigned){
@@ -284,7 +278,7 @@ void set_chunk_strategy_for_dbt(MYSQL *conn, struct db_table *dbt){
       csi=new_real_partition_step_item(partitions,0,0);
     }else{
       if (dbt->split_integer_tables) {
-        csi = initialize_chunk_step_item(conn, dbt, 0, NULL);
+        csi = initialize_chunk_step_item(conn, dbt, 0, rows, NULL);
       }else{
         csi = new_none_chunk_step();
       }
@@ -300,73 +294,96 @@ void set_chunk_strategy_for_dbt(MYSQL *conn, struct db_table *dbt){
 }
 
 gboolean get_next_dbt_and_chunk_step_item(struct db_table **dbt_pointer,struct chunk_step_item **csi, struct MList *dbt_list){
-  g_mutex_lock(dbt_list->mutex);
   GList *iter=dbt_list->list;
   struct db_table *dbt;
   gboolean are_there_jobs_defining=FALSE;
   struct chunk_step_item *lcs;
-//  struct chunk_step_item *(*get_next)(struct db_table *dbt);
-  while (iter){
-    dbt=iter->data;
-    g_mutex_lock(dbt->chunks_mutex);
+
+  gboolean finish=FALSE;
+  guint current_max_threads_running=0;
+  for (current_max_threads_running=0; current_max_threads_running < max_threads_per_table && !finish; current_max_threads_running++){
+//    g_message("Current current_max_threads_running: %d %d", current_max_threads_running, max_threads_per_table);
+
+    g_mutex_lock(dbt_list->mutex);
+    iter=dbt_list->list;
+  //  struct chunk_step_item *(*get_next)(struct db_table *dbt);
+    while (iter && !finish){
+      dbt=iter->data;
+      g_mutex_lock(dbt->chunks_mutex);
 //    g_message("Checking table: %s.%s", d->database->name, d->table);
-    if (dbt->status != DEFINING){
+      if (dbt->status != DEFINING){
 
-      if (dbt->status == UNDEFINED){
+        if (dbt->status == UNDEFINED){
 //        g_message("Checking table: %s.%s DEFINING NOW", d->database->name, d->table);
-        *dbt_pointer=iter->data;
-        dbt->status = DEFINING;
-        are_there_jobs_defining=TRUE;
-        g_mutex_unlock(dbt->chunks_mutex);
-        break;
-      }
+          *dbt_pointer=iter->data;
+          dbt->status = DEFINING;
+          are_there_jobs_defining=TRUE;
+          g_mutex_unlock(dbt->chunks_mutex);
+          finish=TRUE;
+          goto next;
+        }
 
-      // Set by set_chunk_strategy_for_dbt() in working_thread()
-      g_assert(dbt->status == READY);
+        // Set by set_chunk_strategy_for_dbt() in working_thread()
+        g_assert(dbt->status == READY);
 
-      // Initially chunks are set by set_chunk_strategy_for_dbt() and then by
-      // chunk_functions.get_next(d) (see below)
-      if (dbt->chunks == NULL){
-        g_mutex_unlock(dbt->chunks_mutex);
-        goto next;
-      }
+        // Initially chunks are set by set_chunk_strategy_for_dbt() and then by
+        // chunk_functions.get_next(d) (see below)
+        if (dbt->chunks == NULL){
+          g_mutex_unlock(dbt->chunks_mutex);
+          goto next;
+        }
 
-      lcs = (struct chunk_step_item *)g_list_first(dbt->chunks)->data;
-      if (lcs->chunk_type == NONE){
-        *dbt_pointer=iter->data;
-        *csi = lcs;
-        dbt_list->list=g_list_remove(dbt_list->list,dbt);
-        g_mutex_unlock(dbt->chunks_mutex);
-        break;
-      }
+        // Reading first chunk 
+        lcs = (struct chunk_step_item *)g_list_first(dbt->chunks)->data;
+        // If it is a full table scan, we assign it and exit
+        if (lcs->chunk_type == NONE){
+          *dbt_pointer=iter->data;
+          *csi = lcs;
+          dbt_list->list=g_list_remove(dbt_list->list,dbt);
+          g_mutex_unlock(dbt->chunks_mutex);
+          finish=TRUE;
+          goto next;
+        }
 
-      if (dbt->max_threads_per_table <= dbt->current_threads_running){
-        g_mutex_unlock(dbt->chunks_mutex);
-        goto next;
-      }
-      dbt->current_threads_running++;
-      lcs=lcs->chunk_functions.get_next(dbt);
+        // if we reach the max limit of threads per table, we continue with next table
+        if (dbt->max_threads_per_table <= dbt->current_threads_running){
+          g_mutex_unlock(dbt->chunks_mutex);
+          goto next;
+        }
 
-      if (lcs!=NULL){
-        *dbt_pointer=iter->data;
-        *csi = lcs;
-        g_mutex_unlock(dbt->chunks_mutex);
-        break;
+        if (dbt->current_threads_running > current_max_threads_running ){
+          g_mutex_unlock(dbt->chunks_mutex);
+          goto next;
+        }
+
+        lcs=lcs->chunk_functions.get_next(dbt);
+
+        if (lcs!=NULL){
+          dbt->current_threads_running++;
+          *dbt_pointer=iter->data;
+          *csi = lcs;
+          g_mutex_unlock(dbt->chunks_mutex);
+          finish=TRUE;
+          goto next;
+        }else{
+          // If there is no more chunks on this table, we remove it from the list, and continue with the next table
+          iter=iter->next;
+          // Assign iter previous removing dbt from list is important as we might break the list
+          dbt_list->list=g_list_remove(dbt_list->list,dbt);
+          g_mutex_unlock(dbt->chunks_mutex);
+          // we just continue as iter has been already set
+         continue;
+        }
       }else{
-        iter=iter->next;
-        // Assign iter previous removing dbt from list is important as we might break the list
-        dbt_list->list=g_list_remove(dbt_list->list,dbt);
         g_mutex_unlock(dbt->chunks_mutex);
-        continue;
+        are_there_jobs_defining=TRUE;
       }
-    }else{
-      g_mutex_unlock(dbt->chunks_mutex);
-      are_there_jobs_defining=TRUE;
-    }
 next:
-    iter=iter->next;
+      iter=iter->next;
+    }
+    g_mutex_unlock(dbt_list->mutex);
+
   }
-  g_mutex_unlock(dbt_list->mutex);
   return are_there_jobs_defining;
 }
 
