@@ -38,12 +38,13 @@
 #include "mydumper_arguments.h"
 
 gboolean compact = FALSE;
-GMutex *ref_table_mutex = NULL;
-GHashTable *ref_table=NULL;
 guint table_number=0;
 guint server_version= 0;
 GString *headers;
 
+static GMutex *ref_table_mutex = NULL;
+static GHashTable *ref_table=NULL;
+
 const char *routine_type[]= {"FUNCTION", "PROCEDURE", "PACKAGE", "PACKAGE BODY"};
 guint nroutines= 4;
 
@@ -53,10 +54,12 @@ void initialize_common(){
 }
 
 void free_common(){
-  g_mutex_free(ref_table_mutex);
-  ref_table_mutex=NULL;
+  g_mutex_lock(ref_table_mutex);
   g_hash_table_destroy(ref_table);
   ref_table=NULL;
+  g_mutex_unlock(ref_table_mutex);
+  g_mutex_free(ref_table_mutex);
+  ref_table_mutex=NULL;
 }
 
 char * determine_filename (char * table){
@@ -83,7 +86,6 @@ gchar *get_ref_table(gchar *k){
   return val;
 }
 
-
 char * escape_string(MYSQL *conn, char *str){
   char * r=g_new(char, strlen(str) * 2 + 1);
   mysql_real_escape_string(conn, r, str, strlen(str));
@@ -365,25 +367,24 @@ void determine_charset_and_coll_columns_from_show(MYSQL_RES *result, guint *char
 }
 
 
-void initialize_headers(){
-  headers=g_string_sized_new(100);
+void initialize_header_in_gstring(GString *_headers, gchar *charset){
   if (is_mysql_like()) {
-    if (set_names_statement)
-      g_string_printf(headers,"%s;\n",set_names_statement);
-    g_string_append(headers, "/*!40014 SET FOREIGN_KEY_CHECKS=0*/;\n");
+    if (charset)
+      g_string_printf(_headers,"/*!40101 SET NAMES %s*/;\n",charset);
+    g_string_append(_headers, "/*!40014 SET FOREIGN_KEY_CHECKS=0*/;\n");
     if (sql_mode && !compact)
-      g_string_append_printf(headers, "/*!40101 SET SQL_MODE=%s*/;\n", sql_mode);
+      g_string_append_printf(_headers, "/*!40101 SET SQL_MODE=%s*/;\n", sql_mode);
     if (!skip_tz) {
-      g_string_append(headers, "/*!40103 SET TIME_ZONE='+00:00' */;\n");
+      g_string_append(_headers, "/*!40103 SET TIME_ZONE='+00:00' */;\n");
     }
   } else if (get_product() == SERVER_TYPE_TIDB) {
     if (!skip_tz) {
-      g_string_printf(headers, "/*!40103 SET TIME_ZONE='+00:00' */;\n");
+      g_string_printf(_headers, "/*!40103 SET TIME_ZONE='+00:00' */;\n");
     }
   } else {
-    g_string_printf(headers, "SET FOREIGN_KEY_CHECKS=0;\n");
+    g_string_printf(_headers, "SET FOREIGN_KEY_CHECKS=0;\n");
     if (sql_mode && !compact)
-      g_string_append_printf(headers, "SET SQL_MODE=%s;\n", sql_mode);
+      g_string_append_printf(_headers, "SET SQL_MODE=%s;\n", sql_mode);
   }
 }
 
@@ -391,6 +392,11 @@ void initialize_sql_statement(GString *statement){
   g_string_printf(statement,"%s",headers->str);
 }
 
+void initialize_headers(){
+   headers=g_string_sized_new(100);
+   initialize_header_in_gstring(headers,set_names_in_file_by_default);
+}
+
 void set_tidb_snapshot(MYSQL *conn){
   gchar *query = g_strdup_printf("SET SESSION tidb_snapshot = '%s'", tidb_snapshot);
   m_query_critical(conn, query, "Failed to set tidb_snapshot (It could be related to https://github.com/pingcap/tidb/issues/8887)", NULL);
