@@ -19,7 +19,6 @@
                     David Ducos, Percona (david dot ducos at percona dot com)
 */
 #include <gio/gio.h>
-
 #include "mydumper_start_dump.h"
 #include "mydumper_common.h"
 #include "mydumper_jobs.h"
@@ -27,7 +26,8 @@
 #include "mydumper_working_thread.h"
 #include "mydumper_global.h"
 #include "mydumper_arguments.h"
-
+#include "mydumper_create_jobs.h"
+#include "mydumper_chunks.h"
 //
 // Enqueueing in initial_queue
 //
@@ -46,10 +46,10 @@ void create_job_to_dump_table(struct configuration *conf, gboolean is_view, gboo
   g_async_queue_push(conf->initial_queue, j);
 }
 
-void create_job_to_dump_metadata(struct configuration *conf, FILE *mdfile){
+void create_job_to_write_source_and_replica_status(struct configuration *conf, FILE *mdfile){
   struct job *j = g_new0(struct job, 1);
   j->job_data = (void *)mdfile;
-  j->type = JOB_WRITE_MASTER_STATUS;
+  j->type = JOB_WRITE_SOURCE_AND_REPLICA_STATUS;
   g_async_queue_push(conf->initial_queue, j);
 }
 
@@ -97,14 +97,15 @@ void create_job_to_dump_tablespaces(struct configuration *conf){
   g_async_queue_push(conf->schema_queue, j);
 }
 
-void create_database_related_job(struct database *database, struct configuration *conf, enum job_type type, const gchar *suffix) {
+static
+void create_database_related_job(struct database *database, struct configuration *conf, enum job_type type, const gchar *suffix, gboolean checksum_filename) {
   struct job *j = g_new0(struct job, 1);
   struct database_job *dj = g_new0(struct database_job, 1);
   j->job_data = (void *)dj;
   dj->database = database;
   j->type = type;
   dj->filename = build_schema_filename(database->filename, suffix);
-  dj->checksum_filename = schema_checksums;
+  dj->checksum_filename = checksum_filename;
   g_async_queue_push(conf->schema_queue, j);
   return;
 }
@@ -122,11 +123,11 @@ void create_job_to_dump_table_schema(struct db_table *dbt, struct configuration
 }
 
 void create_job_to_dump_schema(struct database *database, struct configuration *conf) {
-  create_database_related_job(database, conf, JOB_CREATE_DATABASE, "schema-create");
+  create_database_related_job(database, conf, JOB_CREATE_DATABASE, "schema-create", schema_checksums);
 }
 
 void create_job_to_dump_post(struct database *database, struct configuration *conf) {
-  create_database_related_job(database, conf, JOB_SCHEMA_POST, "schema-post");
+  create_database_related_job(database, conf, JOB_SCHEMA_POST, "schema-post", routine_checksums);
 }
 
 //
@@ -220,13 +221,13 @@ struct table_job * new_table_job(struct db_table *dbt, char *partition, guint64
   tj->part=part;
   tj->sub_part = 0;
   tj->rows=g_new0(struct table_job_file, 1);
-  tj->rows->file = 0;
+  tj->rows->file = -1;
   tj->rows->filename = NULL;
   if (output_format==SQL_INSERT)
 		tj->sql=NULL;
 	else{
 		tj->sql=g_new0(struct table_job_file, 1);
-    tj->sql->file = 0;
+    tj->sql->file = -1;
     tj->sql->filename = NULL;
   }
   tj->exec_out_filename = NULL;
@@ -242,20 +243,28 @@ struct table_job * new_table_job(struct db_table *dbt, char *partition, guint64
 }
 
 void free_table_job(struct table_job *tj){
-  if (tj->sql){
-    m_close(tj->td->thread_id, tj->sql->file, tj->sql->filename, tj->filesize, tj->dbt);
-    tj->sql->file=0;
+  if (tj->sql && tj->sql->file >= 0){
+    if (tj->sql->file >= 0)
+      m_close(tj->td->thread_id, tj->sql->file, tj->sql->filename, tj->filesize, tj->dbt);
+    tj->sql->file=-1;
     tj->sql=NULL;
   }
   if (tj->rows){
-    m_close(tj->td->thread_id, tj->rows->file, tj->rows->filename, tj->filesize, tj->dbt);
-    tj->rows->file=0;
+    if (tj->rows->file >= 0)
+      m_close(tj->td->thread_id, tj->rows->file, tj->rows->filename, tj->filesize, tj->dbt);
+    tj->rows->file=-1;
     tj->rows=NULL;
   }
 
   if (tj->where!=NULL)
     g_string_free(tj->where,TRUE);
 
+//  if (tj->chunk_step_item){
+//    if (tj->chunk_step_item->chunk_functions.free)
+//      tj->chunk_step_item->chunk_functions.free(tj->chunk_step_item);
+//    g_free(tj->chunk_step_item);
+//    tj->chunk_step_item=NULL;
+//  }
   g_free(tj);
 }
 
