@@ -42,6 +42,8 @@
 guint64 min_integer_chunk_step_size=0;
 guint64 max_integer_chunk_step_size=0;
 
+guint max_time_per_select=MAX_TIME_PER_QUERY;
+
 guint64 gint64_abs(gint64 a){
   if (a >= 0)
     return a;
@@ -49,7 +51,10 @@ guint64 gint64_abs(gint64 a){
 }
 
 static
-void initialize_integer_step(union chunk_step *cs, gboolean is_unsigned, union type type, gboolean is_step_fixed_length, guint64 step, guint64 min_css, guint64 max_css, gboolean check_min, gboolean check_max){
+GString * get_where_from_csi(struct chunk_step_item * csi);
+
+static
+void initialize_integer_step(union chunk_step *cs, gboolean is_unsigned, union type type, gboolean is_step_fixed_length, guint64 step, guint64 min_css, guint64 max_css, gboolean check_min, gboolean check_max, guint64 rows_in_explain){
   cs->integer_step.is_unsigned = is_unsigned;
   cs->integer_step.min_chunk_step_size = min_css;
   cs->integer_step.max_chunk_step_size = max_css;
@@ -69,23 +74,27 @@ void initialize_integer_step(union chunk_step *cs, gboolean is_unsigned, union t
   cs->integer_step.is_step_fixed_length = is_step_fixed_length;
   cs->integer_step.check_max=check_max;
   cs->integer_step.check_min=check_min;
+  cs->integer_step.rows_in_explain=rows_in_explain;
 }
 
 static
-union chunk_step *new_integer_step(gboolean is_unsigned, union type type, gboolean is_step_fixed_length, guint64 step, guint64 min_css, guint64 max_css, gboolean check_min, gboolean check_max){
+union chunk_step *new_integer_step(gboolean is_unsigned, union type type, gboolean is_step_fixed_length, guint64 step, guint64 min_css, guint64 max_css, gboolean check_min, gboolean check_max, guint64 rows_in_explain){
   union chunk_step * cs = g_new0(union chunk_step, 1);
-  initialize_integer_step(cs,is_unsigned, type, is_step_fixed_length, step, min_css, max_css, check_min, check_max);
+  initialize_integer_step(cs,is_unsigned, type, is_step_fixed_length, step, min_css, max_css, check_min, check_max, rows_in_explain);
   return cs;
 }
 
+void free_integer_step_item(struct chunk_step_item * csi);
+
 static
-void initialize_integer_step_item(struct chunk_step_item *csi, gboolean include_null, GString *prefix, gchar *field, gboolean is_unsigned, union type type, guint deep, gboolean is_step_fixed_length, guint64 step, guint64 min_css, guint64 max_css, guint64 part, gboolean check_min, gboolean check_max, struct chunk_step_item * next, guint position, gboolean multicolumn){
-  csi->chunk_step = new_integer_step(is_unsigned, type, is_step_fixed_length, step, min_css, max_css, check_min, check_max);
+void initialize_integer_step_item(struct chunk_step_item *csi, gboolean include_null, GString *prefix, gchar *field, gboolean is_unsigned, union type type, guint deep, gboolean is_step_fixed_length, guint64 step, guint64 min_css, guint64 max_css, guint64 part, gboolean check_min, gboolean check_max, struct chunk_step_item * next, guint position, gboolean multicolumn, guint64 rows_in_explain){
+  csi->chunk_step = new_integer_step(is_unsigned, type, is_step_fixed_length, step, min_css, max_css, check_min, check_max, rows_in_explain);
   csi->chunk_type=INTEGER;
   csi->position=position;
   csi->next=next;
   csi->status = UNASSIGNED;
   csi->chunk_functions.process = &process_integer_chunk;
+  csi->chunk_functions.free=&free_integer_step_item;
 //  csi->chunk_functions.update_where = &get_integer_chunk_where;
   csi->chunk_functions.get_next = &get_next_integer_chunk;
   csi->where=g_string_new("");
@@ -99,9 +108,9 @@ void initialize_integer_step_item(struct chunk_step_item *csi, gboolean include_
   csi->multicolumn=multicolumn;
 }
 
-struct chunk_step_item *new_integer_step_item(gboolean include_null, GString *prefix, gchar *field, gboolean is_unsigned, union type type, guint deep, gboolean is_step_fixed_length, guint64 step, guint64 min_css, guint64 max_css, guint64 part, gboolean check_min, gboolean check_max, struct chunk_step_item * next, guint position, gboolean multicolumn){
+struct chunk_step_item *new_integer_step_item(gboolean include_null, GString *prefix, gchar *field, gboolean is_unsigned, union type type, guint deep, gboolean is_step_fixed_length, guint64 step, guint64 min_css, guint64 max_css, guint64 part, gboolean check_min, gboolean check_max, struct chunk_step_item * next, guint position, gboolean multicolumn, guint64 rows_in_explain){
   struct chunk_step_item *csi = g_new0(struct chunk_step_item,1);
-  initialize_integer_step_item(csi, include_null, prefix, field, is_unsigned, type, deep, is_step_fixed_length, step, min_css, max_css, part, check_min, check_max, next, position, multicolumn);
+  initialize_integer_step_item(csi, include_null, prefix, field, is_unsigned, type, deep, is_step_fixed_length, step, min_css, max_css, part, check_min, check_max, next, position, multicolumn, rows_in_explain);
   return csi;
 }
 
@@ -117,6 +126,19 @@ void free_integer_step_item(struct chunk_step_item * csi){
     free_integer_step(csi->chunk_step);
     csi->chunk_step=NULL;
   }
+  if (csi->where){
+    g_string_free(csi->where, TRUE);
+    csi->where=NULL;
+  }
+  if (csi->field){
+    g_free(csi->field);
+    csi->field=NULL;
+  }
+  if (csi->mutex){
+    g_mutex_free(csi->mutex);
+    csi->mutex=NULL;
+  }
+
   // We cannot free it here, otherwise get_next_integer_chunk() fails
   // g_free(csi);
 }
@@ -130,7 +152,7 @@ void print_type(union type * type, gboolean is_unsigned){
 
 struct chunk_step_item * split_chunk_step(struct chunk_step_item * csi){
   struct chunk_step_item * new_csi = NULL;
-  guint part=csi->part;
+  guint64 part=csi->part;
   gint64 new_minmax_signed = 0;
   guint64 new_minmax_unsigned = 0;
   union type type;
@@ -191,14 +213,22 @@ struct chunk_step_item * split_chunk_step(struct chunk_step_item * csi){
     }
   }
   // print_type(&type, ics->is_unsigned);
-  new_csi = new_integer_step_item(FALSE, NULL, csi->field, csi->chunk_step->integer_step.is_unsigned, type, csi->deep + 1, csi->chunk_step->integer_step.is_step_fixed_length, csi->chunk_step->integer_step.step, csi->chunk_step->integer_step.min_chunk_step_size, csi->chunk_step->integer_step.max_chunk_step_size, part, TRUE, csi->chunk_step->integer_step.check_max, NULL, csi->position, csi->multicolumn);
+  new_csi = new_integer_step_item(FALSE, NULL, csi->field, csi->chunk_step->integer_step.is_unsigned, type, csi->deep + 1, csi->chunk_step->integer_step.is_step_fixed_length, csi->chunk_step->integer_step.step, csi->chunk_step->integer_step.min_chunk_step_size, csi->chunk_step->integer_step.max_chunk_step_size, part, TRUE, TRUE /*csi->chunk_step->integer_step.check_max*/, NULL, csi->position, csi->multicolumn, 0);
   new_csi->status=ASSIGNED;
 
   csi->chunk_step->integer_step.check_max=TRUE;
-  if (ics->is_unsigned)
+  if (ics->is_unsigned){
+//    csi->chunk_step->integer_step.rows_in_explain = csi->chunk_step->integer_step.rows_in_explain * gint64_abs( new_minmax_unsigned - 1 - csi->chunk_step->integer_step.type.unsign.min ) / gint64_abs(csi->chunk_step->integer_step.type.unsign.max - csi->chunk_step->integer_step.type.unsign.min);
+//    new_csi->chunk_step->integer_step.rows_in_explain = csi->chunk_step->integer_step.rows_in_explain * gint64_abs( new_csi->chunk_step->integer_step.type.unsign.max - new_minmax_unsigned - 1 ) / gint64_abs(new_csi->chunk_step->integer_step.type.unsign.max - new_csi->chunk_step->integer_step.type.unsign.min);
+
     csi->chunk_step->integer_step.type.unsign.max = new_minmax_unsigned - 1;
-  else
+  }
+  else{
+//    csi->chunk_step->integer_step.rows_in_explain = csi->chunk_step->integer_step.rows_in_explain * ( new_minmax_signed - 1 - csi->chunk_step->integer_step.type.sign.min ) / (csi->chunk_step->integer_step.type.sign.max - csi->chunk_step->integer_step.type.sign.min);
+//    new_csi->chunk_step->integer_step.rows_in_explain = csi->chunk_step->integer_step.rows_in_explain * gint64_abs( new_csi->chunk_step->integer_step.type.sign.max - new_minmax_signed - 1 ) / gint64_abs(new_csi->chunk_step->integer_step.type.sign.max - new_csi->chunk_step->integer_step.type.sign.min);
+
     csi->chunk_step->integer_step.type.sign.max = new_minmax_signed - 1;
+  }
   csi->deep=csi->deep+1;
 
 
@@ -217,7 +247,7 @@ return ( csi->chunk_step->integer_step.is_step_fixed_length && (
 ) );
 }
 
-
+static
 gboolean is_splitable(struct chunk_step_item *csi){
 return ( !csi->chunk_step->integer_step.is_step_fixed_length  && (( csi->chunk_step->integer_step.is_unsigned && (csi->chunk_step->integer_step.type.unsign.cursor < csi->chunk_step->integer_step.type.unsign.max
         && (
@@ -245,10 +275,40 @@ return ( !csi->chunk_step->integer_step.is_step_fixed_length  && (( csi->chunk_s
 ) );
 }
 
+static
+gboolean is_last_step(struct chunk_step_item *csi){
+return ( !csi->chunk_step->integer_step.is_step_fixed_length  && (
+      ( csi->chunk_step->integer_step.is_unsigned && (csi->chunk_step->integer_step.type.unsign.cursor < csi->chunk_step->integer_step.type.unsign.max
+        && (
+             ( csi->status == DUMPING_CHUNK && (csi->chunk_step->integer_step.type.unsign.max - csi->chunk_step->integer_step.type.unsign.cursor ) <= csi->chunk_step->integer_step.step
+             )
+           )
+    )) || 
+      ( ! csi->chunk_step->integer_step.is_unsigned && (csi->chunk_step->integer_step.type.sign.cursor < csi->chunk_step->integer_step.type.sign.max
+        && (
+             ( csi->status == DUMPING_CHUNK && gint64_abs(csi->chunk_step->integer_step.type.sign.max - csi->chunk_step->integer_step.type.sign.cursor) <= csi->chunk_step->integer_step.step
+             )
+           )
+         )
+)) ); 
+  /*
+  || ( csi->chunk_step->integer_step.is_step_fixed_length && csi->chunk_step->integer_step.step > 0 && (
+(
+  csi->chunk_step->integer_step.is_unsigned && csi->chunk_step->integer_step.type.unsign.max / csi->chunk_step->integer_step.step > csi->chunk_step->integer_step.type.unsign.min / csi->chunk_step->integer_step.step + 1
+)||
+(
+ !csi->chunk_step->integer_step.is_unsigned && csi->chunk_step->integer_step.type.sign.max   / csi->chunk_step->integer_step.step > csi->chunk_step->integer_step.type.sign.min   / csi->chunk_step->integer_step.step + 1
+)
+
+) );*/
+}
+
+
+
 void update_where_on_integer_step(struct chunk_step_item * csi);
 
 struct chunk_step_item *clone_chunk_step_item(struct chunk_step_item *csi){
-  return new_integer_step_item(csi->include_null, csi->prefix, csi->field, csi->chunk_step->integer_step.is_unsigned, csi->chunk_step->integer_step.type, csi->deep, csi->chunk_step->integer_step.is_step_fixed_length, csi->chunk_step->integer_step.step, csi->chunk_step->integer_step.min_chunk_step_size, csi->chunk_step->integer_step.max_chunk_step_size, csi->part, csi->chunk_step->integer_step.check_min, csi->chunk_step->integer_step.check_max, NULL, csi->position, csi->multicolumn);
+  return new_integer_step_item(csi->include_null, csi->prefix, csi->field, csi->chunk_step->integer_step.is_unsigned, csi->chunk_step->integer_step.type, csi->deep, csi->chunk_step->integer_step.is_step_fixed_length, csi->chunk_step->integer_step.step, csi->chunk_step->integer_step.min_chunk_step_size, csi->chunk_step->integer_step.max_chunk_step_size, csi->part, csi->chunk_step->integer_step.check_min, csi->chunk_step->integer_step.check_max, NULL, csi->position, csi->multicolumn, 0);
 }
 
 
@@ -273,6 +333,33 @@ struct chunk_step_item *get_next_integer_chunk(struct db_table *dbt){
       if (csi->status==COMPLETED){
         goto end;
       }
+      if (is_last_step(csi)){
+        trace("Last chunk on step in `%s`.`%s` assigned", dbt->database->name, dbt->table);
+        csi->status=UNSPLITTABLE;
+        csi->deep=csi->deep+1;
+        new_csi=clone_chunk_step_item(csi);
+        new_csi->status=ASSIGNED;
+
+        if (csi->chunk_step->integer_step.is_unsigned){
+          csi->chunk_step->integer_step.type.unsign.max=csi->chunk_step->integer_step.type.unsign.cursor;
+          new_csi->chunk_step->integer_step.type.unsign.min=csi->chunk_step->integer_step.type.unsign.cursor+1;
+        }else{
+          csi->chunk_step->integer_step.type.sign.max=csi->chunk_step->integer_step.type.sign.cursor;
+          new_csi->chunk_step->integer_step.type.sign.min=csi->chunk_step->integer_step.type.sign.cursor+1;
+        }
+
+        new_csi->part+=pow(2,csi->deep);
+        update_where_on_integer_step(new_csi);
+
+        dbt->chunks=g_list_append(dbt->chunks,new_csi);
+        // should I push them again? isn't it pointless?
+//        g_async_queue_push(dbt->chunks_queue, csi);
+//        g_async_queue_push(dbt->chunks_queue, new_csi);
+        //
+        g_mutex_unlock(csi->mutex);
+        return new_csi;
+      
+      }
       if (!is_splitable(csi)){
         if (csi->multicolumn && csi->next && csi->next->chunk_type==INTEGER){
           trace("Multicolumn table checking next");
@@ -387,13 +474,12 @@ gboolean refresh_integer_min_max(MYSQL *conn, struct db_table *dbt, struct chunk
   return TRUE;
 }
 
-
-void update_integer_min(MYSQL *conn, struct db_table *dbt, struct chunk_step_item *csi ){
+static
+gboolean update_integer_min(MYSQL *conn, struct db_table *dbt, struct chunk_step_item *csi ){
   struct integer_step * ics = &(csi->chunk_step->integer_step);
   gchar *query = NULL;
 
-  GString *where = g_string_new("");
-  update_integer_where_on_gstring(where, FALSE, csi->prefix, csi->field, csi->chunk_step->integer_step.is_unsigned, csi->chunk_step->integer_step.type, FALSE);
+  GString *where = get_where_from_csi(csi);
 
   MYSQL_RES *min = m_store_result(conn, query = g_strdup_printf(
                         "SELECT %s %s%s%s FROM %s%s%s.%s%s%s WHERE %s ORDER BY %s%s%s ASC LIMIT 1",
@@ -405,14 +491,14 @@ void update_integer_min(MYSQL *conn, struct db_table *dbt, struct chunk_step_ite
   g_string_free(where,TRUE);
   g_free(query);
 
-  if (!min){
-    return;
-  }
+  if (!min)
+    return FALSE;
+
   MYSQL_ROW row = mysql_fetch_row(min);
 
   if (row==NULL || row[0]==NULL){
     mysql_free_result(min);
-    return;
+    return FALSE;
   }
 
   if (ics->is_unsigned) {
@@ -424,14 +510,15 @@ void update_integer_min(MYSQL *conn, struct db_table *dbt, struct chunk_step_ite
   }
 
   mysql_free_result(min);
+  return TRUE;
 }
 
-void update_integer_max(MYSQL *conn,struct db_table *dbt, struct chunk_step_item *csi ){
+static
+gboolean update_integer_max(MYSQL *conn,struct db_table *dbt, struct chunk_step_item *csi ){
   struct integer_step * ics = &(csi->chunk_step->integer_step);
   gchar *query = NULL;
 
-  GString *where = g_string_new("");
-  update_integer_where_on_gstring(where, FALSE, csi->prefix, csi->field, csi->chunk_step->integer_step.is_unsigned, csi->chunk_step->integer_step.type, FALSE);
+  GString *where = get_where_from_csi(csi);
 
   MYSQL_RES *max = m_store_result(conn, query = g_strdup_printf(
                         "SELECT %s %s%s%s FROM %s%s%s.%s%s%s WHERE %s ORDER BY %s%s%s DESC LIMIT 1",
@@ -443,9 +530,8 @@ void update_integer_max(MYSQL *conn,struct db_table *dbt, struct chunk_step_item
   g_string_free(where,TRUE);
   g_free(query);
 
-  if (!max){
+  if (!max)
     goto cleanup;
-  }
 
   MYSQL_ROW row = mysql_fetch_row(max);
   if (row==NULL || row[0]==NULL){
@@ -458,7 +544,7 @@ cleanup:
 
     if (max)
       mysql_free_result(max);
-    return;
+    return FALSE;
   }
 
   if (ics->is_unsigned) {
@@ -470,6 +556,18 @@ cleanup:
   }
 
   mysql_free_result(max);
+  return TRUE;
+}
+
+static
+gboolean is_last(struct chunk_step_item *csi){
+  g_mutex_lock(csi->mutex);
+  gboolean r = 
+    csi->chunk_step->integer_step.is_unsigned ?
+    csi->chunk_step->integer_step.type.unsign.cursor == csi->chunk_step->integer_step.type.unsign.max :
+    csi->chunk_step->integer_step.type.sign.cursor   == csi->chunk_step->integer_step.type.sign.max;
+  g_mutex_unlock(csi->mutex);
+  return r;
 }
 
 guint process_integer_chunk_step(struct table_job *tj, struct chunk_step_item *csi){
@@ -488,31 +586,52 @@ guint process_integer_chunk_step(struct table_job *tj, struct chunk_step_item *c
 //    m_critical("Thread %d: Trying to process COMPLETED chunk",td->thread_id);
   csi->status = DUMPING_CHUNK;
 
-  if (cs->integer_step.check_max && tj->dbt->max_chunk_step_size!=0 && !cs->integer_step.is_step_fixed_length){
-    trace("Thread %d: Updating MAX", td->thread_id);
-    if (cs->integer_step.is_unsigned)
-      trace("Thread %d: Updating MAX: %ld", td->thread_id, cs->integer_step.type.unsign.max);
+  gboolean c_min=TRUE, c_max=TRUE;
+
+  if (!cs->integer_step.is_step_fixed_length){  
+
+    if (cs->integer_step.check_max /*&& tj->dbt->max_chunk_step_size!=0*/ && !cs->integer_step.is_step_fixed_length){
+      trace("Thread %d: I-Chunk 1: Updating MAX", td->thread_id);
+      if (cs->integer_step.is_unsigned)
+        trace("Thread %d: I-Chunk 1: Updating MAX: %ld", td->thread_id, cs->integer_step.type.unsign.max);
+      else
+        trace("Thread %d: I-Chunk 1: Updating MAX: %ld", td->thread_id, cs->integer_step.type.sign.max);
+      c_max=update_integer_max(td->thrconn, tj->dbt, csi);
+      if (cs->integer_step.is_unsigned)
+        trace("Thread %d: I-Chunk 1: New MAX: %ld", td->thread_id, cs->integer_step.type.unsign.max);
     else
-      trace("Thread %d: Updating MAX: %ld", td->thread_id, cs->integer_step.type.sign.max);
-    update_integer_max(td->thrconn, tj->dbt, csi);
-    if (cs->integer_step.is_unsigned)
-      trace("Thread %d: New MAX: %ld", td->thread_id, cs->integer_step.type.unsign.max);
-    else
-      trace("Thread %d: New MAX: %ld", td->thread_id, cs->integer_step.type.sign.max);
+      trace("Thread %d: I-Chunk 1: New MAX: %ld", td->thread_id, cs->integer_step.type.sign.max);
     cs->integer_step.check_max=FALSE;
   }
-  if (cs->integer_step.check_min && tj->dbt->max_chunk_step_size!=0 && !cs->integer_step.is_step_fixed_length){
+  if (cs->integer_step.check_min /*&& tj->dbt->max_chunk_step_size!=0*/ && !cs->integer_step.is_step_fixed_length){
     if (cs->integer_step.is_unsigned)
-      trace("Thread %d: Updating MIN: %ld", td->thread_id, cs->integer_step.type.unsign.min);
+      trace("Thread %d: I-Chunk 1: Updating MIN: %ld", td->thread_id, cs->integer_step.type.unsign.min);
     else
-      trace("Thread %d: Updating MIN: %ld", td->thread_id, cs->integer_step.type.sign.min);
-    update_integer_min(td->thrconn, tj->dbt, csi);
+      trace("Thread %d: I-Chunk 1: Updating MIN: %ld", td->thread_id, cs->integer_step.type.sign.min);
+    c_min=update_integer_min(td->thrconn, tj->dbt, csi);
     if (cs->integer_step.is_unsigned)
-      trace("Thread %d: New MIN: %ld", td->thread_id, cs->integer_step.type.unsign.min);
+      trace("Thread %d: I-Chunk 1: New MIN: %ld", td->thread_id, cs->integer_step.type.unsign.min);
     else
-      trace("Thread %d: New MIN: %ld", td->thread_id, cs->integer_step.type.sign.min);
+      trace("Thread %d: I-Chunk 1: New MIN: %ld", td->thread_id, cs->integer_step.type.sign.min);
     cs->integer_step.check_min=FALSE;
   }
+  if (!c_min && !c_max){
+    trace("Thread %d: I-Chunk 1: both min and max doesn't exists", td->thread_id);
+    close_files(tj);
+    g_mutex_unlock(csi->mutex);
+    goto update_min;
+//    goto end_process; 
+  }
+
+
+  if ( cs->integer_step.rows_in_explain == 0){
+    GString *_where=get_where_from_csi(csi);
+    cs->integer_step.rows_in_explain=get_rows_from_explain(td->thrconn, tj->dbt, _where, csi->field); 
+    trace("Thread %d: I-Chunk 1: We calculated rows %"G_GUINT64_FORMAT" for %s", td->thread_id, cs->integer_step.rows_in_explain, _where->str);
+    g_string_free(_where, TRUE);
+  }
+  }
+
 
 // Stage 2: Setting cursor
   if (tj->dbt->multicolumn && csi->multicolumn && csi->next == NULL && !cs->integer_step.is_step_fixed_length){
@@ -524,19 +643,19 @@ retry:
         cs->integer_step.type.unsign.cursor = cs->integer_step.type.unsign.max;
       else
         cs->integer_step.type.unsign.cursor = cs->integer_step.type.unsign.min + integer_step_step - 1;
-      trace("Thread %d: cs->integer_step.type.unsign.cursor: %lld", td->thread_id, cs->integer_step.type.unsign.cursor);
+      trace("Thread %d: I-Chunk 2: cs->integer_step.type.unsign.cursor: %lld", td->thread_id, cs->integer_step.type.unsign.cursor);
     }else{
       if (integer_step_step > gint64_abs(cs->integer_step.type.sign.max - cs->integer_step.type.sign.min) + 1)
         cs->integer_step.type.sign.cursor = cs->integer_step.type.sign.max;
       else
         cs->integer_step.type.sign.cursor = cs->integer_step.type.sign.min + integer_step_step - 1;
-      trace("Thread %d: cs->integer_step.type.sign.cursor: %lld", td->thread_id, cs->integer_step.type.sign.cursor);
+      trace("Thread %d: I-Chunk 2: cs->integer_step.type.sign.cursor: %lld", td->thread_id, cs->integer_step.type.sign.cursor);
     }
     update_where_on_integer_step(csi);
     guint64 rows = check_row_count?
                    get_rows_from_count  (td->thrconn, tj->dbt, csi->where):
                    get_rows_from_explain(td->thrconn, tj->dbt, csi->where, csi->field);
-    trace("Thread %d: multicolumn and next == NULL with rows: %lld", td->thread_id, rows);
+    trace("Thread %d: I-Chunk 2: multicolumn and next == NULL with rows: %lld", td->thread_id, rows);
 
     guint64 tmpstep = csi->chunk_step->integer_step.is_unsigned?
                  csi->chunk_step->integer_step.type.unsign.cursor - csi->chunk_step->integer_step.type.unsign.min:
@@ -547,28 +666,28 @@ retry:
     if (integer_step_step>1){
       // rows / num_threads > integer_step_step
       if (rows > tj->dbt->min_chunk_step_size && ( rows > cs->integer_step.step || (tj->num_rows_of_last_run>0 &&rows/100 > tj->num_rows_of_last_run))){
-        trace("Thread %d: integer_step.step>1 then retrying", td->thread_id);
+        trace("Thread %d: I-Chunk 2: integer_step.step>1 then retrying", td->thread_id);
         integer_step_step=integer_step_step/2;
         goto retry;
       }
-      trace("Thread %d: integer_step.step>1 not retrying as rows %lld <=  step %lld and integer_step_step: %lld", td->thread_id, rows, cs->integer_step.step, integer_step_step);
+      trace("Thread %d: I-Chunk 2: integer_step.step>1 not retrying as rows %lld <=  step %lld and integer_step_step: %lld", td->thread_id, rows, cs->integer_step.step, integer_step_step);
       cs->integer_step.step=integer_step_step;
     }else{
       // at this poing cs->integer_step.step == 1 always
       cs->integer_step.step=1;
       if (cs->integer_step.is_unsigned){
-        trace("Thread %d: integer_step.step==1 min: %"G_GUINT64_FORMAT" | max: %"G_GUINT64_FORMAT, td->thread_id, csi->chunk_step->integer_step.type.unsign.min, csi->chunk_step->integer_step.type.unsign.max);
+        trace("Thread %d: I-Chunk 2: integer_step.step==1 min: %"G_GUINT64_FORMAT" | max: %"G_GUINT64_FORMAT, td->thread_id, csi->chunk_step->integer_step.type.unsign.min, csi->chunk_step->integer_step.type.unsign.max);
       }else{
-        trace("Thread %d: integer_step.step==1 min: %"G_GINT64_FORMAT" | max: %"G_GINT64_FORMAT, td->thread_id, csi->chunk_step->integer_step.type.sign.min, csi->chunk_step->integer_step.type.sign.max);
+        trace("Thread %d: I-Chunk 2: integer_step.step==1 min: %"G_GINT64_FORMAT" | max: %"G_GINT64_FORMAT, td->thread_id, csi->chunk_step->integer_step.type.sign.min, csi->chunk_step->integer_step.type.sign.max);
       }
       if (rows > tj->dbt->min_chunk_step_size){
-        csi->next = initialize_chunk_step_item(td->thrconn, tj->dbt, csi->position + 1, csi->where);
+        csi->next = initialize_chunk_step_item(td->thrconn, tj->dbt, csi->position + 1, rows, csi->where);
         if (csi->next){
           csi->next->multicolumn=FALSE;
-          trace("Thread %d: New next with where %s | rows: %lld", td->thread_id, csi->where->str, rows);
+          trace("Thread %d: I-Chunk 2: New next with where %s | rows: %lld", td->thread_id, csi->where->str, rows);
         }
       }else{
-        trace("Thread %d: multicolumn=FALSE", td->thread_id);
+        trace("Thread %d: I-Chunk 2: multicolumn=FALSE", td->thread_id);
         csi->multicolumn=FALSE;
       }
     }
@@ -582,7 +701,7 @@ retry:
       cs->integer_step.type.unsign.cursor = cs->integer_step.type.unsign.min + cs->integer_step.step - 1;
 
     if (cs->integer_step.type.unsign.cursor < cs->integer_step.type.unsign.min)
-      g_error("Thread %d: integer_step.type.unsign.cursor: %"G_GUINT64_FORMAT"  | integer_step.type.unsign.min %"G_GUINT64_FORMAT"  | cs->integer_step.type.unsign.max : %"G_GUINT64_FORMAT" | cs->integer_step.step %ld", td->thread_id, cs->integer_step.type.unsign.cursor, cs->integer_step.type.unsign.min, cs->integer_step.type.unsign.max, cs->integer_step.step);
+      g_error("Thread %d: integer_step.type.unsign.cursor: %"G_GUINT64_FORMAT"  | integer_step.type.unsign.min %"G_GUINT64_FORMAT"  | cs->integer_step.type.unsign.max : %"G_GUINT64_FORMAT" | cs->integer_step.step %"G_GUINT64_FORMAT, td->thread_id, cs->integer_step.type.unsign.cursor, cs->integer_step.type.unsign.min, cs->integer_step.type.unsign.max, cs->integer_step.step);
 
     g_assert(cs->integer_step.type.unsign.cursor >= cs->integer_step.type.unsign.min);
 
@@ -595,7 +714,7 @@ retry:
       cs->integer_step.type.sign.cursor = cs->integer_step.type.sign.min + cs->integer_step.step - 1;
 
     if (cs->integer_step.type.sign.cursor < cs->integer_step.type.sign.min)
-      g_error("Thread %d: integer_step.type.sign.cursor: %"G_GINT64_FORMAT"  | integer_step.type.sign.min %"G_GINT64_FORMAT"  | cs->integer_step.type.sign.max : %"G_GINT64_FORMAT" | cs->integer_step.step %ld", td->thread_id, cs->integer_step.type.sign.cursor, cs->integer_step.type.sign.min, cs->integer_step.type.sign.max, cs->integer_step.step);
+      g_error("Thread %d: integer_step.type.sign.cursor: %"G_GINT64_FORMAT"  | integer_step.type.sign.min %"G_GINT64_FORMAT"  | cs->integer_step.type.sign.max : %"G_GINT64_FORMAT" | cs->integer_step.step %"G_GUINT64_FORMAT, td->thread_id, cs->integer_step.type.sign.cursor, cs->integer_step.type.sign.min, cs->integer_step.type.sign.max, cs->integer_step.step);
 
     g_assert(cs->integer_step.type.sign.cursor >= cs->integer_step.type.sign.min);
 
@@ -616,16 +735,16 @@ retry:
 // Step 3: Executing query and writing data
   update_where_on_integer_step(csi);
   if (csi->prefix)
-    trace("Thread %d: PREFIX: %s WHERE: %s", td->thread_id, csi->prefix->str, csi->where->str);
+    trace("Thread %d: I-Chunk 3: PREFIX: %s WHERE: %s", td->thread_id, csi->prefix->str, csi->where->str);
   else
-    trace("Thread %d: WHERE: %s", td->thread_id, csi->where->str); 
+    trace("Thread %d: I-Chunk 3: WHERE: %s", td->thread_id, csi->where->str); 
 
   if (csi->next !=NULL){
-    trace("Thread %d: going down", td->thread_id);
+    trace("Thread %d: I-Chunk 3: going down", td->thread_id);
     // Multi column
     if (csi->next->needs_refresh)
       if (!refresh_integer_min_max(td->thrconn, tj->dbt, csi->next)){
-        trace("No min and max found");
+        trace("Thread %d: I-Chunk 3: No min and max found", td->thread_id);
         goto update_min;
       }
 
@@ -634,15 +753,17 @@ retry:
   }else{
     g_string_set_size(tj->where,0);
     g_string_append(tj->where, csi->where->str);
-    trace("Thread %d: WHERE in TJ: %s", td->thread_id, tj->where->str);
+    trace("Thread %d: I-Chunk 3: WHERE in TJ: %s", td->thread_id, tj->where->str);
     if (cs->integer_step.is_step_fixed_length) {
       if (cs->integer_step.is_unsigned)
         tj->part= cs->integer_step.type.unsign.min / cs->integer_step.step + 1;
       else
         tj->part= cs->integer_step.type.sign.min   / cs->integer_step.step + 1;      
-//      reopen_files(tj);
       close_files(tj);
       write_table_job_into_file(tj);
+    }else if (is_last(csi)) {
+      trace("Thread %d: I-Chunk 3: Last chunk on `%s`.`%s` no need to calculate anything else after finish", td->thread_id, tj->dbt->database->name, tj->dbt->table);
+      write_table_job_into_file(tj);
     }else{
       GDateTime *from = g_date_time_new_now_local();
       write_table_job_into_file(tj);
@@ -650,18 +771,37 @@ retry:
 
 // Step 3.1: Updating Step length
 
-      GTimeSpan diff=g_date_time_difference(to,from)/G_TIME_SPAN_SECOND;
+      GTimeSpan diff=g_date_time_difference(to,from);
       g_date_time_unref(from);
       g_date_time_unref(to);
       g_mutex_lock(csi->mutex);
-      if (diff > MAX_TIME_PER_QUERY){
-        cs->integer_step.step=cs->integer_step.step  / 2;
-        trace("Decreasing step to size %ld due time %ld seconds", cs->integer_step.step, diff);
-      }else if (diff < MAX_TIME_PER_QUERY){
-        cs->integer_step.step=cs->integer_step.step  * 2 == 0?cs->integer_step.step:cs->integer_step.step  * 2;
-        trace("Increasing step to size %ld due time %ld seconds", cs->integer_step.step, diff);
+
+
+// Let's calculate last run
+//
+// and we also going to calculate the average
+//
+// if last_run is above, then we use it
+// if it is not, we use the average
+//      cs->integer_step.rows_in_explain
+//      tj->num_rows_of_last_run
+
+      if (cs->integer_step.rows_in_explain > tj->num_rows_of_last_run )
+        cs->integer_step.rows_in_explain-=tj->num_rows_of_last_run;
+      else
+        cs->integer_step.rows_in_explain=0;
+
+      if (diff>0 && tj->num_rows_of_last_run>0){
+        cs->integer_step.step=tj->num_rows_of_last_run*max_time_per_select*G_TIME_SPAN_SECOND/diff;
+        trace("Thread %d: I-Chunk 3: Step size on `%s`.`%s` is %ld  ( %ld %ld)", td->thread_id, tj->dbt->database->name, tj->dbt->table, cs->integer_step.step, tj->num_rows_of_last_run, diff);
+      }else{
+        cs->integer_step.step*=2;
+        cs->integer_step.check_min=TRUE;
+        trace("Thread %d: I-Chunk 3: During last query we get zero rows, duplicating the step size to %ld", td->thread_id, cs->integer_step.step);
       }
-//      guint ant=cs->integer_step.step;
+
+
+
       cs->integer_step.step = csi->chunk_step->integer_step.max_chunk_step_size !=0 && cs->integer_step.step > csi->chunk_step->integer_step.max_chunk_step_size ? 
                               csi->chunk_step->integer_step.max_chunk_step_size :
                               cs->integer_step.step;
@@ -683,14 +823,14 @@ update_min:
   if (cs->integer_step.is_unsigned){
     if ( cs->integer_step.type.unsign.cursor+1 < cs->integer_step.type.unsign.min){
       // Overflow
-      trace("Overflow due integer_step.type.unsign.cursor: %"G_GUINT64_FORMAT"  | integer_step.type.unsign.min %"G_GUINT64_FORMAT, cs->integer_step.type.unsign.cursor, cs->integer_step.type.unsign.min);
+      trace("Thread %d: I-Chunk 5: Overflow due integer_step.type.unsign.cursor: %"G_GUINT64_FORMAT"  | integer_step.type.unsign.min %"G_GUINT64_FORMAT, td->thread_id, cs->integer_step.type.unsign.cursor, cs->integer_step.type.unsign.min);
       cs->integer_step.type.unsign.min=cs->integer_step.type.unsign.max;
       cs->integer_step.type.unsign.max--;
     }else
       cs->integer_step.type.unsign.min=cs->integer_step.type.unsign.cursor+1;
   }else{
     if ( cs->integer_step.type.sign.cursor+1 < cs->integer_step.type.sign.min){
-      trace("Overflow due integer_step.type.unsign.cursor: %"G_GUINT64_FORMAT"  | integer_step.type.unsign.min %"G_GINT64_FORMAT, cs->integer_step.type.sign.cursor, cs->integer_step.type.sign.min);
+      trace("Thread %d: I-Chunk 5: Overflow due integer_step.type.unsign.cursor: %"G_GUINT64_FORMAT"  | integer_step.type.unsign.min %"G_GINT64_FORMAT, td->thread_id, cs->integer_step.type.sign.cursor, cs->integer_step.type.sign.min);
       cs->integer_step.type.sign.min=cs->integer_step.type.sign.max;
       cs->integer_step.type.sign.max--;
     }else
@@ -701,10 +841,12 @@ update_min:
          gint64_abs(  csi->chunk_step->integer_step.type.sign.max -   csi->chunk_step->integer_step.type.sign.min);
   tmpstep++;
 
-  trace("Thread %d: integer_step.type.sign.cursor: %"G_GINT64_FORMAT"  | integer_step.type.sign.min %"G_GINT64_FORMAT"  | cs->integer_step.type.sign.max : %"G_GINT64_FORMAT" | cs->integer_step.step %ld | tmpstep: %d", td->thread_id, cs->integer_step.type.sign.cursor, cs->integer_step.type.sign.min, cs->integer_step.type.sign.max, cs->integer_step.step, tmpstep);
+  trace("Thread %d: I-Chunk 5: integer_step.type.sign.cursor: %"G_GINT64_FORMAT"  | integer_step.type.sign.min %"G_GINT64_FORMAT"  | cs->integer_step.type.sign.max : %"G_GINT64_FORMAT" | cs->integer_step.step %ld | tmpstep: %d", td->thread_id, cs->integer_step.type.sign.cursor, cs->integer_step.type.sign.min, cs->integer_step.type.sign.max, cs->integer_step.step, tmpstep);
   cs->integer_step.step=cs->integer_step.step>tmpstep?tmpstep:cs->integer_step.step;
 
-//  g_message("Thread %d: integer_step.type.sign.cursor: %"G_GINT64_FORMAT"  | integer_step.type.sign.min %"G_GINT64_FORMAT"  | cs->integer_step.type.sign.max : %"G_GINT64_FORMAT" | cs->integer_step.step %ld", td->thread_id, cs->integer_step.type.sign.cursor, cs->integer_step.type.sign.min, cs->integer_step.type.sign.max, cs->integer_step.step);
+//  g_message("Thread %d: I-Chunk 5: integer_step.type.sign.cursor: %"G_GINT64_FORMAT"  | integer_step.type.sign.min %"G_GINT64_FORMAT"  | cs->integer_step.type.sign.max : %"G_GINT64_FORMAT" | cs->integer_step.step %ld", td->thread_id, cs->integer_step.type.sign.cursor, cs->integer_step.type.sign.min, cs->integer_step.type.sign.max, cs->integer_step.step);
+
+//end_process:
 
   if (csi->position==0)
     csi->multicolumn=tj->dbt->multicolumn;
@@ -846,6 +988,13 @@ void update_where_on_integer_step(struct chunk_step_item * csi){
   update_integer_where_on_gstring(csi->where, csi->include_null, csi->prefix, csi->field, chunk_step->is_unsigned, chunk_step->type, TRUE);
 }
 
+static
+GString * get_where_from_csi(struct chunk_step_item * csi){
+  GString *where = g_string_new("");
+  update_integer_where_on_gstring(where, FALSE, csi->prefix, csi->field, csi->chunk_step->integer_step.is_unsigned, csi->chunk_step->integer_step.type, FALSE);
+  return where;
+}
+
 void determine_if_we_can_go_deeper( struct chunk_step_item * csi, guint64 rows){
   if (csi->multicolumn && csi->position == 0){
     trace("is_unsigned: %d | rows: %lld | max - min: %lld", csi->chunk_step->integer_step.is_unsigned, rows, csi->chunk_step->integer_step.is_unsigned?(csi->chunk_step->integer_step.type.unsign.max - csi->chunk_step->integer_step.type.unsign.min):gint64_abs(csi->chunk_step->integer_step.type.sign.max -   csi->chunk_step->integer_step.type.sign.min));
