@@ -28,6 +28,7 @@
 #include "mydumper_database.h"
 #include "mydumper_write.h"
 #include "mydumper_global.h"
+#include "mydumper_working_thread.h"
 
 /* Program options */
 gboolean dump_triggers = FALSE;
@@ -35,7 +36,8 @@ gboolean ignore_generated_fields = FALSE;
 gboolean skip_definer = FALSE;
 
 extern gchar *table_engine_for_view_dependency;
-
+extern gchar *case_sensitive_prefix;
+extern gchar *case_sensitive_suffix;
 // Shared variables
 int (*m_open)(char **filename, const char *);
 
@@ -47,16 +49,9 @@ void initialize_jobs(){
 }
 
 static
-gchar * write_checksum_into_file(MYSQL *conn, struct database *database, char *table, gchar *fun(MYSQL *,gchar *,gchar *,int*)) {
-  int errn=0;
-  gchar *checksum=fun(conn, database->name, table, &errn);
-//  g_message("Checksum value: %s", checksum);
-  if (errn != 0 && !(success_on_1146 && errn == 1146)) {
-    g_warning("Writing checksum");
-    errors++;
-    return NULL;
-  }
-  if (checksum == NULL)
+gchar * write_checksum_into_file(MYSQL *conn, struct database *database, char *table, gchar *fun(MYSQL *,gchar *,gchar *)) {
+  gchar *checksum=fun(conn, database->name, table);
+  if (!checksum)
     checksum = g_strdup("0");
   return checksum;
 }
@@ -166,9 +161,16 @@ void write_table_definition_into_file(MYSQL *conn, struct db_table *dbt,
     return;
   }
 
-  const char q= identifier_quote_character;
   GString *statement = g_string_sized_new(statement_size);
-  initialize_sql_statement(statement);
+
+  initialize_header_in_gstring(statement, set_names_in_file_for_sct);
+  if (!g_strcmp0(set_names_in_conn_for_sct, AUTO_CHARSET)){
+    if (dbt->character_set)
+      execute_set_names(conn, dbt->character_set);
+  }else
+    execute_set_names(conn, set_names_in_conn_for_sct);
+
+
 
   if (!write_data(outfile, statement)) {
     g_critical("Could not write schema data for %s.%s", dbt->database->name, dbt->table);
@@ -176,12 +178,12 @@ void write_table_definition_into_file(MYSQL *conn, struct db_table *dbt,
     return;
   }
 
-  query = g_strdup_printf("SHOW CREATE TABLE %c%s%c.%c%s%c", q, dbt->database->name, q, q, dbt->table, q);
+  query = g_strdup_printf("SHOW CREATE TABLE %c%s%c.%c%s%c", identifier_quote_character, dbt->database->name, identifier_quote_character, identifier_quote_character, dbt->table, identifier_quote_character);
   struct M_ROW *mr = m_store_result_row(conn, query, m_critical, m_warning, "Error dumping schemas (%s.%s)", dbt->database->name, dbt->table);
   g_free(query);
   if (!mr->res){
     m_store_result_row_free(mr);
-    return;
+    goto end;
   }
 
   g_string_set_size(statement, 0);
@@ -197,11 +199,15 @@ void write_table_definition_into_file(MYSQL *conn, struct db_table *dbt,
 
   g_string_append(statement, ";\n");
 
+  GString *alter_table_statement=g_string_sized_new(statement_size);
+  GString *alter_table_constraint_statement=g_string_sized_new(statement_size);
+  GString *create_table_statement=g_string_sized_new(statement_size);
+  int flag = global_process_create_table_statement(statement->str, create_table_statement, alter_table_statement, alter_table_constraint_statement, dbt->table, TRUE);
+  if ( !(flag & IS_TRX_TABLE) && trx_tables && sync_thread_lock_mode!=NO_LOCK){
+    m_error("Non transactional table found: `%s`.`%s` on a consitent backup attempt. Restart backup using --trx-tables=0 to indicate that you have non transactional tables.", dbt->database->name, dbt->table);
+  }
+
   if (skip_indexes || skip_constraints){
-    GString *alter_table_statement=g_string_sized_new(statement_size);
-    GString *alter_table_constraint_statement=g_string_sized_new(statement_size);
-    GString *create_table_statement=g_string_sized_new(statement_size);
-    global_process_create_table_statement(statement->str, create_table_statement, alter_table_statement, alter_table_constraint_statement, dbt->table, TRUE);
     if (!write_data(outfile, create_table_statement)) {
       g_critical("Could not write schema for %s.%s", dbt->database->name, dbt->table);
       errors++;
@@ -211,6 +217,7 @@ void write_table_definition_into_file(MYSQL *conn, struct db_table *dbt,
     if (!skip_constraints)
       write_data(outfile, alter_table_constraint_statement);
   }else{
+
     if (!write_data(outfile, statement)) {
       g_critical("Could not write schema for %s.%s", dbt->database->name, dbt->table);
       errors++;
@@ -218,13 +225,19 @@ void write_table_definition_into_file(MYSQL *conn, struct db_table *dbt,
   }
   m_close(0, outfile, filename, 1, dbt);
   g_string_free(statement, TRUE);
+  g_string_free(alter_table_statement, TRUE);
+  g_string_free(alter_table_constraint_statement, TRUE);
+  g_string_free(create_table_statement, TRUE);
 
   if (checksum_filename)
     dbt->schema_checksum=write_checksum_into_file(conn, dbt->database, dbt->table, checksum_table_structure);
   
   if (checksum_index_filename)
     dbt->indexes_checksum=write_checksum_into_file(conn, dbt->database, dbt->table, checksum_table_indexes);
-  
+
+end:
+  execute_set_names(conn, set_names_in_conn_by_default);
+
   return;
 }
 
@@ -387,7 +400,7 @@ void write_view_definition_into_file(MYSQL *conn, struct db_table *dbt, char *tm
   }
   g_string_append(statement, "\n) ENGINE=");
   g_string_append(statement, table_engine_for_view_dependency);
-  if (get_product() == SERVER_TYPE_PERCONA || get_product() == SERVER_TYPE_MYSQL || get_product() == SERVER_TYPE_DOLT)
+  if (get_product() == SERVER_TYPE_PERCONA || get_product() == SERVER_TYPE_MYSQL || get_product() == SERVER_TYPE_RDS || get_product() == SERVER_TYPE_DOLT)
     g_string_append(statement," ENCRYPTION='N'");
   g_string_append(statement,";\n");
 
@@ -456,7 +469,6 @@ static
 void write_sequence_definition_into_file(MYSQL *conn, struct db_table *dbt, char *filename, gboolean checksum_filename) {
   int outfile;
   char *query = NULL;
-  const char q= identifier_quote_character;
   GString *statement = g_string_sized_new(statement_size);
   initialize_sql_statement(statement);
 
@@ -472,8 +484,8 @@ void write_sequence_definition_into_file(MYSQL *conn, struct db_table *dbt, char
   }
 
   // DROP TABLE works for sequences
-  g_string_append_printf(statement, "DROP TABLE IF EXISTS %c%s%c;\n", q, dbt->table, q);
-  g_string_append_printf(statement, "DROP VIEW IF EXISTS %c%s%c;\n", q, dbt->table, q);
+  g_string_append_printf(statement, "DROP TABLE IF EXISTS %c%s%c;\n", identifier_quote_character, dbt->table, identifier_quote_character);
+  g_string_append_printf(statement, "DROP VIEW IF EXISTS %c%s%c;\n" , identifier_quote_character, dbt->table, identifier_quote_character);
 
   if (!write_data(outfile, statement)) {
     g_critical("Could not write schema data for %s.%s", dbt->database->name, dbt->table);
@@ -481,7 +493,7 @@ void write_sequence_definition_into_file(MYSQL *conn, struct db_table *dbt, char
     return;
   }
 
-  query = g_strdup_printf("SHOW CREATE SEQUENCE %c%s%c.%c%s%c", q, dbt->database->name, q, q, dbt->table, q);
+  query = g_strdup_printf("SHOW CREATE SEQUENCE %c%s%c.%c%s%c", identifier_quote_character, dbt->database->name, identifier_quote_character, identifier_quote_character, dbt->table, identifier_quote_character);
   struct M_ROW *mr = m_store_result_row(conn, query, m_critical, m_warning, "Error dumping schemas (%s.%s)", dbt->database->name, dbt->table);
   g_free(query);
   if (!mr->res){
@@ -503,14 +515,14 @@ void write_sequence_definition_into_file(MYSQL *conn, struct db_table *dbt, char
   m_store_result_row_free(mr);
 
   // Get current sequence position
-  query = g_strdup_printf("SELECT next_not_cached_value FROM %c%s%c.%c%s%c", q, dbt->database->name, q, q, dbt->table, q);
+  query = g_strdup_printf("SELECT next_not_cached_value FROM %c%s%c.%c%s%c", identifier_quote_character, dbt->database->name, identifier_quote_character, identifier_quote_character, dbt->table, identifier_quote_character);
   mr = m_store_result_row(conn, query, m_critical, m_warning, "Error dumping schemas (%s.%s)", dbt->database->name, dbt->table);
   g_free(query);
 
   g_string_set_size(statement, 0);
   /* There should never be more than one row */
   if (mr->row){
-    g_string_printf(statement, "DO SETVAL(%c%s%c, %s, 0);\n", q, dbt->table, q, mr->row[0]);
+    g_string_printf(statement, "DO SETVAL(%c%s%c, %s, 0);\n", identifier_quote_character, dbt->table, identifier_quote_character, mr->row[0]);
     if (!write_data(outfile, statement)) {
       g_critical("Could not write schema for %s.%s", dbt->database->name, dbt->table);
       errors++;
@@ -557,7 +569,7 @@ void write_routines_definition_into_file(MYSQL *conn, struct database *database,
     g_assert(nroutines > 0);
     struct M_ROW *mr=NULL;
     for (guint r= 0; r < nroutines; r++) {
-      query= g_strdup_printf("SHOW %s STATUS WHERE CAST(Db AS BINARY) = '%s'", routine_type[r], database->escaped);
+      query= g_strdup_printf("SHOW %s STATUS WHERE %s Db %s = '%s'", routine_type[r], case_sensitive_prefix, case_sensitive_suffix, database->escaped);
       MYSQL_RES *result = m_store_result_critical(conn, query,  "Error dumping %s from %s", routine_type[r], database->name);
       g_free(query);
       if (!result)
