@@ -18,7 +18,20 @@
                     Max Bubenick, Percona RDBA (max dot bubenick at percona dot com)
                     David Ducos, Percona (david dot ducos at percona dot com)
 */
-union chunk_step *new_real_partition_step(GList *partition, guint deep, guint number);
+
+#if !defined(mydumper_mydumper_partition_chunks)
+#define mydumper_mydumper_partition_chunks
+
+#include "mydumper_chunks.h"
+
+struct partition_step{
+  GList *list;
+  gchar *current_partition;
+};
+
+#endif
+
+union chunk_step *new_real_partition_step(GList *partition);
 struct chunk_step_item *get_next_partition_chunk(struct db_table *dbt);
 GList * get_partitions_for_table(MYSQL *conn, struct db_table *dbt);
 void process_partition_chunk(struct table_job *tj, struct chunk_step_item *csi);
