@@ -30,7 +30,7 @@
 #include "mydumper_stream.h"
 #include "mydumper_database.h"
 #include "mydumper_working_thread.h"
-#include "mydumper_pmm_thread.h"
+#include "mydumper_pmm.h"
 #include "mydumper_exec_command.h"
 #include "mydumper_masquerade.h"
 #include "mydumper_chunks.h"
@@ -67,6 +67,9 @@ gboolean replica_stopped = FALSE;
 gboolean merge_dumpdir= FALSE;
 gboolean clear_dumpdir= FALSE;
 gboolean dirty_dumpdir= FALSE;
+gchar *initial_source_log = NULL;
+gchar *initial_source_pos = NULL;
+gchar *initial_source_gtid = NULL;
 
 // Program options used only on this file 
 extern guint ftwrl_max_wait_time;
@@ -82,12 +85,12 @@ static gboolean ftwrl_completed=FALSE;
 static
 void initialize_start_dump(){
   all_dbts=g_hash_table_new(g_str_hash, g_str_equal);
-  initialize_set_names();
+  initialize_table();
   initialize_working_thread();
 	initialize_conf_per_table(&conf_per_table);
 
   // until we have an unique option on lock types we need to ensure this
-  if (sync_thread_lock_mode==NO_LOCK)
+  if (sync_thread_lock_mode==NO_LOCK || sync_thread_lock_mode==SAFE_NO_LOCK)
     trx_tables=TRUE;
 
   // clarify binlog coordinates with --trx-tables
@@ -98,7 +101,6 @@ void initialize_start_dump(){
   if (db){
     db_items=g_strsplit(db,",",0);
   }
-  initialize_pmm();
 }
 
 void set_disk_limits(guint p_at, guint r_at){
@@ -387,7 +389,7 @@ MYSQL *create_main_connection() {
   set_session = g_string_new(NULL);
   set_global = g_string_new(NULL);
   set_global_back = g_string_new(NULL);
-  detect_server_version(conn);
+  server_detect(conn);
   GHashTable * set_session_hash = mydumper_initialize_hash_of_session_variables();
   GHashTable * set_global_hash = g_hash_table_new ( g_str_hash, g_str_equal );
   if (key_file != NULL ){
@@ -412,29 +414,16 @@ MYSQL *create_main_connection() {
 
   switch (get_product()) {
   case SERVER_TYPE_MYSQL:
-    set_transaction_isolation_level_repeatable_read(conn);
-    break;
   case SERVER_TYPE_MARIADB:
+  case SERVER_TYPE_PERCONA:
+  case SERVER_TYPE_DOLT:
+  case SERVER_TYPE_RDS:
+  case SERVER_TYPE_UNKNOWN:
     set_transaction_isolation_level_repeatable_read(conn);
     break;
   case SERVER_TYPE_TIDB:
     data_checksums=FALSE;
     break;
-  case SERVER_TYPE_PERCONA:
-    set_transaction_isolation_level_repeatable_read(conn);
-    break;
-  case SERVER_TYPE_UNKNOWN:
-    set_transaction_isolation_level_repeatable_read(conn);
-    break;
-  case SERVER_TYPE_CLICKHOUSE:
-    data_checksums=FALSE;
-    break;
-  case SERVER_TYPE_DOLT:
-    set_transaction_isolation_level_repeatable_read(conn);
-    break;
-  default:
-    m_critical("Cannot detect server type");
-    break;
   }
 
   g_message("Connected to %s %d.%d.%d", get_product_name(), get_major(), get_secondary(), get_revision());
@@ -619,7 +608,7 @@ void determine_ddl_lock_function(MYSQL ** conn, void(**acquire_global_lock_funct
           *acquire_global_lock_function = &send_flush_table_with_read_lock;
           *release_global_lock_function = &send_unlock_tables;
 
-	  break;
+          break;
         case 5:
           if (get_secondary() == 7) {
             if (no_backup_locks){
@@ -645,6 +634,7 @@ void determine_ddl_lock_function(MYSQL ** conn, void(**acquire_global_lock_funct
           break;
       }
       break;
+    case SERVER_TYPE_RDS:
     case SERVER_TYPE_MYSQL: 
       switch (get_major()) {
         case 8:
@@ -711,7 +701,7 @@ void print_dbt_on_metadata(FILE *mdfile, struct db_table *dbt){
   GString *data = g_string_sized_new(100);
   print_dbt_on_metadata_gstring(dbt, data);
   fprintf(mdfile, "%s", data->str);
-  if (check_row_count && (dbt->rows != dbt->rows_total)) {
+  if (check_row_count && !dbt->object_to_export.no_data && (dbt->rows != dbt->rows_total)) {
     m_critical("Row count mismatch found for %s.%s: got %u of %u expected",
                dbt->database->name, dbt->table, dbt->rows, dbt->rows_total);
   }
@@ -831,109 +821,6 @@ void send_lock_all_tables(MYSQL *conn){
   g_free(query->str);
   g_list_free(tables_lock);
 }
-/*
-static
-void write_replica_info(MYSQL *conn, FILE *file) {
-  MYSQL_RES *slave = NULL;
-  MYSQL_FIELD *fields;
-  MYSQL_ROW row;
-
-  char *slavehost = NULL;
-  char *slavelog = NULL;
-  char *slavepos = NULL;
-  char *slavegtid = NULL;
-
-  char *channel_name = NULL;
-  const char *gtid_title = NULL;
-  guint i;
-  guint isms = 0;
-  MYSQL_RES *rest=NULL;
-  if (get_product() == SERVER_TYPE_MARIADB ){
-    rest=m_store_result(conn, "SELECT @@default_master_connection", m_warning, "Variable @@default_master_connection not found", NULL);
-    if (rest != NULL && mysql_num_rows(rest)) {
-      mysql_free_result(rest);
-      g_message("Multisource slave detected.");
-      isms = 1;
-    }
-  }
-
-  if (isms)
-    m_query_critical(conn, show_all_replicas_status, "Error executing %s", show_all_replicas_status);
-  else
-    m_query_critical(conn, show_replica_status, "Error executing %s", show_replica_status);
-
-  guint slave_count=0;
-  slave = mysql_store_result(conn);
-
-  if (!slave || mysql_num_rows(slave) == 0){
-    goto cleanup;
-  }
-  mysql_free_result(slave);
-  g_message("Stopping replica");
-  replica_stopped=!m_query_warning(conn, stop_replica_sql_thread, "Not able to stop replica",NULL);
-  if (source_control_command==AWS){
-    discard_mysql_output(conn);
-  } 
-
-  if (isms)  
-    m_query_critical(conn, show_all_replicas_status, "Error executing %s", show_all_replicas_status);
-  else
-    m_query_critical(conn, show_replica_status, "Error executing %s", show_replica_status);
-
-  slave = mysql_store_result(conn);
-
-  GString *replication_section_str = g_string_sized_new(100);
-
-  while (slave && (row = mysql_fetch_row(slave))) {
-    g_string_set_size(replication_section_str,0);
-    fields = mysql_fetch_fields(slave);
-    slavepos=NULL;
-    slavelog=NULL;
-    slavehost=NULL;
-    slavegtid=NULL;
-    channel_name=NULL;
-    gtid_title=NULL;
-    for (i = 0; i < mysql_num_fields(slave); i++) {
-      if (!strcasecmp("exec_master_log_pos", fields[i].name)          || !strcasecmp("exec_source_log_pos", fields[i].name)) {
-        slavepos = row[i];
-      } else if (!strcasecmp("relay_master_log_file", fields[i].name) || !strcasecmp("relay_source_log_file", fields[i].name)) {
-        slavelog = row[i];
-      } else if (!strcasecmp("master_host", fields[i].name)           || !strcasecmp("source_host", fields[i].name)) {
-        slavehost = row[i];
-      } else if (!strcasecmp("Executed_Gtid_Set", fields[i].name)){
-        gtid_title="Executed_Gtid_Set";
-        slavegtid = remove_new_line(row[i]);
-      } else if (!strcasecmp("Gtid_Slave_Pos", fields[i].name)        || !strcasecmp("Gtid_source_Pos", fields[i].name)) {
-        gtid_title=g_strdup(fields[i].name);
-        slavegtid = remove_new_line(row[i]);
-      } else if ( ( !strcasecmp("connection_name", fields[i].name) || !strcasecmp("Channel_Name", fields[i].name) ) && strlen(row[i]) > 1) {
-        channel_name = row[i];
-      }
-      g_string_append_printf(replication_section_str,"# %s = ", fields[i].name);
-      (fields[i].type != MYSQL_TYPE_LONG && fields[i].type != MYSQL_TYPE_LONGLONG  && fields[i].type != MYSQL_TYPE_INT24  && fields[i].type != MYSQL_TYPE_SHORT )  ?
-      g_string_append_printf(replication_section_str,"'%s'\n", remove_new_line(row[i])):
-      g_string_append_printf(replication_section_str,"%s\n", remove_new_line(row[i]));
-    }
-    if (slavehost) {
-      slave_count++;
-      fprintf(file, "[replication%s%s]", channel_name!=NULL?".":"", channel_name!=NULL?channel_name:"");
-      fprintf(file, "\n# relay_master_log_file = \'%s\'\n# exec_master_log_pos = %s\n# %s = %s\n",
-              slavelog, slavepos, gtid_title, slavegtid);
-      fprintf(file,"%s",replication_section_str->str);
-      fprintf(file,"# myloader_exec_reset_slave = 0 # 1 means execute the command\n# myloader_exec_change_master = 0 # 1 means execute the command\n# myloader_exec_start_slave = 0 # 1 means execute the command\n");
-      g_message("Written slave status");
-    }
-  }
-  g_string_free(replication_section_str,TRUE);
-  if (slave_count > 1)
-    g_warning("Multisource replication found. Do not trust in the exec_master_log_pos as it might cause data inconsistencies. Search 'Replication and Transaction Inconsistencies' on MySQL Documentation");
-
-  fflush(file);
-cleanup:
-  if (slave)
-    mysql_free_result(slave);
-}
-*/
 
 guint isms = 0;
 static
@@ -972,9 +859,10 @@ cleanup:
 
 // Here is where the backup process start
 
-void start_dump() {
+void start_dump(struct configuration *conf) {
+  memset(conf, 0, sizeof(struct configuration));
+
   MYSQL *conn = NULL, *second_conn = NULL;
-  struct configuration conf;
   char *metadata_partial_filename, *metadata_filename;
   char *u;
   void (*acquire_global_lock_function)(MYSQL *) = NULL;
@@ -1014,17 +902,15 @@ void start_dump() {
 
   initialize_regex(partition_regex);
 
-
   // Connecting to the database
   conn = create_main_connection();
   main_connection = conn;
   second_conn = conn;
-  memset(&conf, 0, sizeof(conf));
-  conf.use_any_index= 1;
+  conf->use_any_index= 1;
 
   if (disk_limits!=NULL){
-    conf.pause_resume = g_async_queue_new();
-    disk_check_thread = m_thread_new("mon_disk",monitor_disk_space_thread, conf.pause_resume, "Monitor thread could not be created");
+    conf->pause_resume = g_async_queue_new();
+    disk_check_thread = m_thread_new("mon_disk",monitor_disk_space_thread, conf->pause_resume, "Monitor thread could not be created");
   }
 
 //  GThread *throttling_thread = 
@@ -1033,10 +919,7 @@ void start_dump() {
 
   // signal_thread is disable if daemon mode
   if (!daemon_mode)
-    sthread = m_thread_new("signal", signal_thread, &conf, "Signal thread could not be created");
-
-  start_pmm_thread(&conf);
-
+    sthread = m_thread_new("signal", signal_thread, conf, "Signal thread could not be created");
 
   // Initilizing METADATA file
   if (stream)
@@ -1065,7 +948,7 @@ void start_dump() {
   }
 
   // If we are locking, we need to be sure there is no long running queries
-  if (sync_thread_lock_mode!=NO_LOCK && is_mysql_like()) {
+  if (sync_thread_lock_mode!=NO_LOCK && sync_thread_lock_mode!=SAFE_NO_LOCK && is_mysql_like()) {
   // We check SHOW PROCESSLIST, and if there're queries
   // larger than preset value, we terminate the process.
   // This avoids stalling whole server with flush.
@@ -1075,6 +958,7 @@ void start_dump() {
   // Recoring that backup has started
   GDateTime *datetime = g_date_time_new_now_local();
   char *datetimestr=g_date_time_format(datetime,"\%Y-\%m-\%d \%H:\%M:\%S");
+  g_date_time_unref(datetime);
   fprintf(mdfile, "# Started dump at: %s\n", datetimestr);
   g_message("Started dump at: %s", datetimestr);
   g_free(datetimestr);
@@ -1086,7 +970,7 @@ void start_dump() {
   if (load_data || csv )
     fprintf(mdfile, "local-infile = 1\n");
   fprintf(mdfile, "\n[myloader_session_variables]");
-  fprintf(mdfile, "\nSQL_MODE=%s /*!40101\n\n", sql_mode);
+  fprintf(mdfile, "\nSQL_MODE=%s /*!40101\n", sql_mode);
   fflush(mdfile);
 
   // Initilizing stream backup
@@ -1094,8 +978,8 @@ void start_dump() {
     if (exec_command)
       g_error("--exec and --stream are not comptabile, use --exec-per-thread instead as file extension is needed to stream the out file");
     initialize_stream();
-    stream_queue_push(NULL, g_strdup(metadata_partial_filename));
     fclose(mdfile);
+    stream_queue_push(NULL, g_strdup(metadata_partial_filename));
     metadata_partial_filename= g_strdup_printf("%s/metadata.partial", dump_directory);
     mdfile= g_fopen(metadata_partial_filename, "w");
     if (!mdfile) {
@@ -1109,7 +993,7 @@ void start_dump() {
 
   // Write replica information
   if (get_product() != SERVER_TYPE_TIDB) {
-    if (source_data >=0 )
+    if (replica_data.enabled)
       m_stop_replica(conn);
   }
 
@@ -1117,7 +1001,10 @@ void start_dump() {
   // and send locks to database if needed
   switch (sync_thread_lock_mode){
     case NO_LOCK:
-      g_warning("Executing in NO_LOCK mode, we are not able to ensure that backup will be consistent");
+      g_message("Executing in NO_LOCK mode, we are not able to ensure that backup will be consistent");
+      break;
+    case SAFE_NO_LOCK:
+      g_message("Executing in SAFE_NO_LOCK mode. This backup will fail if all threads are not in the same point in time, which garanty consitency");
       break;
     case LOCK_ALL:
       send_lock_all_tables(conn);
@@ -1131,7 +1018,7 @@ void start_dump() {
       release_global_lock_function = &send_unlock_tables;
       break;
     case GTID:
-      g_warning("Using binlog_snapshot_gtid_executed which doesn't lock the database but uses best effort to sync the threads");
+      g_message("Using binlog_snapshot_gtid_executed which doesn't lock the database but uses best effort to sync the threads");
       break;
   }
   if (skip_ddl_locks){
@@ -1181,50 +1068,52 @@ void start_dump() {
     if (rest)
       mysql_free_result(rest);
   }
-
+  trace("Initilizing the configuration");
   // Initilizing the configuration
 
-  conf.initial_queue = g_async_queue_new();
-  conf.initial_completed_queue = g_async_queue_new();
+  conf->initial_queue = g_async_queue_new();
+  conf->initial_completed_queue = g_async_queue_new();
 
-  conf.schema_queue = g_async_queue_new();
-  conf.post_data_queue = g_async_queue_new();
-  conf.transactional.queue= g_async_queue_new();
-  conf.transactional.defer= g_async_queue_new();
+  conf->schema_queue = g_async_queue_new();
+  conf->post_data_queue = g_async_queue_new();
+  conf->transactional.queue= g_async_queue_new();
+  conf->transactional.defer= g_async_queue_new();
   // These are initialized in the guts of initialize_start_dump() above
   g_assert(give_me_another_transactional_chunk_step_queue &&
            give_me_another_non_transactional_chunk_step_queue &&
            transactional_table &&
            non_transactional_table);
-  conf.transactional.request_chunk= give_me_another_transactional_chunk_step_queue;
-  conf.transactional.table_list= transactional_table;
-  conf.transactional.descr= "transactional";
-  conf.ready = g_async_queue_new();
-  conf.non_transactional.queue= g_async_queue_new();
-  conf.non_transactional.defer= g_async_queue_new();
-  conf.non_transactional.request_chunk= give_me_another_non_transactional_chunk_step_queue;
-  conf.non_transactional.table_list= non_transactional_table;
-  conf.non_transactional.descr= "non-transactional";
-  conf.ready_non_transactional_queue = g_async_queue_new();
-  conf.unlock_tables = g_async_queue_new();
-  conf.gtid_pos_checked = g_async_queue_new();
-  conf.are_all_threads_in_same_pos = g_async_queue_new();
-  conf.db_ready = g_async_queue_new();
-  conf.binlog_ready = g_async_queue_new();
+  conf->transactional.request_chunk= give_me_another_transactional_chunk_step_queue;
+  conf->transactional.table_list= transactional_table;
+  conf->transactional.descr= "transactional";
+  conf->ready = g_async_queue_new();
+  conf->non_transactional.queue= g_async_queue_new();
+  conf->non_transactional.defer= g_async_queue_new();
+  conf->non_transactional.request_chunk= give_me_another_non_transactional_chunk_step_queue;
+  conf->non_transactional.table_list= non_transactional_table;
+  conf->non_transactional.descr= "non-transactional";
+  conf->ready_non_transactional_queue = g_async_queue_new();
+  conf->unlock_tables = g_async_queue_new();
+  conf->gtid_pos_checked = g_async_queue_new();
+  conf->are_all_threads_in_same_pos = g_async_queue_new();
+  conf->db_ready = g_async_queue_new();
+  conf->source_and_replica_status_queue = g_async_queue_new();
   ready_table_dump_mutex = g_rec_mutex_new();
   g_rec_mutex_lock(ready_table_dump_mutex);
 
-
+  trace("Begin Job Creation");
   // Begin Job Creation
 
   // Create metadata job
-  if (is_mysql_like()) {
-    create_job_to_dump_metadata(&conf, mdfile);
-  }
-
+  if (is_mysql_like())
+    create_job_to_write_source_and_replica_status(conf, mdfile);
+  else
+    g_async_queue_push(conf->source_and_replica_status_queue,GINT_TO_POINTER(1));
+  
+  trace("Create tablespace jobs");
   // Create tablespace jobs
   if (dump_tablespaces){
-    create_job_to_dump_tablespaces(&conf);
+    create_job_to_dump_tablespaces(conf);
   }
 
   // There are 3 ways to dump tables based on the filters
@@ -1234,26 +1123,51 @@ void start_dump() {
   //
   // if tables and db both exists , should not call dump_database_thread
   if (tables && g_strv_length(tables) > 0) {
-    create_job_to_dump_table_list(tables, &conf);
+    trace("Specific tables");
+    create_job_to_dump_table_list(tables, conf);
   } else if (db_items && g_strv_length(db_items) > 0) {
+    trace("Specific databases");
     guint i=0;
     for (i=0;i<g_strv_length(db_items);i++){
       struct database *this_db=new_database(conn,db_items[i],TRUE);
-      create_job_to_dump_database(this_db, &conf);
+      create_job_to_dump_database(this_db, conf);
       if (!no_schemas)
-        create_job_to_dump_schema(this_db, &conf);
+        create_job_to_dump_schema(this_db, conf);
     }
   } else {
-    create_job_to_dump_all_databases(&conf);
+    trace("All databases");
+    create_job_to_dump_all_databases(conf);
   }
 
+  trace("End Job Creation");
   // End Job Creation
 
   // Starting the chunk builder
-  start_chunk_builder(&conf);
+  start_chunk_builder(conf);
 
   // Starting the workers threads
-  start_working_thread(&conf);
+  start_working_thread(conf);
+  g_async_queue_pop(conf->source_and_replica_status_queue);
+  g_async_queue_unref(conf->source_and_replica_status_queue);
+
+  gchar *source_log = NULL;
+  gchar *source_pos = NULL;
+  gchar *source_gtid = NULL;
+  get_binlog_position(conn, &source_log, &source_pos, &source_gtid);
+  
+  if (g_strcmp0(source_log, initial_source_log) ||
+      g_strcmp0(source_pos, initial_source_pos) ||
+      g_strcmp0(source_gtid,initial_source_gtid)){
+    if (sync_thread_lock_mode == NO_LOCK){
+      g_warning("There are differences in the binlog position at the beginning of the backup and after syncing threads, so we cannot guarantee the backup to be consistent due to the use of NO_LOCK. Continues anyway, use SAFE_NO_LOCK otherwise.");
+      trace("Backup will be inconsistent %s %s %d || %s %s %d || %s %s %d", source_log, initial_source_log, g_strcmp0(source_log, initial_source_log), source_pos, initial_source_pos,g_strcmp0(source_pos, initial_source_pos), source_gtid,initial_source_gtid, g_strcmp0(source_gtid,initial_source_gtid));
+    } else if (sync_thread_lock_mode == SAFE_NO_LOCK){
+      trace("Backup will be inconsistent %s %s %d || %s %s %d || %s %s %d", source_log, initial_source_log, g_strcmp0(source_log, initial_source_log), source_pos, initial_source_pos,g_strcmp0(source_pos, initial_source_pos), source_gtid,initial_source_gtid, g_strcmp0(source_gtid,initial_source_gtid));
+      m_error("There are differences in the binlog position at the beginning of the backup and after syncing threads, so we cannot guarantee the backup to be consistent. Stopping backup due to the use of SAFE_NO_LOCK.");
+    }
+  }else{
+    g_message("Backup will be consistent");
+  }
 
   // IMPORTANT: At this point, all the threads are in sync
 
@@ -1261,13 +1175,12 @@ void start_dump() {
     // Releasing locks as user instructed that all tables are transactional
     g_message("Transactions started, unlocking tables");
     if (release_binlog_function != NULL){
-      g_async_queue_pop(conf.binlog_ready);
       g_message("Releasing binlog lock");
       release_binlog_function(second_conn);
     }
     if (release_global_lock_function)
       release_global_lock_function(conn);
-    if (is_mysql_like() && g_async_queue_pop(conf.binlog_ready) && replica_stopped){
+    if (is_mysql_like() && replica_stopped){
       g_message("Starting replica");
       m_query_warning(conn, start_replica_sql_thread, "Not able to start replica", NULL);
 
@@ -1280,9 +1193,9 @@ void start_dump() {
 
   // Every time a schema job is created a counter increases
   // Every time that a schema jobs is completed, the counter decreases
-  // When the counter reaches to 0, it releases conf.db_ready 
+  // When the counter reaches to 0, it releases conf->db_ready 
   g_message("Waiting database finish");
-  g_async_queue_pop(conf.db_ready);
+  g_async_queue_pop(conf->db_ready);
 
   // At this point all schema jobs are completed
 
@@ -1293,11 +1206,11 @@ void start_dump() {
   for (n = 0; n < num_threads; n++) {
     struct job *j = g_new0(struct job, 1);
     j->type = JOB_SHUTDOWN;
-    g_async_queue_push(conf.initial_queue, j);
+    g_async_queue_push(conf->initial_queue, j);
   }
 
   for (n = 0; n < num_threads; n++) {
-    g_async_queue_pop(conf.initial_completed_queue);
+    g_async_queue_pop(conf->initial_completed_queue);
   }
   // at this point initial jobs has been completed
   // which means that all schema jobs has been created 
@@ -1306,26 +1219,25 @@ void start_dump() {
   for (n = 0; n < num_threads; n++) {
     struct job *j = g_new0(struct job, 1);
     j->type = JOB_SHUTDOWN;
-    g_async_queue_push(conf.schema_queue, j);
+    g_async_queue_push(conf->schema_queue, j);
   }
   // In case that we are NOT exporting transactional table, we need to 
   // build the lock table statement, at this stage, before
   // let workers to start dumping data
   if (!trx_tables){
-    build_lock_tables_statement(&conf);
+    build_lock_tables_statement(conf);
   }
   // Allowing workers to start dumping Non-Transactional tables
   for (n = 0; n < num_threads; n++) {
-    g_async_queue_push(conf.ready_non_transactional_queue, GINT_TO_POINTER(1));
+    g_async_queue_push(conf->ready_non_transactional_queue, GINT_TO_POINTER(1));
   }
 
   // Releasing locks if possible
-  if (sync_thread_lock_mode!=NO_LOCK && !trx_tables) {
+  if (sync_thread_lock_mode!=NO_LOCK && sync_thread_lock_mode!=SAFE_NO_LOCK && !trx_tables) {
     for (n = 0; n < num_threads; n++) {
-      g_async_queue_pop(conf.unlock_tables);
+      g_async_queue_pop(conf->unlock_tables);
     }
     if (release_binlog_function != NULL){
-      g_async_queue_pop(conf.binlog_ready);
       g_message("Releasing binlog lock");
       release_binlog_function(second_conn);
     }
@@ -1336,7 +1248,7 @@ void start_dump() {
   }
 
   // At this point, we can start the replica if it was stopped
-  if (is_mysql_like() && g_async_queue_pop(conf.binlog_ready) && replica_stopped){
+  if (is_mysql_like() && replica_stopped){
     g_message("Starting replica");
     m_query_warning(conn, start_replica_sql_thread, "Not able to start replica", NULL);
 
@@ -1344,14 +1256,13 @@ void start_dump() {
       discard_mysql_output(conn);
     }
   }
-  g_async_queue_unref(conf.binlog_ready);
 
   // All the jobs related to post data has been created and enquequed
   // so, we can send the JOB_SHUTDOWN
   for (n = 0; n < num_threads; n++) {
     struct job *j = g_new0(struct job, 1);
     j->type = JOB_SHUTDOWN;
-    g_async_queue_push(conf.post_data_queue, j);
+    g_async_queue_push(conf->post_data_queue, j);
   }
   // At this point the main process, needs to wait the working threads to finish 
   wait_working_thread_to_finish();
@@ -1368,11 +1279,11 @@ void start_dump() {
     release_ddl_lock_function(second_conn);
   }
 
-  g_message("Queue count: %d %d %d %d %d", g_async_queue_length(conf.initial_queue),
-            g_async_queue_length(conf.schema_queue),
-            g_async_queue_length(conf.non_transactional.queue) + g_async_queue_length(conf.non_transactional.defer),
-            g_async_queue_length(conf.transactional.queue) + g_async_queue_length(conf.transactional.defer),
-            g_async_queue_length(conf.post_data_queue));
+  g_message("Queue count: %d %d %d %d %d", g_async_queue_length(conf->initial_queue),
+            g_async_queue_length(conf->schema_queue),
+            g_async_queue_length(conf->non_transactional.queue) + g_async_queue_length(conf->non_transactional.defer),
+            g_async_queue_length(conf->transactional.queue) + g_async_queue_length(conf->transactional.defer),
+            g_async_queue_length(conf->post_data_queue));
 
   // Closing main connection
   if (conn != second_conn)
@@ -1394,30 +1305,30 @@ void start_dump() {
   write_database_on_disk(mdfile);
   g_list_free(table_schemas);
   table_schemas=NULL;
-  stop_pmm_thread();
-  g_async_queue_unref(conf.transactional.defer);
-  conf.transactional.defer= NULL;
-  g_async_queue_unref(conf.transactional.queue);
-  conf.transactional.queue= NULL;
-  g_async_queue_unref(conf.non_transactional.defer);
-  conf.non_transactional.defer= NULL;
-  g_async_queue_unref(conf.non_transactional.queue);
-  conf.non_transactional.queue= NULL;
-  g_async_queue_unref(conf.unlock_tables);
-  conf.unlock_tables=NULL;
-  g_async_queue_unref(conf.ready);
-  conf.ready=NULL;
-  g_async_queue_unref(conf.schema_queue);
-  conf.schema_queue=NULL;
-  g_async_queue_unref(conf.initial_queue);
-  conf.initial_queue=NULL;
-  g_async_queue_unref(conf.post_data_queue);
-  conf.post_data_queue=NULL;
+  g_async_queue_unref(conf->transactional.defer);
+  conf->transactional.defer= NULL;
+  g_async_queue_unref(conf->transactional.queue);
+  conf->transactional.queue= NULL;
+  g_async_queue_unref(conf->non_transactional.defer);
+  conf->non_transactional.defer= NULL;
+  g_async_queue_unref(conf->non_transactional.queue);
+  conf->non_transactional.queue= NULL;
+  g_async_queue_unref(conf->unlock_tables);
+  conf->unlock_tables=NULL;
+  g_async_queue_unref(conf->ready);
+  conf->ready=NULL;
+  g_async_queue_unref(conf->schema_queue);
+  conf->schema_queue=NULL;
+  g_async_queue_unref(conf->initial_queue);
+  conf->initial_queue=NULL;
+  g_async_queue_unref(conf->post_data_queue);
+  conf->post_data_queue=NULL;
 
-  g_async_queue_unref(conf.ready_non_transactional_queue);
-  conf.ready_non_transactional_queue=NULL;
+  g_async_queue_unref(conf->ready_non_transactional_queue);
+  conf->ready_non_transactional_queue=NULL;
+
+  fprintf(mdfile, "[config]\nmax-statement-size = %ld\n", max_statement_size);
 
-  g_date_time_unref(datetime);
   datetime = g_date_time_new_now_local();
   datetimestr=g_date_time_format(datetime,"\%Y-\%m-\%d \%H:\%M:\%S");
   g_date_time_unref(datetime);
@@ -1473,15 +1384,15 @@ void start_dump() {
 
   finalize_masquerade();
 
-  g_async_queue_unref(conf.gtid_pos_checked);
-  g_async_queue_unref(conf.are_all_threads_in_same_pos);
-  g_async_queue_unref(conf.db_ready);
+  g_async_queue_unref(conf->gtid_pos_checked);
+  g_async_queue_unref(conf->are_all_threads_in_same_pos);
+  g_async_queue_unref(conf->db_ready);
   g_rec_mutex_clear(ready_table_dump_mutex);
 //  g_source_remove(SIGINT);
 //  g_source_remove(SIGTERM);
-//  g_main_loop_quit(conf.loop);
-  if (!daemon_mode && conf.loop)
-    g_main_loop_unref(conf.loop);
+//  g_main_loop_quit(conf->loop);
+  if (!daemon_mode && conf->loop)
+    g_main_loop_unref(conf->loop);
 
   if (tables_list)
     g_strfreev(tables);
