@@ -25,6 +25,7 @@
 #include "mydumper_global.h"
 #include "mydumper_stream.h"
 #include "mydumper_file_handler.h"
+#include "mydumper_write.h"
 
 GThread *stream_thread = NULL;
 GThread *metadata_partial_writer_thread = NULL;
@@ -55,7 +56,7 @@ void stream_queue_push(struct db_table *dbt,gchar *filename){
 void *process_stream(void *data){
   (void)data;
   int f=0;
-  char buf[STREAM_BUFFER_SIZE];
+  char *buf=g_new(gchar, STREAM_BUFFER_SIZE);
   int buflen;
   guint64 total_size=0;
   GDateTime *total_start_time=g_date_time_new_now_local();
@@ -101,7 +102,6 @@ void *process_stream(void *data){
           f=open(sf->filename,O_RDONLY);
           if (f < 0){
             m_error("File failed to open: %s (%s). Cancelling",sf->filename, strerror(errno));
-            exit(EXIT_FAILURE);
           }
         }
         trace("Streaming %s", sf->filename);
@@ -216,6 +216,7 @@ void *metadata_partial_writer(void *data){
       if (g_list_length(dbt_list) > 0){  
         filename= make_partial_filename(i);
         i++;
+        initialize_config_on_string(output);
         g_list_foreach(dbt_list,(GFunc)(&print_dbt_on_metadata_gstring),output);
         g_file_set_contents(filename,output->str,output->len,&gerror);
         stream_queue_push(NULL, filename);
