@@ -34,7 +34,7 @@
 #include "mydumper_file_handler.h"
 #include "mydumper_create_jobs.h"
 #include "mydumper_working_thread.h"
-
+#include "mydumper_table.h"
 /* Program options */
 gboolean order_by_primary_key = FALSE;
 gboolean use_savepoints = FALSE;
@@ -52,11 +52,6 @@ gboolean routine_checksums = FALSE;
 gboolean exit_if_broken_table_found = FALSE;
 int build_empty_files = 0;
 
-// Extern
-extern gboolean use_single_column;
-extern guint64 min_integer_chunk_step_size;
-extern guint64 max_integer_chunk_step_size;
-
 // Shared variables
 gint database_counter = 0;
 struct MList  *transactional_table = NULL;
@@ -68,17 +63,18 @@ gchar *exec_per_thread = NULL;
 const gchar *exec_per_thread_extension = NULL;
 gchar **exec_per_thread_cmd=NULL;
 
+extern gchar *initial_source_log;
+extern gchar *initial_source_pos;
+extern gchar *initial_source_gtid;
+
 // Static
 static GMutex *init_mutex = NULL;
 static gchar *binlog_snapshot_gtid_executed = NULL; 
 static char **ignore_engines = NULL;
 static int sync_wait = -1;
 static GMutex *table_schemas_mutex = NULL;
-static GMutex *all_dbts_mutex=NULL;
 static GMutex *trigger_schemas_mutex = NULL;
 static GMutex *view_schemas_mutex = NULL;
-static GHashTable *character_set_hash=NULL;
-static GMutex *character_set_hash_mutex = NULL;
 static const guint tablecol= 0;
 static GThread **threads=NULL;
 static struct thread_data *thread_data;
@@ -96,8 +92,6 @@ void initialize_working_thread(){
     m_error("This should not happen");
   }
 
-  character_set_hash=g_hash_table_new_full ( g_str_hash, g_str_equal, &g_free, &g_free);
-  character_set_hash_mutex = g_mutex_new();
   transactional_table=g_new(struct MList, 1);
   non_transactional_table=g_new(struct MList, 1);
   transactional_table->list=NULL;
@@ -108,7 +102,6 @@ void initialize_working_thread(){
   view_schemas_mutex = g_mutex_new();
   table_schemas_mutex = g_mutex_new();
   trigger_schemas_mutex = g_mutex_new();
-  all_dbts_mutex = g_mutex_new();
   init_mutex = g_mutex_new();
   binlog_snapshot_gtid_executed = NULL;
 
@@ -128,39 +121,7 @@ void initialize_working_thread(){
   if (ignore_engines_str)
     ignore_engines = g_strsplit(ignore_engines_str, ",", 0);
 
-
-// TODO: We need to cleanup this
-
-  if ((exec_per_thread_extension==NULL) && (exec_per_thread != NULL))
-    m_critical("--exec-per-thread-extension needs to be set when --exec-per-thread (%s) is used", exec_per_thread);
-  if ((exec_per_thread_extension!=NULL) && (exec_per_thread == NULL))
-    m_critical("--exec-per-thread needs to be set when --exec-per-thread-extension (%s) is used", exec_per_thread_extension);
-
-  if (compress_method==NULL && exec_per_thread==NULL) {
-    exec_per_thread_extension=EMPTY_STRING;
-    initialize_file_handler(FALSE);
-  }else{
-    if (compress_method!=NULL && exec_per_thread!=NULL )
-      m_critical("--compression and --exec-per-thread are not comptatible");
-    
-    if (compress_method){
-      if ( g_ascii_strcasecmp(compress_method,GZIP)==0){
-        exec_per_thread=g_strdup_printf("%s -c", GZIP);
-        exec_per_thread_extension=GZIP_EXTENSION;
-      }else if (g_ascii_strcasecmp(compress_method,ZSTD)==0){
-        exec_per_thread=g_strdup_printf("%s -c", ZSTD);
-        exec_per_thread_extension=ZSTD_EXTENSION;
-      }
-    }
-    initialize_file_handler(TRUE);
-
-    exec_per_thread_cmd=g_strsplit(exec_per_thread, " ", 0);
-    gchar *tmpcmd=g_find_program_in_path(exec_per_thread_cmd[0]);
-    if (!tmpcmd)
-      m_critical("%s was not found in PATH, use --exec-per-thread for non default locations",exec_per_thread_cmd[0]);
-    exec_per_thread_cmd[0]=tmpcmd;
-  }
-
+  initialize_file_handler();
 
   initialize_jobs();
   initialize_chunk();
@@ -226,79 +187,16 @@ void wait_working_thread_to_finish(){
 }
 
 void finalize_working_thread(){
-  g_hash_table_destroy(character_set_hash);
-  g_mutex_free(character_set_hash_mutex);
   g_mutex_free(view_schemas_mutex);
   g_mutex_free(table_schemas_mutex);
   g_mutex_free(trigger_schemas_mutex);
-  g_mutex_free(all_dbts_mutex);
   g_mutex_free(init_mutex);
   if (binlog_snapshot_gtid_executed!=NULL)
     g_free(binlog_snapshot_gtid_executed);
 
-  finalize_chunk();
   g_free(thread_data);
   g_free(threads);
-}
-
-static
-void get_primary_key(MYSQL *conn, struct db_table * dbt, struct configuration *conf){
-  MYSQL_RES *indexes = NULL;
-  MYSQL_ROW row;
-  dbt->primary_key=NULL;
-  // first have to pick index, in future should be able to preset in
-  //    * configuration too 
-  gchar *query = g_strdup_printf("SHOW INDEX FROM %s%s%s.%s%s%s",
-                        identifier_quote_character_str, dbt->database->name, identifier_quote_character_str, identifier_quote_character_str, dbt->table, identifier_quote_character_str);
-  indexes = m_store_result(conn, query, m_warning, "Failed to execute SHOW INDEX over %s", dbt->database->name);
-  g_free(query);
-
-  if (indexes){
-    while ((row = mysql_fetch_row(indexes))) {
-      if (!strcmp(row[2], "PRIMARY") ) {
-        // Pick first column in PK, cardinality doesn't matter
-        dbt->primary_key=g_list_append(dbt->primary_key,g_strdup(row[4]));
-      }
-    }
-    if (dbt->primary_key)
-      goto cleanup;
-
-    // If no PK found, try using first UNIQUE index
-    mysql_data_seek(indexes, 0);
-    while ((row = mysql_fetch_row(indexes))) {
-      if (!strcmp(row[1], "0")) {
-        // Again, first column of any unique index 
-        dbt->primary_key=g_list_append(dbt->primary_key,g_strdup(row[4]));
-      }
-    }
-
-    if (dbt->primary_key)
-      goto cleanup;
-
-    // Still unlucky? Pick any high-cardinality index 
-    if (!dbt->primary_key && conf->use_any_index) {
-      guint64 max_cardinality = 0;
-      guint64 cardinality = 0;
-      gchar *field=NULL;
-      mysql_data_seek(indexes, 0);
-      while ((row = mysql_fetch_row(indexes))) {
-        if (!strcmp(row[3], "1")) {
-          if (row[6])
-            cardinality = strtoul(row[6], NULL, 10);
-          if (cardinality > max_cardinality) {
-            field = g_strdup(row[4]);
-            max_cardinality = cardinality;
-          }
-        }
-      }
-      if (field)
-        dbt->primary_key=g_list_append(dbt->primary_key,field);
-    }
-  }
-
-cleanup:
-  if (indexes)
-    mysql_free_result(indexes);
+  finalize_table();
 }
 
 void thd_JOB_DUMP_ALL_DATABASES( struct thread_data *td, struct job *job){
@@ -417,11 +315,8 @@ void get_table_info_to_process_from_list(MYSQL *conn, struct configuration *conf
     g_async_queue_push(conf->db_ready,GINT_TO_POINTER(1));
 //    g_rec_mutex_unlock(ready_database_dump_mutex);
   }
-
 }
 
-
-
 void thd_JOB_DUMP_TABLE_LIST(struct thread_data *td, struct job *job){
   struct dump_table_list_job * dtlj = (struct dump_table_list_job *)job->job_data;
   get_table_info_to_process_from_list(td->thrconn, td->conf, dtlj->table_list);
@@ -547,62 +442,58 @@ void check_connection_status(struct thread_data *td){
 }
 
 
-/* Write some stuff we know about snapshot, before it changes */
-void write_snapshot_info(MYSQL *conn, FILE *file) {
-  char *masterlog = NULL;
-  char *masterpos = NULL;
-  char *mastergtid = NULL;
-
-  
+void get_binlog_position(MYSQL *conn, char **masterlog, char **masterpos, char **mastergtid){
   struct M_ROW *mr = m_store_result_row(conn, show_binary_log_status, m_warning, m_message, "Couldn't get master position", NULL);
   if ( mr->row ) {
-    masterlog = g_strdup(mr->row[0]);
-    masterpos = g_strdup(mr->row[1]);
-    /* Oracle/Percona GTID */
+    *masterlog = g_strdup(mr->row[0]);
+    *masterpos = g_strdup(mr->row[1]);
+    // Oracle/Percona GTID
     if (mysql_num_fields(mr->res) == 5) {
-      mastergtid = g_strdup(remove_new_line(mr->row[4]));
+      *mastergtid = g_strdup(remove_new_line(mr->row[4]));
     } else {
-      /* Let's try with MariaDB 10.x */
-      /* Use gtid_binlog_pos due to issue with gtid_current_pos with galera
- *        * cluster, gtid_binlog_pos works as well with normal mariadb server
- *               * https://jira.mariadb.org/browse/MDEV-10279 */
+      // Let's try with MariaDB 10.x
+      // Use gtid_binlog_pos due to issue with gtid_current_pos with galera
+      // cluster, gtid_binlog_pos works as well with normal mariadb server
+      // https://jira.mariadb.org/browse/MDEV-10279      
       m_store_result_row_free(mr);
       mr = m_store_result_row(conn, "SELECT @@gtid_binlog_pos", NULL, NULL, "Failed to get @@gtid_binlog_pos", NULL);
       if (mr->row){
-        mastergtid = g_strdup(remove_new_line(mr->row[0]));
+        *mastergtid = g_strdup(remove_new_line(mr->row[0]));
       }
     }
   }
   m_store_result_row_free(mr);
+}
 
-  if (masterlog) {
-    fprintf(file, "[source]\n# Channel_Name = '' # It can be use to setup replication FOR CHANNEL\n");
-    if (source_data > 0){
+
+/* Write some stuff we know about snapshot, before it changes */
+void write_snapshot_info(MYSQL *conn, FILE *file) {
+
+  get_binlog_position(conn, &initial_source_log, &initial_source_pos, &initial_source_gtid);
+
+  if (initial_source_log) {
+    fprintf(file, "\n[source]\n# Channel_Name = '' # It can be use to setup replication FOR CHANNEL\n");
+    if (source_data.enabled){
       fprintf(file, "#SOURCE_HOST = \"%s\"\n#SOURCE_PORT = \n#SOURCE_USER = \"\"\n#SOURCE_PASSWORD = \"\"\n", hostname?hostname:"");
-      if (((source_data) & (1<<(3)))>0)
+      if (source_data.source_ssl)
         fprintf(file, "SOURCE_SSL = 1\n");
       else
         fprintf(file, "#SOURCE_SSL = {0|1}\n");
-      fprintf(file, "executed_gtid_set = \"%s\"\n", mastergtid);
-      if (((source_data) & (1<<(4)))>0){
+      if (initial_source_gtid && strlen(initial_source_gtid)>0)
+        fprintf(file, "executed_gtid_set = \"%s\"\n", initial_source_gtid);
+      if (source_data.auto_position){
+        fprintf(file, "#SOURCE_LOG_FILE = \"%s\"\n#SOURCE_LOG_POS = %s\n", initial_source_log, initial_source_pos);
         fprintf(file, "SOURCE_AUTO_POSITION = 1\n");
-        fprintf(file, "#SOURCE_LOG_FILE = \"%s\"\n#SOURCE_LOG_POS = %s\n", masterlog, masterpos);
       }else{
-        fprintf(file, "SOURCE_LOG_FILE = \"%s\"\nSOURCE_LOG_POS = %s\n", masterlog, masterpos);
+        fprintf(file, "SOURCE_LOG_FILE = \"%s\"\nSOURCE_LOG_POS = %s\n", initial_source_log, initial_source_pos);
         fprintf(file, "#SOURCE_AUTO_POSITION = {0|1}\n");
       }
       fprintf(file, "myloader_exec_reset_replica = %d\nmyloader_exec_change_source = %d\nmyloader_exec_start_replica = %d\n",
-          ((source_data) & (1<<(0))) , ((source_data) & (1<<(1)))>0?1:0, ((source_data) & (1<<(2)))>0?1:0);
-    }else{
-      fprintf(file, "File = %s\nPosition = %s\nExecuted_Gtid_Set = %s\n",
-					masterlog, masterpos, mastergtid);
+          source_data.exec_reset_replica?1:0 , source_data.exec_change_source?1:0, source_data.exec_start_replica?1:0);
 		}
     g_message("Written master status");
   }
 
-  g_free(masterlog);
-  g_free(masterpos);
-  g_free(mastergtid);
   fflush(file);
 }
 
@@ -662,11 +553,27 @@ void write_replica_info(MYSQL *conn, FILE *file) {
     }
     if (slavehost) {
       slave_count++;
-      fprintf(file, "[replication%s%s]", channel_name!=NULL?".":"", channel_name!=NULL?channel_name:"");
-      fprintf(file, "\n# relay_master_log_file = \'%s\'\n# exec_master_log_pos = %s\n# %s = %s\n",
-              slavelog, slavepos, gtid_title, slavegtid);
+      fprintf(file, "\n[replication%s%s]\n", channel_name!=NULL?".":"", channel_name!=NULL?channel_name:"");
+
+      if (slavegtid && strlen(slavegtid)>0)
+        fprintf(file, "%s = \"%s\"\n", gtid_title, slavegtid);
+
+      if (replica_data.auto_position){
+        fprintf(file, "#SOURCE_LOG_FILE = \"%s\"\n#SOURCE_LOG_POS = %s\n", slavelog, slavepos);
+        fprintf(file, "SOURCE_AUTO_POSITION = 1\n");
+      }else{
+        fprintf(file, "SOURCE_LOG_FILE = \"%s\"\nSOURCE_LOG_POS = %s\n", slavelog, slavepos);
+        fprintf(file, "#SOURCE_AUTO_POSITION = {0|1}\n");
+      }
+
       fprintf(file,"%s",replication_section_str->str);
-      fprintf(file,"# myloader_exec_reset_slave = 0 # 1 means execute the command\n# myloader_exec_change_master = 0 # 1 means execute the command\n# myloader_exec_start_slave = 0 # 1 means execute the command\n");
+      if (replica_data.source_ssl)
+        fprintf(file, "SOURCE_SSL = 1\n");
+      else
+        fprintf(file, "#SOURCE_SSL = {0|1}\n");
+
+      fprintf(file, "myloader_exec_reset_replica = %d\nmyloader_exec_change_source = %d\nmyloader_exec_start_replica = %d\n",
+          replica_data.exec_reset_replica?1:0 , replica_data.exec_change_source?1:0, replica_data.exec_start_replica?1:0);
       g_message("Written slave status");
     }
   }
@@ -695,16 +602,13 @@ gboolean process_job_builder_job(struct thread_data *td, struct job *job){
     case JOB_TABLE:
       thd_JOB_TABLE(td, job);
       break;
-    case JOB_WRITE_MASTER_STATUS:
-      write_snapshot_info(td->thrconn, job->job_data);
+    case JOB_WRITE_SOURCE_AND_REPLICA_STATUS:
+//      if (source_data.enabled)
+        write_snapshot_info(td->thrconn, job->job_data);
       // Write replica information
-      if (get_product() != SERVER_TYPE_TIDB) {
-        if (source_data >=0 )
+      if ((get_product() != SERVER_TYPE_TIDB) && replica_data.enabled)
           write_replica_info(td->thrconn, job->job_data);
-      }
-      g_async_queue_push(td->conf->binlog_ready,GINT_TO_POINTER(1));
-      g_async_queue_push(td->conf->binlog_ready,GINT_TO_POINTER(1));
-      g_async_queue_push(td->conf->binlog_ready,GINT_TO_POINTER(1));
+      g_async_queue_push(td->conf->source_and_replica_status_queue,GINT_TO_POINTER(1));
       g_free(job);
       break;
     case JOB_SHUTDOWN:
@@ -758,17 +662,18 @@ gboolean process_job(struct thread_data *td, struct job *job){
     case JOB_SCHEMA_POST:
       do_JOB_SCHEMA_POST(td,job);
       break;
-    case JOB_WRITE_MASTER_STATUS:
+/*    case JOB_WRITE_MASTER_STATUS:
       write_snapshot_info(td->thrconn, job->job_data);
-      g_async_queue_push(td->conf->binlog_ready,GINT_TO_POINTER(1));
+      g_async_queue_push(td->conf->source_and_replica_status_queue,GINT_TO_POINTER(1));
       g_free(job);
       break;
+      */
     case JOB_SHUTDOWN:
       g_free(job);
       return FALSE;
       break;
     default:
-      m_error("Something very bad happened!");
+      m_error("Something very bad happened! %d", job->type);
     }
   return TRUE;
 }
@@ -836,9 +741,6 @@ void update_estimated_remaining_chunks_on_dbt(struct db_table *dbt){
       case INTEGER:
         total+=((union chunk_step *)(l->data))->integer_step.estimated_remaining_steps;
         break;
-      case CHAR:
-        total+=((union chunk_step *)(l->data))->char_step.estimated_remaining_steps;
-        break;
       default:
         total++;
         break; 
@@ -933,253 +835,6 @@ void *working_thread(struct thread_data *td) {
   return NULL;
 }
 
-static
-GString *get_selectable_fields(MYSQL *conn, char *database, char *table) {
-  MYSQL_ROW row;
-
-  GString *field_list = g_string_new("");
-
-  gchar *query =
-      g_strdup_printf("select COLUMN_NAME from information_schema.COLUMNS "
-                      "where TABLE_SCHEMA='%s' and TABLE_NAME='%s' and extra "
-                      "not like '%%VIRTUAL GENERATED%%' and extra not like '%%STORED GENERATED%%' ORDER BY ORDINAL_POSITION ASC",
-                      database, table);
-  MYSQL_RES *res=m_store_result_critical(conn, query, "Failed to get Selectable Fields", NULL);
-  g_free(query);
-
-  gboolean first = TRUE;
-  while ((row = mysql_fetch_row(res))) {
-    if (first) {
-      first = FALSE;
-    } else {
-      g_string_append(field_list, ",");
-    }
-
-    char *field_name= identifier_quote_character_protect(row[0]);
-    gchar *tb = g_strdup_printf("%s%s%s", identifier_quote_character_str, field_name, identifier_quote_character_str);
-    g_free(field_name);
-    g_string_append(field_list, tb);
-    g_free(tb);
-  }
-
-  mysql_free_result(res);
-
-  return field_list;
-}
-
-static
-gboolean has_json_fields(MYSQL *conn, char *database, char *table) {
-  gchar *query =
-      g_strdup_printf("select COLUMN_NAME from information_schema.COLUMNS "
-                      "where TABLE_SCHEMA='%s' and TABLE_NAME='%s' and "
-                      "COLUMN_TYPE ='json'",
-                      database, table);
-  struct M_ROW *mr = m_store_result_row(conn, query, m_critical, m_warning, "Failed to get JSON fields on %s.%s: %s", database, table, query);
-  g_free(query);
-  if (mr->row){
-    m_store_result_row_free(mr);
-    return TRUE;
-  }
-  m_store_result_row_free(mr);
-  return FALSE;
-}
-
-static
-gboolean detect_generated_fields(MYSQL *conn, gchar *database, gchar* table) {
-  gboolean result = FALSE;
-  if (ignore_generated_fields)
-    return FALSE;
-
-  gchar *query = g_strdup_printf(
-      "SELECT COLUMN_NAME FROM information_schema.COLUMNS "
-      "WHERE TABLE_SCHEMA='%s' AND TABLE_NAME='%s' AND extra LIKE '%%GENERATED%%' AND extra NOT LIKE '%%DEFAULT_GENERATED%%' "
-      "LIMIT 1",
-      database, table);
-
-  struct M_ROW *mr = m_store_result_row(conn, query, m_warning, m_message, "Failed to detect generated fields", NULL);
-  g_free(query);
-  result=mr->row!=NULL;
-  m_store_result_row_free(mr);
-  return result;
-}
-
-static
-gchar *get_character_set_from_collation(MYSQL *conn, gchar *collation){
-  g_mutex_lock(character_set_hash_mutex);
-  gchar *character_set = g_hash_table_lookup(character_set_hash, collation);
-  if (character_set == NULL){
-    gchar *query =
-      g_strdup_printf("SELECT CHARACTER_SET_NAME FROM INFORMATION_SCHEMA.COLLATIONS "
-                      "WHERE collation_name='%s'",
-                      collation);
-    struct M_ROW *mr = m_store_result_row(conn, query, m_critical, m_warning, "Failed to get CHARACTER_SET from collation %s", collation);
-    g_free(query);
-    if (mr->row)
-      g_hash_table_insert(character_set_hash, g_strdup(collation), character_set=g_strdup(mr->row[0]));
-    m_store_result_row_free(mr);
-  }
-  g_mutex_unlock(character_set_hash_mutex);
-  return character_set;
-}
-
-static
-void get_primary_key_separated_by_comma(struct db_table * dbt) {
-  GString *field_list = g_string_new(""); 
-  GList *list=dbt->primary_key;
-  gboolean first = TRUE;
-  while (list){
-    if (first) {
-      first = FALSE;
-    }else{
-      g_string_append(field_list, ",");
-    }
-    char *field_name= identifier_quote_character_protect((char*) list->data);
-    gchar *tb = g_strdup_printf("%s%s%s", identifier_quote_character_str, field_name, identifier_quote_character_str);
-    g_free(field_name);
-    g_string_append(field_list, tb);
-    list=list->next;
-  }
-  if (field_list->len>0)
-    dbt->primary_key_separated_by_comma = g_string_free(field_list, FALSE); 
-}
-
-static
-gboolean new_db_table(struct db_table **d, MYSQL *conn, struct configuration *conf,
-                      struct database *database, char *table, char *table_collation,
-                      gboolean is_sequence)
-{
-  gchar * lkey = build_dbt_key(database->name,table);
-	g_mutex_lock(all_dbts_mutex);
-  struct db_table *dbt = g_hash_table_lookup(all_dbts, lkey);
-  gboolean b;
-  if (dbt){
-    g_free(lkey);
-    b=FALSE;
-    g_mutex_unlock(all_dbts_mutex);
-  }else{
-    dbt = g_new(struct db_table, 1);
-		dbt->key=lkey;
-    dbt->status = UNDEFINED;
-    g_hash_table_insert(all_dbts, lkey, dbt);
-    g_mutex_unlock(all_dbts_mutex);
-    dbt->database = database;
-    dbt->table = identifier_quote_character_protect(table);
-    dbt->table_filename = get_ref_table(dbt->table);
-    dbt->is_sequence= is_sequence;
-    if (table_collation==NULL)
-      dbt->character_set = NULL;
-    else{
-      dbt->character_set=get_character_set_from_collation(conn, table_collation);
-      if ( dbt->character_set == NULL)
-        g_warning("Collation '%s' not found on INFORMATION_SCHEMA.COLLATIONS used by `%s`.`%s`",table_collation,database->name,table);
-    }
-    dbt->has_json_fields = has_json_fields(conn, dbt->database->name, dbt->table);
-    dbt->rows_lock= g_mutex_new();
-    dbt->escaped_table = escape_string(conn,dbt->table);
-    dbt->where=g_hash_table_lookup(conf_per_table.all_where_per_table, lkey);
-    dbt->limit=g_hash_table_lookup(conf_per_table.all_limit_per_table, lkey);
-    parse_object_to_export(&(dbt->object_to_export),g_hash_table_lookup(conf_per_table.all_object_to_export, lkey));
-
-    dbt->partition_regex=g_hash_table_lookup(conf_per_table.all_partition_regex_per_table, lkey);
-    dbt->max_threads_per_table=max_threads_per_table;
-    dbt->current_threads_running=0;
-
-    // Load chunk step size values
-    gchar *rows_p_chunk=g_hash_table_lookup(conf_per_table.all_rows_per_table, lkey);
-    if (rows_p_chunk )
-      dbt->split_integer_tables=parse_rows_per_chunk(rows_p_chunk, &(dbt->min_chunk_step_size), &(dbt->starting_chunk_step_size), &(dbt->max_chunk_step_size),"Invalid option on rows in configuration file");
-    else{
-      dbt->split_integer_tables=split_integer_tables;
-      dbt->min_chunk_step_size=min_chunk_step_size;
-      dbt->starting_chunk_step_size=starting_chunk_step_size;
-      dbt->max_chunk_step_size=max_chunk_step_size;
-    }
-    if (dbt->min_chunk_step_size == 1 && dbt->min_chunk_step_size == dbt->starting_chunk_step_size && dbt->starting_chunk_step_size != dbt->max_chunk_step_size ){
-      dbt->min_chunk_step_size = 2;
-      dbt->starting_chunk_step_size = 2;
-      g_warning("Setting min and start rows per file to 2 on %s", lkey);
-    }
-    dbt->is_fixed_length=dbt->min_chunk_step_size != 0 && dbt->min_chunk_step_size == dbt->starting_chunk_step_size && dbt->starting_chunk_step_size == dbt->max_chunk_step_size;
-    // We are disabling the file size limit when the user set a specific --rows size
-    dbt->chunk_filesize=dbt->is_fixed_length?0:chunk_filesize;
-
-    if ( dbt->min_chunk_step_size==0)
-      dbt->min_chunk_step_size=MIN_CHUNK_STEP_SIZE;
-
-    // honor --rows-hard
-    if ( max_integer_chunk_step_size != 0 && (dbt->max_chunk_step_size > max_integer_chunk_step_size || dbt->max_chunk_step_size == 0 ))
-      dbt->max_chunk_step_size=max_integer_chunk_step_size;
-    if ( min_integer_chunk_step_size != 0 && (dbt->min_chunk_step_size < min_integer_chunk_step_size ))
-      dbt->min_chunk_step_size=min_integer_chunk_step_size;
-
-
-    dbt->num_threads=g_hash_table_lookup(conf_per_table.all_num_threads_per_table, lkey)?strtoul(g_hash_table_lookup(conf_per_table.all_num_threads_per_table, lkey), NULL, 10):num_threads;
-    dbt->estimated_remaining_steps=1;
-    dbt->min=NULL;
-    dbt->max=NULL;
-    dbt->chunks=NULL;
-    dbt->load_data_header=NULL;
-    dbt->load_data_suffix=NULL;
-    dbt->insert_statement=NULL;
-    dbt->chunks_mutex=g_mutex_new();
-    dbt->chunks_queue=g_async_queue_new();
-    dbt->chunks_completed=g_new(int,1);
-    *(dbt->chunks_completed)=0;
-    get_primary_key(conn,dbt,conf);
-    dbt->primary_key_separated_by_comma = NULL;
-    if (order_by_primary_key)
-      get_primary_key_separated_by_comma(dbt);
-    dbt->multicolumn = !use_single_column && g_list_length(dbt->primary_key) > 1;
-
-    gchar *columns_on_select=g_hash_table_lookup(conf_per_table.all_columns_on_select_per_table, lkey);
-
-    dbt->columns_on_insert=g_hash_table_lookup(conf_per_table.all_columns_on_insert_per_table, lkey);
-
-    dbt->select_fields=NULL;
-
-    if (columns_on_select){
-      dbt->select_fields=g_string_new(columns_on_select);
-    }else if (!dbt->columns_on_insert){
-      dbt->complete_insert = complete_insert || detect_generated_fields(conn, dbt->database->escaped, dbt->escaped_table);
-      if (dbt->complete_insert) {
-        dbt->select_fields = get_selectable_fields(conn, dbt->database->escaped, dbt->escaped_table);
-      }
-    }
-
-//    dbt->anonymized_function=get_anonymized_function_for(conn, dbt);
-    dbt->anonymized_function=NULL;
-    dbt->indexes_checksum=NULL;
-    dbt->data_checksum=NULL;
-    dbt->schema_checksum=NULL;
-    dbt->triggers_checksum=NULL;
-    dbt->rows=0;
- // dbt->chunk_functions.process=NULL;
-    b=TRUE;
-  }
-  *d=dbt;
-  return b; 
-}
-
-void free_db_table(struct db_table * dbt){
-  g_mutex_lock(dbt->chunks_mutex);
-  g_mutex_free(dbt->rows_lock);
-  g_free(dbt->escaped_table);
-  if (dbt->insert_statement)
-    g_string_free(dbt->insert_statement,TRUE);
-  if (dbt->select_fields)
-    g_string_free(dbt->select_fields, TRUE);
-  if (dbt->min!=NULL) g_free(dbt->min);
-  if (dbt->max!=NULL) g_free(dbt->max);
-  g_free(dbt->data_checksum);
-  dbt->data_checksum=NULL;
-  g_free(dbt->chunks_completed);
-
-  g_free(dbt->table);
-  g_mutex_unlock(dbt->chunks_mutex);
-  g_mutex_free(dbt->chunks_mutex);
-  g_free(dbt);
-}
-
 static
 void new_table_to_dump(MYSQL *conn, struct configuration *conf, gboolean is_view,
                        gboolean is_sequence, struct database * database, char *table,
@@ -1352,7 +1007,7 @@ void dump_database_thread(MYSQL *conn, struct configuration *conf, struct databa
       g_warning("Broken table detected, please review: %s.%s", database->name,
                 row[0]);
       if (exit_if_broken_table_found)
-        exit(EXIT_FAILURE);
+        m_error("Broken table detected");
       dump = 0;
     }
 
@@ -1416,13 +1071,8 @@ void dump_database_thread(MYSQL *conn, struct configuration *conf, struct databa
 
   mysql_free_result(result);
 
-  if (determine_if_schema_is_elected_to_dump_post(conn,database)) {
+  if (determine_if_schema_is_elected_to_dump_post(conn,database))
     create_job_to_dump_post(database, conf);
-//    struct schema_post *sp = g_new(struct schema_post, 1);
-//    sp->database = database;
-//    schema_post = g_list_prepend(schema_post, sp);
-  }
-
 
   if (dump_triggers && database->dump_triggers)
     create_job_to_dump_schema_triggers(database, conf);
