@@ -23,6 +23,31 @@
 #define INSERT "INSERT"
 #define REPLACE "REPLACE"
 #define UNLOCK_TABLES "UNLOCK TABLES"
+
+
+#ifndef _src_mydumper_working_thread_h
+#define _src_mydumper_working_thread_h
+
+struct thread_data_buffers {
+  GString *statement;
+  GString *row;
+  GString *escaped;
+  GString *column;
+};
+
+struct thread_data {
+  struct configuration *conf;
+  guint thread_id;
+  char *table_name;
+  MYSQL *thrconn;
+  gchar *binlog_snapshot_gtid_executed;
+  GMutex *pause_resume_mutex;
+  struct thread_data_buffers thread_data_buffers;
+};
+
+#endif
+
+
 typedef gchar * (*fun_ptr2)(gchar **);
 
 void initialize_working_thread();
@@ -36,3 +61,4 @@ void build_lock_tables_statement(struct configuration *conf);
 void check_pause_resume( struct thread_data *td );
 void update_estimated_remaining_chunks_on_dbt(struct db_table *dbt);
 void free_db_table(struct db_table * dbt);
+void get_binlog_position(MYSQL *conn, char **masterlog, char **masterpos, char **mastergtid);
