@@ -47,6 +47,8 @@ gchar *where_option=NULL;
 
 const gchar *insert_statement=INSERT;
 guint statement_size = 1000000;
+guint64 max_statement_size=0;
+GMutex *max_statement_size_mutex=NULL;
 guint complete_insert = 0;
 guint chunk_filesize = 0;
 gboolean load_data = FALSE;
@@ -72,13 +74,15 @@ gboolean hex_blob = FALSE;
 
 gboolean update_files_on_table_job(struct table_job *tj)
 {
-  if (tj->rows->file == 0){
+  if (tj->rows->file < 0){
     tj->rows->filename = build_rows_filename(tj->dbt->database->filename, tj->dbt->table_filename, tj->part, tj->sub_part);
     tj->rows->file = m_open(&(tj->rows->filename),"w");
+    trace("Thread %d: Filename assigned(%d): %s", tj->td->thread_id, tj->rows->file, tj->rows->filename);
 
     if (tj->sql){
       tj->sql->filename =build_sql_filename(tj->dbt->database->filename, tj->dbt->table_filename, tj->part, tj->sub_part);
       tj->sql->file = m_open(&(tj->sql->filename),"w");
+      trace("Thread %d: Filename assigned: %s", tj->td->thread_id, tj->sql->filename);
       return TRUE;
     }
   }
@@ -87,19 +91,25 @@ gboolean update_files_on_table_job(struct table_job *tj)
 
 void message_dumping_data_short(struct table_job *tj){
   g_mutex_lock(transactional_table->mutex);
+  guint transactional_table_size = g_list_length(transactional_table->list);
+  g_mutex_unlock(transactional_table->mutex);
   g_mutex_lock(non_transactional_table->mutex);
+  guint non_transactional_table_size = g_list_length(non_transactional_table->list);
+  g_mutex_unlock(non_transactional_table->mutex);
   g_message("Thread %d: %s%s%s.%s%s%s [ %"G_GINT64_FORMAT"%% ] | Tables: %u/%u",
                     tj->td->thread_id,
                     identifier_quote_character_str, masquerade_filename?tj->dbt->database->filename:tj->dbt->database->name, identifier_quote_character_str,
                     identifier_quote_character_str, masquerade_filename?tj->dbt->table_filename:tj->dbt->table, identifier_quote_character_str,
-                    tj->dbt->rows_total!=0?100*tj->dbt->rows/tj->dbt->rows_total:0, g_list_length(transactional_table->list)+g_list_length(non_transactional_table->list),g_hash_table_size(all_dbts));
-  g_mutex_unlock(transactional_table->mutex);
-  g_mutex_unlock(non_transactional_table->mutex);
+                    tj->dbt->rows_total!=0?100*tj->dbt->rows/tj->dbt->rows_total:0, non_transactional_table_size+transactional_table_size, g_hash_table_size(all_dbts));
 }
 
 void message_dumping_data_long(struct table_job *tj){
   g_mutex_lock(transactional_table->mutex);
+  guint transactional_table_size = g_list_length(transactional_table->list);
+  g_mutex_unlock(transactional_table->mutex);
   g_mutex_lock(non_transactional_table->mutex);
+  guint non_transactional_table_size = g_list_length(non_transactional_table->list);
+  g_mutex_unlock(non_transactional_table->mutex);
   g_message("Thread %d: dumping data from %s%s%s.%s%s%s%s%s%s%s%s%s%s%s%s%s into %s | Completed: %"G_GINT64_FORMAT"%% | Remaining tables: %u / %u",
                     tj->td->thread_id,
                     identifier_quote_character_str, masquerade_filename?tj->dbt->database->filename:tj->dbt->database->name, identifier_quote_character_str, 
@@ -109,9 +119,7 @@ void message_dumping_data_long(struct table_job *tj){
                      (tj->where->len && where_option )                    ? " AND "   : "" ,   where_option ?   where_option : "",
                     ((tj->where->len || where_option ) && tj->dbt->where) ? " AND "   : "" , tj->dbt->where ? tj->dbt->where : "",
                     order_by_primary_key && tj->dbt->primary_key_separated_by_comma ? " ORDER BY " : "", order_by_primary_key && tj->dbt->primary_key_separated_by_comma ? tj->dbt->primary_key_separated_by_comma : "",
-                    tj->rows->filename, tj->dbt->rows_total!=0?100*tj->dbt->rows/tj->dbt->rows_total:0, g_list_length(transactional_table->list)+g_list_length(non_transactional_table->list),g_hash_table_size(all_dbts));
-  g_mutex_unlock(transactional_table->mutex);
-  g_mutex_unlock(non_transactional_table->mutex);
+                    tj->rows->filename, tj->dbt->rows_total!=0?100*tj->dbt->rows/tj->dbt->rows_total:0, non_transactional_table_size+transactional_table_size,g_hash_table_size(all_dbts));
 }
 
 void (*message_dumping_data)(struct table_job *tj);
@@ -136,6 +144,7 @@ void initialize_write(){
   if(fields_escaped_by && strlen(fields_escaped_by)>1)
     m_critical("--fields-escaped-by must be a single character");
 
+  max_statement_size_mutex=g_mutex_new();
 
   switch (output_format){
 		case CLICKHOUSE:
@@ -271,6 +280,9 @@ void finalize_write(){
   g_free(statement_terminated_by);
 }
 
+gboolean is_hex_blob (MYSQL_FIELD field){
+  return hex_blob && (field.type == MYSQL_TYPE_BLOB || ( field.charsetnr== 63 && (field.type == MYSQL_TYPE_VAR_STRING || field.type == MYSQL_TYPE_STRING )));
+}
 
 GString *append_load_data_columns(GString *statement, MYSQL_FIELD *fields, guint num_fields){
   guint i = 0;
@@ -292,7 +304,7 @@ GString *append_load_data_columns(GString *statement, MYSQL_FIELD *fields, guint
       g_string_append(str, fields[i].name);
       g_string_append(str, " USING UTF8MB4)");
       appendable=TRUE;
-    }else if (hex_blob && fields[i].type == MYSQL_TYPE_BLOB){
+    }else if ( is_hex_blob(fields[i])){
       g_string_append_c(statement,'@');
       g_string_append(statement, fields[i].name);
       if (str->len > 4)
@@ -385,9 +397,9 @@ gboolean real_write_data(int file, float *filesize, GString *data) {
   ssize_t r = 0;
   gboolean second_write_zero = FALSE;
   while (written < data->len) {
-    r=write(file, data->str + written, data->len);
+    r=write(file, data->str + written, data->len - written);
     if (r < 0) {
-      g_critical("Couldn't write data to a file: %s", strerror(errno));
+      g_critical("Couldn't write data to a file(%d): %s", file, strerror(errno));
       errors++;
       return FALSE;
     }
@@ -414,7 +426,7 @@ gboolean write_data(int file, GString *data) {
 }
 
 void initialize_load_data_statement_suffix(struct db_table *dbt, MYSQL_FIELD * fields, guint num_fields){
-  gchar *character_set=set_names_str != NULL ? set_names_str : dbt->character_set /* "BINARY"*/;
+  gchar *character_set=set_names_in_conn_by_default != NULL ? set_names_in_conn_by_default : dbt->character_set /* "BINARY"*/;
   GString *load_data_suffix=g_string_sized_new(statement_size);
   g_string_append_printf(load_data_suffix, "%s' INTO TABLE %s%s%s ", exec_per_thread_extension, identifier_quote_character_str, dbt->table, identifier_quote_character_str);
   if (character_set && strlen(character_set)!=0)
@@ -448,7 +460,7 @@ void initialize_load_data_statement_suffix(struct db_table *dbt, MYSQL_FIELD * f
 }
 
 void initialize_clickhouse_statement_suffix(struct db_table *dbt, MYSQL_FIELD * fields, guint num_fields){
-  gchar *character_set=set_names_str != NULL ? set_names_str : dbt->character_set /* "BINARY"*/;
+  gchar *character_set=set_names_in_conn_by_default != NULL ? set_names_in_conn_by_default : dbt->character_set /* "BINARY"*/;
   dbt->load_data_suffix=g_string_sized_new(statement_size);
   g_string_append_printf(dbt->load_data_suffix, "%s' INTO TABLE %s%s%s ", exec_per_thread_extension, identifier_quote_character_str, dbt->table, identifier_quote_character_str);
   if (character_set && strlen(character_set)!=0)
@@ -495,15 +507,26 @@ void initialize_load_data_header(struct db_table *dbt, MYSQL_FIELD *fields, guin
   g_string_append(dbt->load_data_header,lines_terminated_by);
 }
 
+static
 gboolean write_statement(int load_data_file, float *filessize, GString *statement, struct db_table * dbt){
   if (!real_write_data(load_data_file, filessize, statement)) {
     g_critical("Could not write out data for %s.%s", dbt->database->name, dbt->table);
     return FALSE;
   }
+  g_mutex_lock(max_statement_size_mutex);
+  if (statement->len > max_statement_size)
+    max_statement_size=statement->len;
+  g_mutex_unlock(max_statement_size_mutex);
   g_string_set_size(statement, 0);
   return TRUE;
 }
 
+void initialize_config_on_string(GString *output){
+  g_mutex_lock(max_statement_size_mutex);
+  g_string_append_printf(output,"[config]\nmax-statement-size = %ld\n", max_statement_size);
+  g_mutex_unlock(max_statement_size_mutex);
+}
+
 void write_load_data_statement(struct table_job * tj){
   GString *statement = g_string_sized_new(statement_size);
   char * basename=g_path_get_basename(tj->rows->filename);
@@ -532,38 +555,10 @@ gboolean write_header(struct table_job * tj){
   return TRUE;
 }
 
-/*
-guint64 get_estimated_remaining_chunks_on_dbt(struct db_table *dbt){
-  GList *l=dbt->chunks;
-  guint64 total=0;
-  while (l!=NULL){
-    total+=((union chunk_step *)(l->data))->integer_step.estimated_remaining_steps;
-    l=l->next;
-  }
-  return total;
-}
-*/
-
-guint64 get_estimated_remaining_of(struct MList *mlist){
-  g_mutex_lock(mlist->mutex);
-  GList *tl=mlist->list;
-  guint64 total=0;
-  while (tl!=NULL){
-    total+=((struct db_table *)(tl->data))->estimated_remaining_steps;
-    tl=tl->next;
-  }
-  g_mutex_unlock(mlist->mutex);
-  return total;
-}
-
-guint64 get_estimated_remaining_of_all_chunks(){
-  return get_estimated_remaining_of(non_transactional_table) + get_estimated_remaining_of(transactional_table);
-}
-
 void write_load_data_column_into_string( MYSQL *conn, gchar **column, MYSQL_FIELD field, gulong length, struct thread_data_buffers buffers){
     if (!*column) {
       g_string_append(buffers.column, "\\N");
-    } else if ( field.type == MYSQL_TYPE_BLOB && hex_blob ) {
+    } else if ( is_hex_blob(field) ) {
       g_string_set_size(buffers.escaped, length * 2 + 1);
       mysql_hex_string(buffers.escaped->str,*column,length);
       g_string_append(buffers.column,buffers.escaped->str);
@@ -590,7 +585,7 @@ void write_sql_column_into_string( MYSQL *conn, gchar **column, MYSQL_FIELD fiel
     } else if ( length == 0){
       g_string_append_c(buffers.column,*fields_enclosed_by);
       g_string_append_c(buffers.column,*fields_enclosed_by);
-    } else if ( field.type == MYSQL_TYPE_BLOB && hex_blob ) {
+    } else if ( is_hex_blob(field) ) {
       g_string_set_size(buffers.escaped, length * 2 + 1);
       g_string_append(buffers.column,"0x");
       mysql_hex_string(buffers.escaped->str,*column,length);
@@ -620,6 +615,7 @@ void write_column_into_string_with_terminated_by(MYSQL *conn, gchar * row, MYSQL
     column=row;
   if (f){
    if (f->is_pre){
+     // apply and constant as they alter the data
      write_column_into_string( conn, &(column), field, rlength, buffers);
      column=f->function(&(buffers.column->str), &rlength, f);
      g_string_printf(buffers.column,"%s",column);
@@ -655,9 +651,9 @@ void update_dbt_rows(struct db_table * dbt, guint64 num_rows){
 }
 
 void close_file(struct table_job * tj, struct table_job_file *tjf){
-  if (tjf->file!=0){
+  if (tjf->file >= 0){
     m_close(tj->td->thread_id, tjf->file, tjf->filename, 1, tj->dbt);
-    tjf->file=0;
+    tjf->file=-1;
     g_free(tjf->filename);
     tjf->filename=NULL;
   }
@@ -676,6 +672,7 @@ void close_files(struct table_job * tj){
   close_file(tj, tj->rows);
 }
 
+static
 void reopen_files(struct table_job * tj){
   close_files(tj);
   switch (output_format){
@@ -730,7 +727,7 @@ void write_result_into_file(MYSQL *conn, MYSQL_RES *result, struct table_job * t
       }
 	  	break;
     case CLICKHOUSE:
-      if (tj->rows->file == 0){
+      if (tj->rows->file < 0){
         update_files_on_table_job(tj);
       }
 			if (dbt->load_data_suffix==NULL){
@@ -752,7 +749,7 @@ void write_result_into_file(MYSQL *conn, MYSQL_RES *result, struct table_job * t
       g_string_append(tj->td->thread_data_buffers.statement, dbt->insert_statement->str);
       break;
 		case SQL_INSERT:
-      if (tj->rows->file == 0){
+      if (tj->rows->file < 0){
         update_files_on_table_job(tj);
   		}
       if (dbt->insert_statement==NULL){
@@ -770,7 +767,11 @@ void write_result_into_file(MYSQL *conn, MYSQL_RES *result, struct table_job * t
   message_dumping_data(tj);
 
   GDateTime *from = g_date_time_new_now_local();
+  GDateTime *to = NULL;
+  GTimeSpan diff=0;
 	while ((row = mysql_fetch_row(result))) {
+// Uncomment next line if you need to simulate a slow read which is useful when calculate the chunk size
+//    g_usleep(1);
     lengths = mysql_fetch_lengths(result);
     num_rows++;
     // prepare row into statement_row
@@ -786,9 +787,11 @@ void write_result_into_file(MYSQL *conn, MYSQL_RES *result, struct table_job * t
       }
       g_string_append(tj->td->thread_data_buffers.statement, statement_terminated_by);
       if (!write_statement(tj->rows->file, &(tj->filesize), tj->td->thread_data_buffers.statement, dbt)) {
+        g_critical("Fail to write on %s", tj->rows->filename);
         return;
       }
 			update_dbt_rows(dbt, num_rows);
+      tj->num_rows_of_last_run+=num_rows;
 			num_rows=0;
 			num_rows_st=0;
 			tj->st_in_file++;
@@ -796,12 +799,15 @@ void write_result_into_file(MYSQL *conn, MYSQL_RES *result, struct table_job * t
       if (output_format == SQL_INSERT || output_format == CLICKHOUSE){
 				g_string_append(tj->td->thread_data_buffers.statement, dbt->insert_statement->str);
 			}
-      GDateTime *to = g_date_time_new_now_local();
-      GTimeSpan diff=g_date_time_difference(to,from)/G_TIME_SPAN_SECOND;
+      to = g_date_time_new_now_local();
+      diff=g_date_time_difference(to,from)/G_TIME_SPAN_SECOND;
       if (diff > 4){
         g_date_time_unref(from);
         from=to;
+        to=NULL;
         message_dumping_data(tj);
+      }else{
+        g_date_time_unref(to);
       }
 
 			check_pause_resume(tj->td);
@@ -831,15 +837,18 @@ void write_result_into_file(MYSQL *conn, MYSQL_RES *result, struct table_job * t
     g_string_set_size(tj->td->thread_data_buffers.row, 0);
   }
   update_dbt_rows(dbt, num_rows);
+  tj->num_rows_of_last_run+=num_rows;
   if (num_rows_st > 0 && tj->td->thread_data_buffers.statement->len > 0){
     if (output_format == SQL_INSERT || output_format == CLICKHOUSE)
 			g_string_append(tj->td->thread_data_buffers.statement, statement_terminated_by);
     if (!write_statement(tj->rows->file, &(tj->filesize), tj->td->thread_data_buffers.statement, dbt)) {
+      g_critical("Fail to write on %s", tj->rows->filename);
       return;
     }
 		tj->st_in_file++;
   }
-  tj->num_rows_of_last_run=num_rows;
+  g_date_time_unref(from);
+
 //  g_string_free(statement, TRUE);
 //  g_string_free(escaped, TRUE);
 //  g_string_free(statement_row, TRUE);	
@@ -854,6 +863,8 @@ void write_table_job_into_file(struct table_job * tj){
 //  if (throttle_time)
   g_usleep(throttle_time);
 
+  tj->num_rows_of_last_run=0;
+
   /* Ghm, not sure if this should be statement_size - but default isn't too big
    * for now */
   /* Poor man's database code */
@@ -881,29 +892,10 @@ void write_table_job_into_file(struct table_job * tj){
       g_warning("Thread %d: Retrying last failed executed statement", tj->td->thread_id);
 
       result = m_use_result(conn, query, NULL, "Failed to execute query on second try", NULL);
-      if (!result) {
-       // ERROR 1146
-        if (success_on_1146 && mysql_errno(conn) == 1146) {
-          g_warning("Thread %d: Error dumping table (%s.%s) data: %s\nQuery: %s", tj->td->thread_id, tj->dbt->database->name, tj->dbt->table,
-                  mysql_error(conn), query);
-        } else {
-          g_critical("Thread %d: Error dumping table (%s.%s) data: %s\nQuery: %s ", tj->td->thread_id, tj->dbt->database->name, tj->dbt->table,
-                     mysql_error(conn), query);
-          errors++;
-        }
+      if (!result) 
         goto cleanup;
-      }
-    }else{
-      if (success_on_1146 && mysql_errno(conn) == 1146) {
-        g_warning("Thread %d: Error dumping table (%s.%s) data: %s\nQuery: %s", tj->td->thread_id, tj->dbt->database->name, tj->dbt->table,
-                  mysql_error(conn), query);
-      } else {
-        g_critical("Thread %d: Error dumping table (%s.%s) data: %s\nQuery: %s ", tj->td->thread_id, tj->dbt->database->name, tj->dbt->table,
-                   mysql_error(conn), query);
-        errors++;
-      }
+    }else
       goto cleanup;
-    }
   }
 
   /* Poor man's data dump code */
