@@ -19,10 +19,16 @@
                     David Ducos, Percona (david dot ducos at percona dot com)
 */
 #define LOAD_DATA_PREFIX "LOAD DATA LOCAL INFILE '" 
+
+#ifndef _src_mydumper_write_h
+#define _src_mydumper_write_h
+#include "mydumper_create_jobs.h"
+#endif
+
 void load_write_entries(GOptionGroup *main_group, GOptionContext *context);
 void initialize_write();
+void initialize_config_on_string(GString *output);
 void finalize_write();
 void write_table_job_into_file(struct table_job *tj);
 gboolean write_data(int file, GString *data);
-void reopen_files(struct table_job * tj);
 void close_files(struct table_job * tj);
