@@ -34,7 +34,7 @@
 #include "myloader_common.h"
 #include "myloader_directory.h"
 #include "myloader_restore.h"
-#include "myloader_pmm_thread.h"
+#include "myloader_pmm.h"
 #include "myloader_restore_job.h"
 #include "myloader_intermediate_queue.h"
 #include "myloader_arguments.h"
@@ -70,10 +70,9 @@ guint sequences = 0;
 guint sequences_processed = 0;
 GMutex sequences_mutex;
 gchar *source_db = NULL;
-gchar *purge_mode_str=NULL;
 extern guint errors;
 guint max_errors= 0;
-struct restore_errors detailed_errors = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
+struct restore_errors detailed_errors = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
 guint max_threads_for_schema_creation=4;
 guint max_threads_for_index_creation=4;
 guint max_threads_for_post_creation= 1;
@@ -81,25 +80,17 @@ guint retry_count= 10;
 gboolean stream = FALSE;
 gboolean no_delete = FALSE;
 
-GMutex *load_data_list_mutex=NULL;
-GHashTable * load_data_list = NULL;
-//unsigned long long int total_data_sql_files = 0;
-//unsigned long long int progress = 0;
-//GHashTable *db_hash=NULL;
 extern GHashTable *db_hash;
 extern gboolean shutdown_triggered;
 extern gboolean skip_definer;
 extern gboolean local_infile;
+extern guint64 max_transaction_size;
 
 const char DIRECTORY[] = "import";
 
 struct configuration_per_table conf_per_table = {NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL};
 GHashTable * set_session_hash=NULL;
 
-gchar *pmm_resolution = NULL;
-gchar *pmm_path = NULL;
-gboolean pmm = FALSE;
-
 GHashTable * myloader_initialize_hash_of_session_variables(){
   GHashTable * _set_session_hash=initialize_hash_of_session_variables();
   if (commit_count > 1)
@@ -109,6 +100,15 @@ GHashTable * myloader_initialize_hash_of_session_variables(){
   return _set_session_hash;
 }
 
+static
+void detect_group_replication_transaction_size_limit(MYSQL * conn) {
+  guint64 _max_transaction_size=0;
+  struct M_ROW *mr = m_store_result_row(conn, "SELECT @@group_replication_transaction_size_limit / 1024 / 1024",m_message, m_message, "Using default transaction limit", NULL);
+  if (mr->row)
+    _max_transaction_size=strtoll(mr->row[0], NULL, 10);
+  max_transaction_size=_max_transaction_size > max_transaction_size ? _max_transaction_size : max_transaction_size;
+  m_store_result_row_free(mr);
+}
 
 gchar * print_time(GTimeSpan timespan){
   GTimeSpan days   = timespan/G_TIME_SPAN_DAY;
@@ -126,6 +126,48 @@ gint compare_by_time(gconstpointer a, gconstpointer b){
 }
 
 
+void initialize_directories(){
+  char *current_dir=g_get_current_dir();
+  if (!input_directory) {
+    if (stream){
+      GDateTime * datetime = g_date_time_new_now_local();
+      char *datetimestr;
+      datetimestr=g_date_time_format(datetime,"\%Y\%m\%d-\%H\%M\%S");
+
+      directory = g_strdup_printf("%s/%s-%s",current_dir, DIRECTORY, datetimestr);
+      g_date_time_unref(datetime);
+      g_free(datetimestr);
+    }else{
+      if(!help)
+        m_critical("a directory needs to be specified, see --help\n");
+    }
+  } else {
+    directory=g_str_has_prefix(input_directory,"/")?input_directory:g_strdup_printf("%s/%s", current_dir, input_directory);
+    if (stream){
+      if (g_file_test(input_directory,G_FILE_TEST_IS_DIR) && !no_stream)
+          m_critical("Backup directory (-d) must not exist when --stream / --stream=TRADITIONAL");
+    }else{
+      if (!g_file_test(input_directory,G_FILE_TEST_IS_DIR))
+        m_critical("the specified directory doesn't exists\n");
+      char *p = g_strdup_printf("%s/metadata", directory);
+      if (!g_file_test(p, G_FILE_TEST_EXISTS)) {
+        m_critical("the specified directory %s is not a mydumper backup as metadata file was not found in it",directory);
+      }
+    }
+  }
+
+  if (fifo_directory){
+    if (fifo_directory[0] != '/' ){
+      gchar *tmp_fifo_directory=fifo_directory;
+      fifo_directory=g_strdup_printf("%s/%s", current_dir, tmp_fifo_directory);
+    }
+  }else{
+    // Set fifo temporary director
+    fifo_directory=build_tmp_dir_name();
+  }
+  g_free(current_dir);
+}
+
 void show_dbt(void* _key, void* dbt, void *total){
   (void) dbt;
   (void) total;
@@ -143,11 +185,14 @@ void create_database(struct thread_data *td, gchar *database) {
   const gchar *filepath = g_strdup_printf("%s/%s-schema-create.sql%s",
                                             directory, database, exec_per_thread_extension);
 
+  if (drop_database)
+    execute_drop_database(td, database);
   if (g_file_test(filepath, G_FILE_TEST_EXISTS)) {
     g_atomic_int_add(&(detailed_errors.schema_errors), restore_data_from_mydumper_file(td, filename, TRUE, NULL));
   } else {
     GString *data = g_string_new("CREATE DATABASE IF NOT EXISTS ");
     g_string_append_printf(data,"`%s`", database);
+    trace("Creating schema %s", database);
     if (restore_data_in_gstring_extended(td, data , TRUE, NULL, m_critical, "Failed to create database: %s", database) )
       g_atomic_int_inc(&(detailed_errors.schema_errors));
     g_string_free(data, TRUE);
@@ -156,180 +201,7 @@ void create_database(struct thread_data *td, gchar *database) {
   return;
 }
 
-
-void print_errors(){
-  if (detailed_errors.tablespace_errors ||
-    detailed_errors.schema_errors ||
-    detailed_errors.data_errors ||
-    detailed_errors.view_errors ||
-    detailed_errors.sequence_errors ||
-    detailed_errors.index_errors ||
-    detailed_errors.trigger_errors ||
-    detailed_errors.constraints_errors ||
-    detailed_errors.post_errors ||
-    detailed_errors.retries)
-    g_message(
-    "Errors found:\n"
-    "- Tablespace:\t%d\n"
-    "- Schema:    \t%d\n"
-    "- Data:      \t%d\n"
-    "- View:      \t%d\n"
-    "- Sequence:  \t%d\n"
-    "- Index:     \t%d\n"
-    "- Trigger:   \t%d\n"
-    "- Constraint:\t%d\n"
-    "- Post:      \t%d\n"
-    "Retries:\t%d",
-    detailed_errors.tablespace_errors,
-    detailed_errors.schema_errors,
-    detailed_errors.data_errors,
-    detailed_errors.view_errors,
-    detailed_errors.sequence_errors,
-    detailed_errors.index_errors,
-    detailed_errors.trigger_errors,
-    detailed_errors.constraints_errors,
-    detailed_errors.post_errors,
-    detailed_errors.retries);
-
-
-}
-
-int main(int argc, char *argv[]) {
-  struct configuration conf = {NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, NULL};
-
-  GError *error = NULL;
-  GOptionContext *context;
-
-  source_data=-1;
-  setlocale(LC_ALL, "");
-  g_thread_init(NULL);
-  set_thread_name("MNT");
-
-  signal(SIGCHLD, SIG_IGN);
-
-  context = load_contex_entries();
-
-  gchar ** tmpargv=g_strdupv(argv);
-  int tmpargc=argc;
-  if (!g_option_context_parse(context, &tmpargc, &tmpargv, &error)) {
-    m_critical("option parsing failed: %s, try --help\n", error->message);
-  }
-  g_strfreev(tmpargv);
-
-  if (db == NULL && source_db != NULL) {
-    db = g_strdup(source_db);
-  }
-
-  if (help){
-    printf("%s", g_option_context_get_help (context, FALSE, NULL));
-  }
-
-  if (debug) {
-    set_debug();
-    set_verbose(3);
-  } else {
-    set_verbose(verbose);
-  }
-
-  if (overwrite_unsafe)
-    overwrite_tables= TRUE;
-
-  check_num_threads();
-
-  if (num_threads > max_threads_per_table)
-    g_message("Using %u loader threads (%u per table)", num_threads, max_threads_per_table);
-  else
-    g_message("Using %u loader threads", num_threads);
-
-  initialize_common_options(context, "myloader");
-//  g_option_context_free(context);
-  conf.context=context;  
-
-  hide_password(argc, argv);
-  ask_password();
-
-  if (program_version) {
-    print_version("myloader");
-    exit(EXIT_SUCCESS);
-  }
-
-  initialize_set_names();
-
-
-  load_data_list_mutex=g_mutex_new();
-  load_data_list = g_hash_table_new ( g_str_hash, g_str_equal );
-
-  if (pmm_path){
-    pmm=TRUE;
-    if (!pmm_resolution){
-      pmm_resolution=g_strdup("high");
-    }
-  }else if (pmm_resolution){
-    pmm=TRUE;
-    pmm_path=g_strdup_printf("/usr/local/percona/pmm2/collectors/textfile-collector/%s-resolution",pmm_resolution);
-  }
-
-  GThread *pmmthread = NULL;
-  if (pmm){
-
-    g_message("Using PMM resolution %s at %s", pmm_resolution, pmm_path);
-    pmmthread =
-        m_thread_new("myloader_pmm",pmm_thread, &conf, "PMM thread could not be created");
-    if (pmmthread == NULL) {
-      m_critical("Could not create pmm thread");
-    }
-  }
-//  initialize_job(purge_mode_str);
-  initialize_restore_job();
-  char *current_dir=g_get_current_dir();
-  if (!input_directory) {
-    if (stream){
-      GDateTime * datetime = g_date_time_new_now_local();
-      char *datetimestr;
-      datetimestr=g_date_time_format(datetime,"\%Y\%m\%d-\%H\%M\%S");
-
-      directory = g_strdup_printf("%s/%s-%s",current_dir, DIRECTORY, datetimestr);
-      create_dir(directory);
-      g_date_time_unref(datetime);
-      g_free(datetimestr); 
-    }else{
-      if(!help)
-        m_critical("a directory needs to be specified, see --help\n");
-    }
-  } else {
-    directory=g_str_has_prefix(input_directory,"/")?input_directory:g_strdup_printf("%s/%s", current_dir, input_directory);
-    if (stream){
-      if (!g_file_test(input_directory,G_FILE_TEST_IS_DIR)){
-        create_dir(directory);
-      }else{
-        if (!no_stream){
-          m_critical("Backup directory (-d) must not exist when --stream / --stream=TRADITIONAL");
-        }
-      }
-    }else{
-      if (!g_file_test(input_directory,G_FILE_TEST_IS_DIR)){
-        m_critical("the specified directory doesn't exists\n");
-      }
-      char *p = g_strdup_printf("%s/metadata", directory);
-      if (!g_file_test(p, G_FILE_TEST_EXISTS)) {
-        m_critical("the specified directory %s is not a mydumper backup as metadata file was not found in it",directory);
-      }
-//      initialize_directory();
-    }
-  }
-  if (fifo_directory){
-    if (fifo_directory[0] != '/' ){
-      gchar *tmp_fifo_directory=fifo_directory;
-      fifo_directory=g_strdup_printf("%s/%s", current_dir, tmp_fifo_directory);
-    }
-  }else{
-    // Create temporary director
-    fifo_directory=build_tmp_dir_name();
-  }
-  create_dir(fifo_directory);
-  g_message("Using %s as FIFO directory, please remove it if restoration fails", fifo_directory);
-
-  if (help){
+void print_help(){
     print_string("host", hostname);
     print_string("user", username);
     print_string("password", password);
@@ -376,7 +248,6 @@ int main(int argc, char *argv[]) {
     print_bool("no-schemas",no_schemas);
 
     print_bool("local-infile", local_infile);
-    print_string("purge-mode",purge_mode_str);
     print_bool("disable-redo-log",disable_redo_log);
     print_string("checksum",checksum_str);
     print_bool("overwrite-tables",overwrite_tables);
@@ -395,7 +266,7 @@ int main(int argc, char *argv[]) {
     print_int("rows",rows);
     print_int("queries-per-transaction",commit_count);
     print_bool("append-if-not-exist",append_if_not_exist);
-    print_string("set-names",set_names_str);
+    print_string("set-names",set_names_in_conn_by_default);
 
     print_bool("skip-definer",skip_definer);
     print_bool("help",help);
@@ -414,9 +285,148 @@ int main(int argc, char *argv[]) {
     print_string("defaults-extra-file",defaults_extra_file);
     print_string("fifodir",fifo_directory);
     exit(EXIT_SUCCESS);
+}
+
+
+void print_errors(){
+  if (detailed_errors.tablespace_errors ||
+    detailed_errors.schema_errors ||
+    detailed_errors.data_errors ||
+    detailed_errors.view_errors ||
+    detailed_errors.sequence_errors ||
+    detailed_errors.index_errors ||
+    detailed_errors.trigger_errors ||
+    detailed_errors.constraints_errors ||
+    detailed_errors.post_errors ||
+    detailed_errors.retries)
+    g_message(
+    "Errors found:\n"
+    "- Tablespace:\t%d\n"
+    "- Schema:    \t%d\n"
+    "- Data:      \t%d\n"
+    "- View:      \t%d\n"
+    "- Sequence:  \t%d\n"
+    "- Index:     \t%d\n"
+    "- Trigger:   \t%d\n"
+    "- Constraint:\t%d\n"
+    "- Post:      \t%d\n"
+    "Warnings found:\n"
+    "- Data:\t%d\n"
+    "Retries:\t%d",
+    detailed_errors.tablespace_errors,
+    detailed_errors.schema_errors,
+    detailed_errors.data_errors,
+    detailed_errors.view_errors,
+    detailed_errors.sequence_errors,
+    detailed_errors.index_errors,
+    detailed_errors.trigger_errors,
+    detailed_errors.constraints_errors,
+    detailed_errors.post_errors,
+    detailed_errors.data_warnings,
+    detailed_errors.retries);
+
+
+}
+
+int main(int argc, char *argv[]) {
+  struct configuration conf = {NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, NULL};
+
+  GError *error = NULL;
+  GOptionContext *context;
+
+  setlocale(LC_ALL, "");
+  g_thread_init(NULL);
+  set_thread_name("MNT");
+
+  signal(SIGCHLD, SIG_IGN);
+
+  context = load_contex_entries();
+
+  gchar ** tmpargv=g_strdupv(argv);
+  int tmpargc=argc;
+  if (!g_option_context_parse(context, &tmpargc, &tmpargv, &error)) {
+    m_critical("option parsing failed: %s, try --help\n", error->message);
   }
 
-  g_free(current_dir);
+  // Loading the defaults file:
+  initialize_common_options(context, "myloader");
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+  g_strfreev(tmpargv);
+
+  if (db == NULL && source_db != NULL) {
+    db = g_strdup(source_db);
+  }
+
+  if (overwrite_unsafe)
+    overwrite_tables= TRUE;
+
+  check_num_threads();
+
+  if (num_threads > max_threads_per_table)
+    g_message("Using %u loader threads (%u per table)", num_threads, max_threads_per_table);
+  else
+    g_message("Using %u loader threads", num_threads);
+
+  initialize_set_names();
+
+  if (debug) {
+    set_debug();
+    verbose=3;
+  }
+
+  if (help){
+    printf("%s", g_option_context_get_help (context, FALSE, NULL));
+  }
+
+  if (program_version) {
+    print_version("myloader");
+    if (!help)
+      exit(EXIT_SUCCESS);
+    printf("\n");
+  }
+
+  if (help)
+    print_help();
+
+  set_verbose(verbose);
+
+  g_message("MyDumper restore version: %s", VERSION);
+
+  // Starts modifying file in disk, creating objects and restore
+
+  hide_password(argc, argv);
+  ask_password();
+
+  initialize_pmm();
+
+  initialize_restore_job();
+  initialize_directories();
+
+  conf.context=context;
+
+  initialize_restore();
+
+  if(stream && !no_stream)
+    create_dir(directory);
+  create_dir(fifo_directory);
+
+  g_message("Using %s as FIFO directory, please remove it if restoration fails", fifo_directory);
+
+  start_pmm_thread((void *)&conf);
+
   g_chdir(directory);
   /* Process list of tables to omit if specified */
   if (tables_skiplist_file)
@@ -436,7 +446,7 @@ int main(int argc, char *argv[]) {
   set_session = g_string_new(NULL);
   set_global = g_string_new(NULL);
   set_global_back = g_string_new(NULL);
-  detect_server_version(conn);
+  server_detect(conn);
   set_session_hash = myloader_initialize_hash_of_session_variables();
   GHashTable * set_global_hash = g_hash_table_new ( g_str_hash, g_str_equal );
   if (key_file != NULL ){
@@ -445,6 +455,8 @@ int main(int argc, char *argv[]) {
   }
 	initialize_conf_per_table(&conf_per_table);
   load_per_table_info_from_key_file(key_file, &conf_per_table, NULL );
+  if (max_transaction_size == DEFAULT_MAX_TRANSACTION_SIZE)
+    detect_group_replication_transaction_size_limit(conn);
 
   // To here.
   conf.database_queue = g_async_queue_new();
@@ -458,7 +470,7 @@ int main(int argc, char *argv[]) {
   conf.ready = g_async_queue_new();
   conf.pause_resume = g_async_queue_new();
   conf.table_list_mutex = g_mutex_new();
-  conf.stream_queue = g_async_queue_new();
+//  conf.stream_queue = g_async_queue_new();
   conf.table_hash = g_hash_table_new ( g_str_hash, g_str_equal );
   conf.table_hash_mutex=g_mutex_new();
 
@@ -499,6 +511,8 @@ int main(int argc, char *argv[]) {
 
   if (!stream){
     wait_directory_to_process_metadata();
+  }else{
+    wait_stream_to_process_metadata_header();
   }
 
   remove_ignore_set_session_from_hash();
@@ -635,10 +649,7 @@ int main(int argc, char *argv[]) {
   g_free(directory);
   free_loader_threads();
 
-  if (pmm){
-    kill_pmm_thread();
-//    g_thread_join(pmmthread);
-  }
+  stop_pmm_thread();
 
 //  g_hash_table_foreach(conf.table_hash,&show_dbt, NULL);
   free_table_hash(conf.table_hash);
