@@ -31,14 +31,11 @@
 gboolean control_job_ended=FALSE;
 gboolean all_jobs_are_enqueued=FALSE;
 gboolean dont_wait_for_schema_create=FALSE;
-/* refresh_db_queue is for data loads */
-GAsyncQueue *refresh_db_queue = NULL, *here_is_your_job=NULL, *data_queue=NULL;
-//GAsyncQueue *give_me_another_job_queue = NULL;
+/* control_job_queue is for data loads */
+GAsyncQueue *control_job_queue = NULL, *data_job_queue=NULL, *data_queue=NULL;
 static GThread *control_job_t = NULL;
 
 gint last_wait=0;
-//guint index_threads_counter = 0;
-//GMutex *index_mutex=NULL;
 void *control_job_thread(struct configuration *conf);
 
 static GMutex *cjt_mutex= NULL;
@@ -54,23 +51,22 @@ void cjt_resume()
 }
 
 void initialize_control_job (struct configuration *conf){
-  refresh_db_queue = g_async_queue_new();
-  here_is_your_job = g_async_queue_new();
+  control_job_queue = g_async_queue_new();
+  data_job_queue = g_async_queue_new();
   last_wait = num_threads;
   data_queue = g_async_queue_new();
   cjt_mutex= g_mutex_new();
   cjt_cond= g_cond_new();
-//  give_me_another_job_queue = g_async_queue_new();
   control_job_t = m_thread_new("myloader_ctr",(GThreadFunc)control_job_thread, conf, "Control job thread could not be created");
-
-//  index_threads_counter = 0;
 }
 
 void wait_control_job()
 {
+  trace("Waiting control job to finish");
   g_thread_join(control_job_t);
   g_mutex_free(cjt_mutex);
   g_cond_free(cjt_cond);
+  trace("Control job to finished");
 }
 
 struct control_job * new_control_job (enum control_job_type type, void *job_data, struct database *use_database) {
@@ -78,8 +74,6 @@ struct control_job * new_control_job (enum control_job_type type, void *job_data
   j->type = type;
   j->use_database=use_database;
   switch (type){
-    case JOB_WAIT:
-      j->data.queue = (GAsyncQueue *)job_data;
     case JOB_SHUTDOWN:
       break;
     default:
@@ -88,113 +82,22 @@ struct control_job * new_control_job (enum control_job_type type, void *job_data
   return j;
 }
 
-gboolean process_job(struct thread_data *td, struct control_job *job, gboolean *retry)
-{
-  switch (job->type) {
-    case JOB_RESTORE: {
-//      g_message("Restore Job");
-      gboolean res= process_restore_job(td, job->data.restore_job);
-      if (retry)
-        *retry= res;
-      return TRUE;
-    }
-    case JOB_WAIT:
-//      g_message("Wait Job");
-      g_async_queue_push(td->conf->ready, GINT_TO_POINTER(1));
-//      GAsyncQueue *queue=job->data.queue;
-      g_async_queue_pop(job->data.queue);
-      break;
-    case JOB_SHUTDOWN: // TODO: do we need JOB_SHUTDOWN for data_queue as it is done in here_is_your_job queue?
-//      g_message("Thread %d: shutting down", td->thread_id);
-      g_free(job);
-      return FALSE;
-      break;
-    default:
-      g_free(job);
-      g_critical("Something very bad happened!(1)");
-      exit(EXIT_FAILURE);
-  }
-  g_free(job);
-  return TRUE;
-}
-void schema_file_missed_lets_continue(struct thread_data * td){
-  g_mutex_lock(td->conf->table_list_mutex);
-  GList * iter=td->conf->table_list;
-  guint i=0;
-  while (iter != NULL){
-    struct db_table * dbt = iter->data;
-    g_mutex_lock(dbt->mutex);
-    dbt->schema_state=CREATED;
-    for(i=0; i<g_list_length(dbt->restore_job_list); i++){
-      g_async_queue_push(td->conf->stream_queue, GINT_TO_POINTER(DATA));
-    }
-    g_mutex_unlock(dbt->mutex);
-    iter=iter->next;
-  }
-  g_mutex_unlock(td->conf->table_list_mutex);
+void control_job_queue_push(enum file_type current_ft){
+  trace("control_job_queue <- %s", ft2str(current_ft));
+  g_async_queue_push(control_job_queue, GINT_TO_POINTER(current_ft));
 }
 
-gboolean are_we_waiting_for_schema_jobs_to_complete(struct thread_data * td){
-  if (g_async_queue_length(td->conf->database_queue) > 0 ||
-      g_async_queue_length(td->conf->table_queue) > 0 ||
-      g_async_queue_length(td->conf->retry_queue) > 0)
-    return TRUE;
-
-  g_mutex_lock(td->conf->table_list_mutex);
-  GList * iter=td->conf->table_list;
-  while (iter != NULL){
-    struct db_table * dbt = iter->data;
-    g_mutex_lock(dbt->mutex);
-    if (dbt->schema_state==CREATING){
-      g_mutex_unlock(dbt->mutex);
-      g_mutex_unlock(td->conf->table_list_mutex);
-      return TRUE;
-    }
-    g_mutex_unlock(dbt->mutex);
-    iter=iter->next;
-  }
-  g_mutex_unlock(td->conf->table_list_mutex);
-  return FALSE;
+enum file_type request_restore_data_job(){
+  control_job_queue_push(REQUEST_DATA_JOB);
+  enum file_type ft=(enum file_type)GPOINTER_TO_INT(g_async_queue_pop(data_job_queue));
+  trace("data_job_queue -> %s", ft2str(ft));
+  return ft;
 }
 
-gboolean are_we_waiting_for_create_schema_jobs_to_complete(struct thread_data * td){
-  if (g_async_queue_length(td->conf->database_queue)>0 )
-    return TRUE;
-
-  g_mutex_lock(td->conf->table_list_mutex);
-  GList * iter=td->conf->table_list;
-  while (iter != NULL){
-    struct db_table * dbt = iter->data;
-    g_mutex_lock(dbt->mutex);
-    if (dbt->schema_state==CREATING){
-      g_mutex_unlock(dbt->mutex);
-      g_mutex_unlock(td->conf->table_list_mutex);
-      return TRUE;
-    }
-    g_mutex_unlock(dbt->mutex);
-    iter=iter->next;
-  }
-  g_mutex_unlock(td->conf->table_list_mutex);
-  return FALSE;
-}
-
-gboolean are_available_jobs(struct thread_data * td){
-  g_mutex_lock(td->conf->table_list_mutex);
-  GList * iter=td->conf->table_list;
-
-  while (iter != NULL){
-    struct db_table * dbt = iter->data;
-    g_mutex_lock(dbt->mutex);
-    if (dbt->schema_state!=CREATED || g_list_length(dbt->restore_job_list) > 0){
-      g_mutex_unlock(dbt->mutex);
-      g_mutex_unlock(td->conf->table_list_mutex);
-      return TRUE;
-    }
-    g_mutex_unlock(dbt->mutex);
-    iter=iter->next;
-  }
-  g_mutex_unlock(td->conf->table_list_mutex);
-  return FALSE;
+struct restore_job * request_next_data_job(){
+  struct restore_job *rj= (struct restore_job *)g_async_queue_pop(data_queue);
+  trace("data_queue -> %s: %s.%s, threads %u", rjtype2str(rj->type), rj->dbt->database->real_database, rj->dbt->real_table, rj->dbt->current_threads);
+  return rj;
 }
 
 gboolean give_me_next_data_job_conf(struct configuration *conf, struct restore_job ** rj){
@@ -256,7 +159,8 @@ gboolean give_me_next_data_job_conf(struct configuration *conf, struct restore_j
       // We found a job that we can process!
       job = dbt->restore_job_list->data;
       GList * current = dbt->restore_job_list;
-      dbt->restore_job_list = dbt->restore_job_list->next;
+//      dbt->restore_job_list = dbt->restore_job_list->next;
+      dbt->restore_job_list = g_list_remove_link(dbt->restore_job_list, current);
       g_list_free_1(current);
       dbt->current_threads++;
       g_mutex_unlock(dbt->mutex);
@@ -269,14 +173,8 @@ gboolean give_me_next_data_job_conf(struct configuration *conf, struct restore_j
       trace("No remaining jobs on %s.%s", dbt->database->real_database, dbt->real_table); 
       if (all_jobs_are_enqueued && dbt->current_threads == 0 && (g_atomic_int_get(&(dbt->remaining_jobs))==0 )){
         dbt->schema_state = DATA_DONE;
-        gboolean res= enqueue_index_for_dbt_if_possible(conf,dbt);
-//          create_index_job(conf, dbt, -1);
-        if (res) {
-          giveup= FALSE;
-          trace("%s.%s queuing indexes, prohibiting finish", dbt->database->real_database, dbt->real_table);
-        } else {
-          trace("%s.%s skipping indexes, voting for finish", dbt->database->real_database, dbt->real_table);
-        }
+        enqueue_index_for_dbt_if_possible(conf,dbt);
+        trace("%s.%s queuing indexes, voting for finish", dbt->database->real_database, dbt->real_table);
       }
     }
     
@@ -288,25 +186,21 @@ gboolean give_me_next_data_job_conf(struct configuration *conf, struct restore_j
   return giveup;
 }
 
-void refresh_db_and_jobs(enum file_type current_ft){
+void enroute_into_the_right_queue_based_on_file_type(enum file_type current_ft){
   switch (current_ft){
     case SCHEMA_CREATE:
     case SCHEMA_TABLE:
     case SCHEMA_SEQUENCE:
-      schema_queue_push(current_ft);
-      break;
-    case DATA:
-      trace("refresh_db_queue <- %s", ft2str(current_ft));
-      g_async_queue_push(refresh_db_queue, GINT_TO_POINTER(current_ft));
+      schema_queue_push(current_ft, "");
       break;
     case INTERMEDIATE_ENDED:
-      schema_queue_push(current_ft);
-      trace("refresh_db_queue <- %s", ft2str(current_ft));
-      g_async_queue_push(refresh_db_queue, GINT_TO_POINTER(current_ft));
+      schema_queue_push(current_ft, "");
+      control_job_queue_push(current_ft);
       break;
+    case REQUEST_DATA_JOB:
+    case DATA:
     case SHUTDOWN:
-      trace("refresh_db_queue <- %s", ft2str(current_ft));
-      g_async_queue_push(refresh_db_queue, GINT_TO_POINTER(current_ft));
+      control_job_queue_push(current_ft);
     default:
       break;
   }
@@ -316,55 +210,23 @@ void maybe_shutdown_control_job()
 {
    if (g_atomic_int_dec_and_test(&last_wait)){
      trace("SHUTDOWN last_wait_control_job_to_shutdown");
-     refresh_db_and_jobs(SHUTDOWN);
+     enroute_into_the_right_queue_based_on_file_type(SHUTDOWN);
    }
 }
-/*
-void wake_threads_waiting(struct configuration *conf, guint *threads_waiting){
-  struct restore_job *rj=NULL;
-  if (*threads_waiting>0){
-    gboolean giveup=FALSE;
-    while ( 0<*threads_waiting && !giveup){
-      giveup = give_me_next_data_job_conf(conf, &rj);
-      if (rj != NULL){
-//          g_message("DATA pushing");
-        *threads_waiting=*threads_waiting - 1;
-        trace("data_queue <- %s: %s", rjtype2str(rj->type), rj->dbt ? rj->dbt->table : rj->filename);
-        g_async_queue_push(data_queue,rj);
-        trace("here_is_your_job <- %s", ft2str(DATA));
-        g_async_queue_push(here_is_your_job, GINT_TO_POINTER(DATA));
-      }
-    }
-    if (giveup) {
-      if (control_job_ended){
-        *threads_waiting=*threads_waiting - 1;
-        trace("refresh_db_queue <- %s", ft2str(THREAD));
-        g_async_queue_push(refresh_db_queue, GINT_TO_POINTER(THREAD));
-      }
-    }
-  }
-}
-*/
+
 void wake_threads_waiting(guint *threads_waiting){
   while(*threads_waiting>0){
-    trace("refresh_db_queue <- %s", ft2str(THREAD));
-    g_async_queue_push(refresh_db_queue, GINT_TO_POINTER(THREAD));
+    control_job_queue_push(REQUEST_DATA_JOB);
     *threads_waiting=*threads_waiting - 1;
   }
 }
 
-
-
 void *control_job_thread(struct configuration *conf){
   enum file_type ft;
   struct restore_job *rj=NULL;
   guint _num_threads = num_threads;
   guint threads_waiting = 0; //num_threads;
-//  GHashTableIter iter;
-//  gchar * lkey=NULL;
   gboolean giveup;
-//  struct database * real_db_name = NULL;
-//  struct control_job *job = NULL;
   gboolean cont=TRUE;
   set_thread_name("CJT");
   if (overwrite_tables && !overwrite_unsafe && cjt_paused) {
@@ -376,23 +238,21 @@ void *control_job_thread(struct configuration *conf){
   }
   trace("Thread control_job_thread started");
   while(cont){
-    ft=(enum file_type)GPOINTER_TO_INT(g_async_queue_timeout_pop(refresh_db_queue,10000000));
-    if (ft==GPOINTER_TO_INT(NULL))
-      ft=THREAD;
-    trace("refresh_db_queue -> %s (%u loaders waiting)", ft2str(ft), threads_waiting);
+    ft=(enum file_type)GPOINTER_TO_INT(g_async_queue_pop(control_job_queue));
+    trace("control_job_queue -> %s (%u loaders waiting)", ft2str(ft), threads_waiting);
     switch (ft){
     case DATA:
       wake_threads_waiting(&threads_waiting);
       break;
-    case THREAD:
+    case REQUEST_DATA_JOB:
 //      g_message("Thread is asking for job");
       giveup = give_me_next_data_job_conf(conf, &rj);
       if (rj != NULL){
         trace("job available in give_me_next_data_job_conf");
         trace("data_queue <- %s: %s", rjtype2str(rj->type), rj->dbt ? rj->dbt->table : rj->filename);
         g_async_queue_push(data_queue,rj);
-        trace("here_is_your_job <- %s", ft2str(DATA));
-        g_async_queue_push(here_is_your_job, GINT_TO_POINTER(DATA));
+        trace("data_job_queue <- %s", ft2str(DATA));
+        g_async_queue_push(data_job_queue, GINT_TO_POINTER(DATA));
       }else{
         trace("No job available");
         if (all_jobs_are_enqueued && giveup){
@@ -400,13 +260,11 @@ void *control_job_thread(struct configuration *conf){
           control_job_ended = TRUE;
           guint i;
           for (i=0;i<num_threads;i++){
-            trace("here_is_your_job <- %s", ft2str(SHUTDOWN));
-            g_async_queue_push(here_is_your_job, GINT_TO_POINTER(SHUTDOWN));
+            trace("data_job_queue <- %s", ft2str(SHUTDOWN));
+            g_async_queue_push(data_job_queue, GINT_TO_POINTER(SHUTDOWN));
           }
-//          trace("refresh_db_queue <- %s", ft2str(THREAD));
-//          g_async_queue_push(refresh_db_queue, GINT_TO_POINTER(THREAD));            
         }else{
-          trace("Thread will be waiting");
+          trace("Thread will be waiting | all_jobs_are_enqueued: %d | giveup: %d", all_jobs_are_enqueued, giveup);
           if (threads_waiting<_num_threads)
               threads_waiting++;
         }
