@@ -31,12 +31,16 @@
 #include "myloader_restore.h"
 
 struct statement * new_statement();
+guint64 max_transaction_size=DEFAULT_MAX_TRANSACTION_SIZE;
 gboolean skip_definer = FALSE;
 GAsyncQueue *connection_pool = NULL;
 GAsyncQueue *restore_queues=NULL;
 GAsyncQueue *free_results_queue=NULL;
 int (*restore_data_from_file) (struct thread_data *, const char *, gboolean , struct database *) = NULL;
 
+GMutex *load_data_list_mutex=NULL;
+GHashTable * load_data_list = NULL;
+
 void *restore_thread(MYSQL *thrconn);
 struct statement release_connection_statement = {0, 0, NULL, NULL, CLOSE, FALSE, NULL, 0, NULL, NULL};
 struct io_restore_result end_restore_thread = { NULL, NULL};
@@ -45,6 +49,11 @@ GThread **restore_threads=NULL;
 
 extern gchar *ignore_errors;
 
+void initialize_restore(){
+  load_data_list_mutex=g_mutex_new();
+  load_data_list = g_hash_table_new ( g_str_hash, g_str_equal );
+}
+
 struct connection_data *new_connection_data(MYSQL *thrconn){
   struct connection_data *cd=g_new(struct connection_data,1);
   if (thrconn)
@@ -225,6 +234,7 @@ int restore_insert(struct connection_data *cd, struct thread_data*td,
   next_line=g_strstr_len(current_line, -1, "\n");
   GString * new_insert=g_string_sized_new(strlen(insert_statement_prefix));
   guint current_rows=0;
+  guint64 transaction_size=0;
   do {
     current_rows=0;
     g_string_set_size(new_insert, 0);
@@ -242,6 +252,11 @@ int restore_insert(struct connection_data *cd, struct thread_data*td,
       current_offset_line++;
     } while ((rows == 0 || current_rows < rows) && next_line != NULL);
     if (current_rows > 1 || (current_rows==1 && line_len>0) ){
+      if (cd->transaction && ((max_transaction_size * 1024 * 1024 < new_insert->len + transaction_size) )){ //|| (max_transaction_size * 1024 * 1024 < transaction_size + max_statement_size ))){
+        tr+=m_commit_and_start_transaction(cd,query_counter);
+        transaction_size=0;
+      }
+      transaction_size+=new_insert->len;
       tr=restore_data_in_gstring_by_statement(cd, new_insert, FALSE, query_counter);
       g_usleep(throttle_time);
       g_mutex_lock(dbt->mutex);
@@ -249,6 +264,7 @@ int restore_insert(struct connection_data *cd, struct thread_data*td,
       g_mutex_unlock(dbt->mutex);
       if (cd->transaction && *query_counter == commit_count) {
         tr+=m_commit_and_start_transaction(cd,query_counter);
+        transaction_size=0;
       }
 
       if (tr > 0){
@@ -256,6 +272,7 @@ int restore_insert(struct connection_data *cd, struct thread_data*td,
       }
       if (mysql_warning_count(cd->thrconn)){
         g_warning("Connection %ld: Warnings found during INSERT between lines: %d and %d: %s",cd->connection_id, offset_line,current_offset_line, show_warnings_if_possible(cd->thrconn));
+        detailed_errors.data_warnings+=mysql_warning_count(cd->thrconn);
       }
     }else
       tr=0;
@@ -597,11 +614,14 @@ int restore_data_from_mydumper_file(struct thread_data *td, const char *filename
             gchar *from_equal=g_strstr_len(data->str, strlen(data->str),"=");
             if (from_equal && ignore_set_list ){
               *from_equal='\0';
-              if (!is_in_ignore_set_list(data->str)) {
+              gchar * var_name=g_strrstr(data->str," ");
+              var_name++;
+              if (!is_in_ignore_set_list(var_name)) {
                 *from_equal='=';
                 g_string_append(header,data->str);
               }else{
                 *from_equal='=';
+                goto STMT_IGNORED;
               }
             }else{
               g_string_append(header,data->str);
@@ -614,13 +634,14 @@ int restore_data_from_mydumper_file(struct thread_data *td, const char *filename
           ir=NULL;
           process_result_statement(cd->queue->result, &ir, m_critical, "(2)Error occurs processing file %s", filename);
         }
-        r|= ir->result;
 
+        r|= ir->result;
+        if (ir->result>0)
+          g_critical("(1)Error occurs processing file %s",filename);
+
+STMT_IGNORED:
         g_string_set_size(data, 0);
         preline=line+1;
-        if (ir->result>0){
-          g_critical("(1)Error occurs processing file %s",filename);
-        }
       }
     } else {
       g_critical("error reading file %s (%d)", filename, errno);
