@@ -17,3 +17,5 @@
 #include "myloader.h"
 void initialize_stream (struct configuration *conf);
 void wait_stream_to_finish();
+void wait_stream_to_process_metadata_header();
+void metadata_has_been_processed();
