@@ -20,3 +20,5 @@ void initialize_worker_index(struct configuration *conf);
 void wait_index_worker_to_finish();
 void create_index_shutdown_job(struct configuration *conf);
 void start_optimize_keys_all_tables();
+void enqueue_indexes_if_possible(struct configuration *conf);
+void enqueue_index_for_dbt_if_possible(struct configuration *conf, struct db_table * dbt);
