@@ -26,9 +26,10 @@ enum server_type {
   SERVER_TYPE_MARIADB,
   SERVER_TYPE_PERCONA,
   SERVER_TYPE_CLICKHOUSE,
+  SERVER_TYPE_RDS,
   SERVER_TYPE_DOLT
 };
-void detect_server_version(MYSQL * conn);
+void server_detect(MYSQL * conn);
 gboolean server_support_tablespaces();
 int get_product();
 int get_major();
